import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * This class displays a panel with three text boxes, for the name,
 * rank and size of a matroid represented over a finite field, and a
 * button.  When the user presses the button, the class validates the
 * input as correct specifications for a new FiniteFieldMatroid and
 * then raises an event for its listeners, who can then get its values
 * via getRank, getName and getSize.
 * 
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 4 June 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class FiniteFieldMatroidSpecPanel extends JPanel 
  implements ActionListener
{
  //
  // private member data
  //
  private JTextField mFieldTextField;
  private JTextField mRankTextField;
  private JTextField mSizeTextField;
  private JTextField mNameTextField;
  private JButton mButton;
  private JLabel mMessageLabel;
  private FiniteField mField;
  private int mSize;
  private int mRank;
  private String mName;
  private boolean mHasValidInput;
  private LinkedList mListeners;

  private static String FIELD_TEXT = "Field size: ";
  private static String RANK_TEXT = "Rank: ";
  private static String SIZE_TEXT = "Size: ";
  private static String NAME_TEXT = "Name: ";
  private static String PROMPT = 
    "Enter the name, field size, rank and size below:    ";

  /**
   * constructs a panel with labels and text boxes for parameter input
   * and a continue button
   */
  FiniteFieldMatroidSpecPanel()
  {
    JPanel rowPanel;
    JPanel colPanel;

    setLayout(new BorderLayout());

    colPanel = new JPanel();
    colPanel.setLayout(new BoxLayout(colPanel, BoxLayout.Y_AXIS));

    //
    // add name label & text box
    //
    rowPanel = new JPanel();
    rowPanel.setLayout(new BoxLayout(rowPanel, BoxLayout.X_AXIS));
    rowPanel.add(new JLabel(NAME_TEXT));
    mNameTextField = new JTextField(5);
    rowPanel.add(mNameTextField);
    colPanel.add(rowPanel);

    //
    // add field size label & text box
    //
    rowPanel = new JPanel();
    rowPanel.setLayout(new BoxLayout(rowPanel, BoxLayout.X_AXIS));
    rowPanel.add(new JLabel(FIELD_TEXT));
    mFieldTextField = new JTextField(5);
    rowPanel.add(mFieldTextField);
    colPanel.add(rowPanel);

    //
    // add size label & text box
    //
    rowPanel = new JPanel();
    rowPanel.setLayout(new BoxLayout(rowPanel, BoxLayout.X_AXIS));
    rowPanel.add(new JLabel(SIZE_TEXT));
    mSizeTextField = new JTextField(5);
    rowPanel.add(mSizeTextField);
    colPanel.add(rowPanel);

    //
    // add rank label & text box
    //
    rowPanel = new JPanel();
    rowPanel.setLayout(new BoxLayout(rowPanel, BoxLayout.X_AXIS));
    rowPanel.add(new JLabel(RANK_TEXT));
    mRankTextField = new JTextField(5);
    rowPanel.add(mRankTextField);
    colPanel.add(rowPanel);

    add(colPanel, BorderLayout.CENTER);
    //
    // add continue button
    //
    mButton = new JButton("Continue");
    mButton.addActionListener(this);

    //
    // add message label
    //
    mMessageLabel = new JLabel(PROMPT);
    
    add(mMessageLabel, BorderLayout.NORTH);

    add(mButton, BorderLayout.SOUTH);
  }

  /** 
   * sets the default value for rank of matroid (used when specifying a linear
   * space by matrix)
   *
   * @param defaultRank default value for rank to display
   */
  public void setDefaultRank(int defaultRank)
  {
    mRankTextField.setText(String.valueOf(defaultRank));
  }

  /** 
   * adds an <code>ActionListener</code>
   *
   * @param l reference to listener
   */
  public void addActionListener(ActionListener l)
  {
    if (!(mListeners instanceof LinkedList)) {
      mListeners = new LinkedList();
    }

    mListeners.add(l);
  }

  //
  // listen to button
  //
  public void actionPerformed(ActionEvent e)
  {
    ActionListener l;
    ActionEvent f;

    if (inputIsValid()) {
      mMessageLabel.setText(PROMPT);
      if (mListeners instanceof LinkedList) {
	for (Iterator i=mListeners.iterator(); i.hasNext();) {
	  l = (ActionListener)i.next();
	  l.actionPerformed(new ActionEvent(this,
					    ActionEvent.ACTION_PERFORMED,
					    "Specs selected"));
	}
      }
      else {
	mMessageLabel.setText("Okay!");
      }
    }
  }

  /** 
   * validates input and returns a boolean value indicating input status
   *
   * @return <code>true</code> if and only if input is valid specification
   * for a matroid over a finite field
   */
  public boolean inputIsValid()
  {
    mHasValidInput = (retrieveName() && 
		      retrieveField() &&
		      retrieveRank() && 
		      retrieveSize());
    return mHasValidInput;
  }

  /** 
   * returns the size entered for the new matroid 
   *
   * @return size of matroid
   */
  public int getMatroidSize()
  {
    int nAnswer = -1;
    if (mHasValidInput) {
      nAnswer = mSize;
    }
    return nAnswer;
  }

  /**
   * returns the finite field selected for the matroid
   *
   * @return <code>FiniteField</code> reference
   */
  public FiniteField getField()
  {
    return mField;
  }

  /**
   * returns the rank selected for the new matroid
   *
   * @return rank of matroid
   */
  public int getMatroidRank()
  {
    int nAnswer = -1;
    if (mHasValidInput) {
      nAnswer = mRank;
    }
    return nAnswer;
  }

  /** 
   * returns the name entered for the new matroid
   *
   * @return string with new name
   */
  public String getMatroidName()
  {
    String answer = (String)null;
    if (mHasValidInput) {
      answer = new String(mName);
    }
    return answer;
  }

  //
  // make sure the name string is not empty
  //
  private boolean retrieveName()
  {
    boolean bAns = false;
    if (mNameTextField.getText().length() > 0) {
      mName = mNameTextField.getText();
      bAns = true;
    }
    return bAns;
  }

  //
  // make sure the field size entered is valid
  //
  private boolean retrieveField()
  {
    boolean bAns = true;
    int q = 0;

    try {
      q = Integer.parseInt(mFieldTextField.getText());
      mField = FiniteFieldFactory.createFiniteField(q);
    }
    catch (NumberFormatException nfe) {
      mMessageLabel.setText("Field size must be a number");
      bAns = false;
    }
    catch (IllegalArgumentException iee) {
      mMessageLabel.setText("Field size must be a prime power");
      bAns = false;
    }
    return bAns;
  }

  // 
  // get the rank off the form and make sure it is positive
  //
  private boolean retrieveRank()
  {
    boolean bAnswer = true;
    int nValue=0;
    String theText;

    theText = mRankTextField.getText();
    if (theText.equals("")) {
      mMessageLabel.setText("No rank entered");
      bAnswer = false;
    }
    else {
      try {
	nValue = Integer.parseInt(theText);
      }
      catch (NumberFormatException e) {
	mMessageLabel.setText("Rank is not a number");
	bAnswer = false;
      }

      if (bAnswer) {
	if (nValue > 0) {
	  mRank = nValue;
	}
	else {
	  mMessageLabel.setText("Rank is not positive");
	  bAnswer = false;
	}
      }
    }

    return bAnswer;
  }

  //
  // get the size off the form and make sure it is positive
  //
  private boolean retrieveSize()
  {
    boolean bAnswer = true;
    int nValue=0;
    String theText;

    theText = mSizeTextField.getText();
    if (theText.equals("")) {
      mMessageLabel.setText("No size entered");
      bAnswer = false;
    }
    else {
      try {
	nValue = Integer.parseInt(theText);
      }
      catch (NumberFormatException e) {
	mMessageLabel.setText("Size is not a number");
	bAnswer = false;
      }
      if (bAnswer) {
	if (nValue > 0) {
	  mSize = nValue;
	}
	else {
	  mMessageLabel.setText("Size is not positive");
	  bAnswer = false;
	}
      }
    }

    return bAnswer;
  }

  /**
   * main method allows panel to be tested individually from the command line
   *
   * @param arg array of command line arguments (ignored)
   */
  public static void main(String[] arg)
  {
    JFrame parent = new JFrame("FiniteFieldMatroidSpecPanel");
    Container contentPane = parent.getContentPane();
    FiniteFieldMatroidSpecPanel specPanel = new FiniteFieldMatroidSpecPanel();

    specPanel.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  FiniteFieldMatroidSpecPanel sp = 
	    (FiniteFieldMatroidSpecPanel)e.getSource();
	  System.out.println("Matroid name: " + sp.getMatroidName());
	  System.out.println("Field size:   " + sp.getField().getFieldSize());
	  System.out.println("Matroid size: " + sp.getMatroidSize());
	  System.out.println("Matroid rank: " + sp.getMatroidRank());
	  System.exit(0);
	}
      });

    parent.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  System.exit(0);
	}
      });

    contentPane.add(specPanel);
    parent.pack();
    parent.setVisible(true);
  }

}
