import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * This interface describes the functions needed to provide a graphical
 * user interface for a Toolkit.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 14 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public interface GUI
{
  /**
   * Allows the client to programatically select an object from the list
   * of open objects
   *
   * @param openObject reference to object to select
   */
  public void selectOpenObject(Object openObject);

  /**
   * Returns a reference to the currently selected object
   *
   * @return reference to selected object
   */
  public Object getSelectedOpenObject();

  /**
   * Provides the UI with a list of classes which can currently be created.
   *
   * @param creatableClasses Vector of creatable Classes
   */
  public void setCreatableClassList(Vector creatableClasses);

  /**
   * Provides the UI with a list of the names associated with the classes
   * which can currently be created--vector must be the same size as the
   * creatable class list vector.
   *
   * @param creatableClassNames Vector of Strings with names
   * @throws IllegalArgumentException if <code>creatableClassNames</code>
   * is invalid
   */
  public void setCreatableClassNames(Vector creatableClassNames)
  throws IllegalArgumentException;

  /**
   * Provides the UI with the contents of its Task menu.
   *
   * @param toolMenu JMenu containing Task menu for UI
   */
  public void setTaskMenu(JMenu toolMenu);

  /**
   * Provides the UI with the contents of its View menu.
   *
   * @param viewMenu JMenu containing View menu for the UI
   */
  public void setViewMenu(JMenu viewMenu);

  /**
   * Adds an action listener to the UI.
   *
   * @param a The ActionListener to be added.
   */
  public void addActionListener(ActionListener a);

  /**
   * Displays the passed JPanel, indicates the parent object whose name should
   * be displayed on the list of open objects, and returns a handle for 
   * reference to the displayed panel.
   *
   * @param newPanel The panel to be displayed
   * @param caption Caption for the new panel's window
   * @param parent Parent object (name must show on open objects list)
   * @return reference number for new panel
   */
  public int displayPanel(JPanel newPanel, String caption, Object parent);

  /** 
   * Removes the displayed panel with the specified handle from the display
   *
   * @param thePanel the panel to remove
   */
  public void removePanel(JPanel thePanel);

  /** 
   * Determines whether the panel with the specified handle should be
   * visible.
   *
   * @param handle The handle of the panel to query
   * @param isVisible Indicates whether the panel should be visible
   */
  public void setPanelVisible(JPanel thePanel, boolean isVisible);

  /**
   * Indicates whether the panel with the specified handle is currently
   * visible.
   *
   * @return boolean answer
   */
  public boolean getPanelVisible(JPanel thePanel);

  /**
   * Displays a dialog box with the passed message
   *
   * @param message message to display
   */
  public void showMessageDialog(Object message);

  /**
   * Displays a warning dialog box with the passed message
   *
   * @param message message to display
   */
  public void showWarningDialog(Object message);

  /**
   * Displays an error dialog box with the passed message
   *
   * @param message message to display
   */
  public void showErrorDialog(Object message);

  /**
   * Displays an input dialog box with the passed message and 
   * returns the string the user entered.
   *
   * @param message message to display
   */
  public String showInputDialog(Object message);

  /**
   * Adds a message to the GUI's status display
   *
   * @param message message to display
   */
  public void addMessage(String message);
}
