import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * A <code>MultiSelectPanel</code> displays a list of items and two
 * buttons, Continue and Cancel.  The list allows multiple selections.
 * When either of the buttons is pressed, the object sends an
 * <code>ActionPerformed</code> message.  If the Cancel button was
 * pressed, method <code>wasCanceled</code> returns true.
 * <code>getSelectedItems</code> returns an array of the selected
 * items from the list otherwise.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 6 Jun 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class MultiSelectPanel extends JPanel implements ActionListener
{
  //
  // private data members
  // 
  private JList mList;
  private ListModel mModel;
  private JButton mContinue;
  private JButton mCancel;
  private boolean mWasCanceled;
  private Vector mListeners;
  private String mClient;

  /**
   * The constructor accepts a String containing a prompt and a Vector
   * of the items to display on the list
   *
   * @param prompt String with prompt for panel
   * @param items Vector of items to display
   * @param client name of this dialog's client
   */
  public MultiSelectPanel(String prompt, Vector items, String client) 
  {
    mListeners = new Vector();
    setLayout(new BorderLayout());

    mClient = new String(client);

    JLabel promptLabel = new JLabel(prompt);
    add(promptLabel, BorderLayout.NORTH);

    mList = new JList(items);
    mList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    add(mList, BorderLayout.CENTER);

    JPanel southPanel = new JPanel();
    southPanel.setLayout(new BoxLayout(southPanel, BoxLayout.X_AXIS));

    mContinue = new JButton("Continue");
    mContinue.addActionListener(this);
    
    mCancel = new JButton("Cancel");
    mCancel.addActionListener(this);

    southPanel.add(mContinue);
    southPanel.add(mCancel);

    add(southPanel, BorderLayout.SOUTH);
  }

  /**
   * returns client name
   * 
   * @return client name
   */
  public String getClient()
  {
    return mClient;
  }

  /**
   * returns value indicating whether the user pressed the Cancel button
   *
   * @return <code>true</code> if and only if the user pressed Cancel
   */
  public boolean wasCanceled()
  {
    return mWasCanceled;
  }

  /**
   * returns an array of the selected values from the list
   *
   * @return array of <code>Object</code>s
   */
  public Object[] getSelectedValues()
  {
    return mList.getSelectedValues();
  }

  /**
   * adds an action listener to this panel
   * 
   * @param listener reference to listener to add
   */
  public void addActionListener(ActionListener listener)
  {
    mListeners.add(listener);
  }

  /**
   * removes an action listener from this panel
   *
   * @param listener reference to listener to remove
   */
  public void removeActionListener(ActionListener listener)
  {
    mListeners.remove(listener);
  }

  /**
   * responds to button clicks by sending a message on to this object's
   * listeners
   *
   * @param event <code>ActionEvent</code> from button
   */
  public void actionPerformed(ActionEvent event)
  {
    mWasCanceled = (event.getSource() == mCancel);

    ActionEvent myEvent = new ActionEvent(this,
					  ActionEvent.ACTION_PERFORMED,
					  new String("button clicked"));

    for (Iterator pListener = mListeners.iterator(); pListener.hasNext(); ) {
      ((ActionListener)pListener.next()).actionPerformed(myEvent);
    }
  }

  /**
   * allows this class to be tested from the command line
   *
   * @param arg array of command line arguments, used to fill list for test
   */
  public static void main(String[] arg)
  {
    JFrame frame = new JFrame("MultiSelectPanel");
    MultiSelectPanel panel;
    Vector items = new Vector();
    int i;

    if (arg.length == 0) {
      System.out.println("Usage: java MultiSelectPanel <item1> <item2> ...");
    }
    else {
      for (i=0; i<arg.length; i++) {
	items.add(arg[i]);
      }
      
      panel = new MultiSelectPanel("MultiSelectPanel test", items, "client");

      frame.addWindowListener(new WindowAdapter() {
	  public void windowClosing(WindowEvent e) {
	    System.exit(0);
	  }
	});

      panel.addActionListener(new ActionListener() {
	  public void actionPerformed(ActionEvent f) {
	    MultiSelectPanel msp = (MultiSelectPanel)f.getSource();

	    if (msp.wasCanceled()) {
	      System.out.println("Form was canceled");
	    }
	    else {
	      Object[] sel = msp.getSelectedValues();
	      System.out.println("Selected values:");
	      for (int i = 0; i<sel.length; i++) {
		System.out.println("  " + sel[i].toString());
	      }
	    }

	    System.exit(0);
	  }
	});

      frame.getContentPane().add(panel);
      frame.pack();
      frame.setVisible(true);
    }
  }
}
