import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * A <code>NewLinearSpacePanel</code> displays three input areas: one
 * for the user to specify the name of a new <code>LinearSpace</code>
 * to create, one to specify the size of the new matroid, and the
 * third for the user to identify its bases. It also displays a
 * Continue button.  When the user presses the Continue button, the
 * object attempts to create a new <code>LinearSpace</code> instance
 * using the input values.  The panel can then provide a reference to
 * the resulting <code>LinearSpace</code> instance to its client.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 19 April 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class NewLinearSpacePanel extends JPanel implements ActionListener
{
  //
  // private data members
  //
  JTextField   mNameField;
  JTextField   mSizeField;
  JTextField   mRankField;
  JTextArea    mTextArea;
  JLabel       mMessageLabel;
  LinearSpace  mLinearSpace;
  String       mLinearSpaceName;
  int          mLinearSpaceSize;
  int          mLinearSpaceRank;
  Vector       mSubsets;
  Vector       mActionListeners;
  int          mBadSubsetLine;

  /**
   * sets up the panel
   */ 
  public NewLinearSpacePanel()
  {
    mLinearSpaceSize = 0;
    mLinearSpace = (LinearSpace)null;
    setLayout(new BorderLayout());
    
    // 
    // set up panel with size & name fields in north area
    //
    JPanel colPanel = new JPanel();
    colPanel.setLayout(new BoxLayout(colPanel, BoxLayout.Y_AXIS));
    JPanel rowPanel;

    // name label & field
    rowPanel = new JPanel();
    rowPanel.setLayout(new BoxLayout(rowPanel, BoxLayout.X_AXIS));
    mNameField = new JTextField();
    rowPanel.add(new JLabel("Name:"));
    rowPanel.add(mNameField);
    colPanel.add(rowPanel);

    // size label & field
    rowPanel = new JPanel();
    rowPanel.setLayout(new BoxLayout(rowPanel, BoxLayout.X_AXIS));
    mSizeField = new JTextField();
    rowPanel.add(new JLabel("Size:"));
    rowPanel.add(mSizeField);
    colPanel.add(rowPanel);

    // rank label & field
    rowPanel = new JPanel();
    rowPanel.setLayout(new BoxLayout(rowPanel, BoxLayout.X_AXIS));
    mRankField = new JTextField();
    mRankField.setText("3");
    rowPanel.add(new JLabel("Rank:"));
    rowPanel.add(mRankField);
    colPanel.add(rowPanel);
    
    add(colPanel, BorderLayout.NORTH);

    // set up text area for subsets in center area
    mTextArea = new JTextArea(10, 20);
    JScrollPane scrollPane = 
      new JScrollPane(mTextArea,
		      JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
		      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    add(scrollPane, BorderLayout.CENTER);
   
    // set up continue button and message label in south area
    colPanel = new JPanel();
    colPanel.setLayout(new BoxLayout(colPanel, BoxLayout.Y_AXIS));
    mMessageLabel = new JLabel("Press continue when finished");
    colPanel.add(mMessageLabel);
    JButton continueButton = new JButton("Continue");
    continueButton.addActionListener(this);
    colPanel.add(continueButton);
    add(colPanel, BorderLayout.SOUTH);
  }

  /**
   * returns the <code>LinearSpace</code> entered by the user
   *
   * @return reference to space
   */
  public LinearSpace getLinearSpace()
  {
    return mLinearSpace;
  }

  /**
   * adds an action listener to this panel
   *
   * @param al <code>ActionListener</code> reference
   */
  public void addActionListener(ActionListener al)
  {
    if (!(mActionListeners instanceof Vector)) {
      mActionListeners = new Vector();
    }
    mActionListeners.add(al);
  }

  /**
   * responds to actions on panel controls
   *
   * @param e <code>ActionEvent</code> sent by control
   */
  public void actionPerformed(ActionEvent e) 
  {
    boolean ready = true;
    ready = ready && validateName();
    if (!ready) {
      mMessageLabel.setText("You must enter a name");
    }

    if (ready) {
      ready = ready && validateSize();
      if (!ready) {
	mMessageLabel.setText("Incorrect size entered");
      }
    }

    if (ready) {
      ready = ready && validateRank();
      if (!ready) {
	mMessageLabel.setText("The rank must be 1, 2 or 3");
      }
    }

    if (ready) {
      ready = ready && validateSubsets();
      if (!ready) {
	mMessageLabel.setText("Incorrect subset in line " + mBadSubsetLine);
      }
    }

    
    if (ready) {
      try {
	mLinearSpace = new LinearSpace(mLinearSpaceSize,
				       mLinearSpaceRank,
				       mSubsets);
	mLinearSpace.setName(mLinearSpaceName);
      }
      catch (Exception g) {
	mTextArea.setText("Error:" + g.toString() + '\n' + 
			  mTextArea.getText());
	mMessageLabel.setText("Unable to initialize LinearSpace:"
			      + " see text above");
	ready = false;
      }
    }

    if (ready) {
      ActionListener listener;
      Iterator pListener;
      ActionEvent f = new ActionEvent(this, 
				      ActionEvent.ACTION_PERFORMED,
				      "new LinearSpace");
      for (pListener = mActionListeners.iterator(); pListener.hasNext(); ) {
	listener = (ActionListener)pListener.next();
	listener.actionPerformed(f);
      }
    }
  }

  //
  // private method validates name of matroid
  //
  private boolean validateName()
  {
    mLinearSpaceName = mNameField.getText().trim();
    return (!(mLinearSpaceName.equals("")));
  }

  //
  // private method validates size of matroid
  //
  private boolean validateSize()
  {
    String s = mSizeField.getText();
    try {
      mLinearSpaceSize = Integer.valueOf(s).intValue();
    }
    catch (Exception e) {
      mLinearSpaceSize = 0;
    }
    return (mLinearSpaceSize > 0);
  }

  //
  // private method validates rank of matroid
  //
  private boolean validateRank()
  {
    String s = mRankField.getText();
    try {
      mLinearSpaceRank = Integer.valueOf(s).intValue();
    }
    catch (Exception e) {
      mLinearSpaceRank = 0;
    }
    return ((mLinearSpaceRank > 0) && (mLinearSpaceRank < 4));
  }

  //
  // private method validates subsets of matroid -- size must be validated 1st
  //
  private boolean validateSubsets()
  {
    mSubsets = new Vector();
    int lNum = 1;
    boolean answer = false;
    answer = (mLinearSpaceSize > 0);
    if (answer) {
      String text = mTextArea.getText();
      String oneLine;
      StringTokenizer st = new StringTokenizer(text, "\n");
      for (; st.hasMoreElements(); ) {
	oneLine = (String)st.nextElement();
	if (!(getSubset(oneLine))) {
	  mBadSubsetLine = lNum;
	  answer = false;
	  break;
	}
	lNum++;
      }
    }

    return answer;
  }

  // 
  // private method tries to get a subset out of the entries in the passed
  // string, and returns a boolean value indicating whether it worked--if
  // it did, the subset is added to mSubsets
  //
  private boolean getSubset(String s)
  {
    boolean answer = true;
    Subset ss;
    int i;
    ss = new Subset(mLinearSpaceSize);

    for (StringTokenizer st = new StringTokenizer(s); st.hasMoreElements(); ) {
      i = Integer.valueOf((String)st.nextElement()).intValue();
      if ((i > 0) && (i <= mLinearSpaceSize)) {
	ss.set(i-1);
      }
      else {
	answer = false;
	break;
      }
    }
    if (answer) {
      mSubsets.add(ss);
    }
    return answer;
  }

  /**
   * allows this class to be tested from the command-line
   *
   * @param arg array of command-line arguments; ignored
   */
  public static void main(String[] arg)
  {
    JFrame frame = new JFrame("NewLinearSpacePanel");
    NewLinearSpacePanel panel;

    panel = new NewLinearSpacePanel();

    panel.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  NewLinearSpacePanel p = (NewLinearSpacePanel)e.getSource();
	  LinearSpace ls = p.getLinearSpace();
	  
	  System.out.println("Name: " + ls.getName());
	  System.out.println("Rank: " + ls.getRank());
	  System.out.println("Size: " + ls.getSize());
	  System.out.println("Lines:");
	  
	  Vector lines = ls.getLineVector();
	  for (Iterator pLine = lines.iterator(); pLine.hasNext(); ) {
	    System.out.println(((Subset)pLine.next()).toString(true));
	  }

	  System.exit(0);
	}
      });

    frame.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  System.exit(0);
	}
      });

    frame.getContentPane().add(panel);
    frame.pack();
    frame.setVisible(true);    
  }
}
