package Oid;
import java.util.*;
import java.math.*;

/**
 * The BasisMatroidBasisAlgorithm class, which implements the SubsetLister 
 * interface for use in the Matroid framework, simply retrieves the list of 
 * bases from a passed BasisMatroid.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 9 Oct 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class BasisMatroidBasisAlgorithm implements SubsetLister,
						   ComplexityProvider
{   
  //
  // private data members
  //
  private BasisMatroid mMatroid;
  private Vector mBases;

  /** 
   * sets the reference object for this SubsetLister 
   *
   * @param arg reference object
   */
  public void setInstance(Object arg)
  throws IllegalArgumentException
  {
    if (!(arg instanceof BasisMatroid)) {
      throw new IllegalArgumentException("Non-BasisMatroid passed to " + 
					 "BasisMatroidBasisAlgorithm");
    }
    mMatroid = (BasisMatroid)arg;
    determineBases();
  }

  /** 
   * returns the reference object for this SubsetLister 
   *
   * @return reference object
   */
  public Object getInstance()
  {
    return mMatroid;
  }

  /** 
   * generates a list of the relevant Subsets for the reference object 
   *
   * @return subset list
   * @throws NullPointerException if no reference object has been set
   */
  public Collection getSubsetList()
  throws NullPointerException
  {
    return mBases;
  }

  /** 
   * generates a Vector containing indexed collections of Subsets for 
   * the reference object -- trivial for this algorithm
   *
   * @return subset list vector
   * @throws NullPointerException if no reference object has been set
   */
  public Vector getSubsetListVector()
  throws NullPointerException
  {
    Vector dummy = new Vector();
    dummy.add(mBases);
    return dummy;
  }

  /** 
   * generates a Vector containing the number of Subsets at each index
   * for the reference object -- trivial for this algorithm
   *
   * @return subset count chain 
   * @throws NullPointerException if no reference object has been set
   */
  public Vector getSubsetCountChain()
  throws NullPointerException
  {
    Vector dummy = new Vector();
    dummy.add(new Integer(mBases.size()));
    return dummy;
  }

  /**
   * returns the name of the property by which the subsets are indexed
   *
   * @return name of array index property
   * @throws NullPointerException if no reference object has been set
   */
  public String getArrayIndexName()
  throws NullPointerException
  {
    return new String("(no ranking)");
  }

  /** 
   * returns a description of the algorithm used to generate subsets 
   *
   * @return description of algorithm
   */
  public String getAlgorithmDescription()
  {
    return new String("This algorithm simply retrieves the bases from a" +
		      " BasisMatroid");
  }

  /** 
   * returns the name of this algorithm 
   * 
   * @return name of algorithm
   */
  public String getName()
  {
    return new String("BasisMatroid basis algorithm");
  }

  // private method determines bases of passed matroid
  private void determineBases()
  {
    mBases = mMatroid.getBases();
  }
  //
  // ComplexityProvider interface methods
  //
  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name, or -1 if the method cannot
   * currently be executed
   *
   * @param methodName name of method to check
   * @return complexity estimate
   */
  public long getComplexityEstimate(String methodName)
  {
    long answer = -1;

    if ((mMatroid instanceof BasisMatroid) && 
	(mMatroid instanceof ComplexityProvider) &&
	(methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain"))) {
      answer = 1;
    }

    return answer;
  }
  
  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   *
   * @param methodName name of method to check
   * @param argument method argument
   * @return complexity estimate
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    long answer = -1;

    if ((argument instanceof BasisMatroid) && 
	(argument instanceof ComplexityProvider) &&
	(methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain"))) {
      answer = 1;
    }

    return answer;
  }

  /**
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name and collection of arguments
   * (which must be placed in correct order for iteration), or -1 if the
   * method cannot be executed with the passed arguments
   *
   * @param methodName name of method to check
   * @param arguments method arguments
   * @return complexity estimate
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    long answer = -1;

    if (arguments.size() > 0) {
      Object argument = arguments.iterator().next();
      if ((argument instanceof BasisMatroid) && 
	  (argument instanceof ComplexityProvider) &&
	  (methodName.equals("getSubsetList") ||
	   methodName.equals("getSubsetListVector") ||
	   methodName.equals("getSubsetCountChain"))) {
	answer = 1;
      }
    }
    return answer;
  }
}
