package Oid;
import java.util.*;
import java.math.*;

/**
 * This class implements the SubsetLister interface with an algorithm 
 * for identifying the hyperplanes of a matroid that will work for 
 * BasisMatroids.  It determines the dual of the matroid it is given,
 * uses an instance of DefaultCircuitAlgorithm to determine its circuits,
 * and then takes their complements to obtain the hyperplanes of the 
 * original matroid.
 *
 * @author  Robert J. Kingan and Sandra R. Kingan
 * @date 14 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class BasisMatroidHyperplaneAlgorithm implements SubsetLister,
							ComplexityProvider
{
  //
  // private data members
  //
  private BasisMatroid mMatroid;
  private Vector       mHyperplanes;
  private Vector       mChain;
  private DefaultCircuitAlgorithm mCCTAlg;

  /**
   * default constructor
   */
  public BasisMatroidHyperplaneAlgorithm()
  {
    mCCTAlg = new DefaultCircuitAlgorithm();
  }

  //
  // SubsetLister interface methods
  //

  /** 
   * sets the reference object for this SubsetLister 
   *
   * @param arg reference object
   */
  public void setInstance(Object arg)
    throws IllegalArgumentException
  {
    boolean isValid = true;
    String invalidReason = new String("");

    if (!(arg instanceof BasisMatroid)) {
      isValid = false;
      invalidReason = new String("This algorithm requires a BasisMatroid");
    }
    else if (!(arg instanceof ComplexityProvider)) {
      isValid = false;
      invalidReason = new String("Unable to determine whether dual exists");
    }
    else if (((ComplexityProvider)arg).getComplexityEstimate("getDual") < 0) {
      isValid = false;
      invalidReason = new String("Cannot produce matroid dual");
    }

    if (!isValid) {
      throw new IllegalArgumentException(invalidReason);
    }

    mMatroid = (BasisMatroid)arg;
    determineHyperplanes();
    determineChain();
  }

  /** 
   * returns the reference object for this SubsetLister 
   * 
   * @return reference object
   */
  public Object getInstance()
  {
    return mMatroid;
  }

  /** 
   * generates a list of the relevant Subsets for the reference object 
   *
   * @return subset list
   * @throws NullPointerException if no reference object has been set
   */
  public Collection getSubsetList()
    throws NullPointerException
  {
    if (!(mHyperplanes instanceof Vector)) {
      throw new NullPointerException("No matroid provided to algorithm");
    }
    
    Vector oneVector = new Vector();
    for (Iterator pLevel = mHyperplanes.iterator(); pLevel.hasNext(); ) {
      oneVector.addAll((Vector)pLevel.next());
    }

    return oneVector;
  }

  /** 
   * generates a Vector containing indexed collections of Subsets for 
   * the reference object
   *
   * @return subset list vector
   * @throws NullPointerException if no reference object has been set
   */
  public Vector getSubsetListVector()
    throws NullPointerException
  {
    if (!(mHyperplanes instanceof Vector)) {
      throw new NullPointerException("No matroid provided to algorithm");
    }
    return mHyperplanes;
  }

  /** 
   * generates a Vector containing the number of Subsets at each index
   * for the reference object
   *
   * @return subset count chain
   * @throws NullPointerException if no reference object has been set
   */
  public Vector getSubsetCountChain()
    throws NullPointerException 
  {
    if (!(mHyperplanes instanceof Vector)) {
      throw new NullPointerException("No matroid provided to algorithm");
    }
    return mChain;
  }

  /**
   * returns the name of the property by which the subsets are indexed
   *
   * @return name of indexed property
   * @throws NullPointerException if no reference object has been set
   */
  public String getArrayIndexName()
    throws NullPointerException
  {
    return new String("size");
  }

  /** 
   * returns a description of the algorithm used to generate subsets 
   *
   * @return description of algorithm
   */
  public String getAlgorithmDescription()
  {
    String answer = new String();
    answer += 
      "This class implements the SubsetLister interface with an algorithm ";
    answer += 
      "for identifying the hyperplanes of a matroid that will work for ";
    answer += 
      "BasisMatroids.  It determines the dual of the matroid it is given,";
    answer += 
      "uses an instance of DefaultCircuitAlgorithm to determine its circuits,";
    answer += 
      "and then takes their complements to obtain the hyperplanes of the ";
    answer += 
      "original matroid.";

    return answer;
  }

  /** 
   * returns the name of the algorithm 
   *
   * @return name of algorithm
   */
  public String getName()
  {
    return new String("Basis Matroid Hyperplane Algorithm");
  }

  //
  // private method determines hyperplanes
  //
  private void determineHyperplanes()
  {
    //
    // set up vector with different sizes
    //
    mHyperplanes = new Vector();
    int i, n = mMatroid.getSize()-1;
    for (i=0; i<n; i++) {
      mHyperplanes.add(new Vector());
    }

    BasisMatroid dual = mMatroid.getDual();
    
    mCCTAlg.setInstance(dual);
    Collection cocircuits = mCCTAlg.getSubsetList();

    Subset cocct;
    Subset hyperplane;
    for (Iterator pCoCCT = cocircuits.iterator(); pCoCCT.hasNext(); ) {
      hyperplane = ((Subset)pCoCCT.next()).complement();
      i = hyperplane.numElements();
      ((Vector)mHyperplanes.get(i)).add(hyperplane);
    }
  }

  //
  // private method determines chain of hyperplane sizes
  //
  private void determineChain()
  {
    int i, n = mMatroid.getSize()-1;
    mChain = new Vector();
    for (i=0; i<n; i++) {
      mChain.add(new Integer(((Vector)mHyperplanes.get(i)).size()));
    }
  }

  //
  // ComplexityProvider methods
  //
  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name, or -1 if the method cannot
   * currently be executed
   *
   * @param methodName name of method to check
   * @return complexity estimate
   */
  public long getComplexityEstimate(String methodName) 
  {
    long answer = -1;

    if ((mMatroid instanceof BasisMatroid) &&
	((methodName.equals("getSubsetList")) ||
	 (methodName.equals("getSubsetListVector")) ||
	 (methodName.equals("getSubsetCountChain")))) {
      answer = getComplexity(mMatroid);
    }
    return answer;
  }
  
  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   *
   * @param methodName name of method to check
   * @param argument argument to method
   * @return complexity estimate
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    long answer  = -1;
    if ((argument instanceof BasisMatroid) &&
	(argument instanceof ComplexityProvider) &&
	((methodName.equals("getSubsetList")) ||
	 (methodName.equals("getSubsetListVector")) ||
	 (methodName.equals("getSubsetCountChain")))) {
      answer = getComplexity(argument);
    }
    return answer;
  }

  /**
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name and collection of arguments
   * (which must be placed in correct order for iteration), or -1 if the
   * method cannot be executed with the passed arguments
   *
   * @param methodName name of method to check
   * @param arguments arguments to method
   * @return complexity estimate
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    long answer  = -1;
    if (arguments.size() > 0) {
      Object argument = arguments.iterator().next();
      if ((argument instanceof BasisMatroid) &&
	  (argument instanceof ComplexityProvider) &&
	  ((methodName.equals("getSubsetList")) ||
	   (methodName.equals("getSubsetListVector")) ||
	   (methodName.equals("getSubsetCountChain")))) {
	answer = getComplexity(argument);
      }
    }
    return answer;
  }

  //
  // private method does complexity calculation
  //
  private long getComplexity(Object argument)
  {
    // this routine "knows" how to compute the complexity estimate for
    // DefaultCircuitAlgorithm and copies it; that's bad, and will need to
    // be fixed later
    long dualSteps;
    long steps;
    long p;
    int rank;
    rank = ((Matroid)argument).getSize() - ((Matroid)argument).getRank();
    steps = 
	((ComplexityProvider)argument).getComplexityEstimate("isIndependent");
    dualSteps = 
	((ComplexityProvider)argument).getComplexityEstimate("getDual");
    p = (long)Math.pow(2, rank+1);
    return dualSteps + steps * ((p-1) * (p-2) / 2);
  }
}
