package Oid;
import java.util.*;

/**
 * This class implements the SubsetLister interface with an algorithm 
 * for identifying the flats of a matroid that will work for any Matroid
 * implementation.  It begins by taking the closures of the single-element
 * subsets of the matroid, and then builds up the list of flats by taking
 * pairwise unions of flats, taking closures, and throwing new flats onto
 * the list.  It is not very efficient, since the closure operation (at 
 * least in the GF(p) representation) is not very efficient.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 25 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class DefaultFlatAlgorithm implements SubsetLister,
					     ComplexityProvider
{
  //
  // private data members
  //
  private Matroid mMatroid;
  private Vector  mFlats;
  private Vector  mChain;

  /**
   * sets the reference object for this SubsetLister
   *
   * @param arg reference object
   * @throws IllegalArgumentException if <code>arg</code> is not a 
   * <code>Matroid</code> 
   */
  public void setInstance(Object arg)
    throws IllegalArgumentException
  {
    if (!(arg instanceof Matroid)) {
      throw new IllegalArgumentException("Matroid required in " + 
					 "DefaultFlatAlgorithm");
    }
    mMatroid = (Matroid)arg;
    determineFlats();
    determineChain();
  }

  /**
   * returns the reference object for this SubsetLister 
   *
   * @return reference object
   */
  public Object getInstance()
  {
    return mMatroid;
  }

  /**
   * generates a list of the relevant Subsets for the reference object 
   *
   * @return <code>Collection</code> of subsets
   * @throws NullPointerException if no matroid was set for the algorithm
   */
  public Collection getSubsetList()
    throws NullPointerException 
  {
    if (!(mMatroid instanceof Matroid)) {
      throw new NullPointerException("No matroid set in DefaultFlatAlgorithm");
    }
    
    int i;
    Vector v;
    Vector allFlats = new Vector();
    for (i=0; i<mFlats.size(); i++) {
      v = (Vector)mFlats.get(i);
      allFlats.addAll(v);
    }
    return allFlats;
  }

  /** 
   * generates a Vector containing indexed collections of Subsets for 
   * the reference object
   *
   * @return <code>Vector</code> of collections of subsets
   * @throws NullPointerException if no matroid was set for the algorithm
   */
  public Vector getSubsetListVector()
    throws NullPointerException
  {
    if (!(mMatroid instanceof Matroid)) {
      throw new NullPointerException("No matroid set in DefaultFlatAlgorithm");
    }
    return mFlats;
  }

  /** 
   * generates a Vector containing the number of Subsets at each index
   * for the reference object
   *
   * @return <code>Vector</code> of subset counts
   * @throws NullPointerException if no matroid was set for the algorithm
   */
  public Vector getSubsetCountChain()
    throws NullPointerException
  {
    if (!(mMatroid instanceof Matroid)) {
      throw new NullPointerException("No matroid set in DefaultFlatAlgorithm");
    }
    return mChain;
  }

  /**
   * returns the name of the property by which the subsets are indexed
   * 
   * @return name of indexed property
   * @throws NullPointerException if no matroid was set for the algorithm
   */
  public String getArrayIndexName()
    throws NullPointerException
  {
    return new String("Size");
  }

  /**
   * returns a description of the algorithm used to generate subsets 
   *
   * @return algorithm description
   */
  public String getAlgorithmDescription()
  {
    StringBuffer sb = new StringBuffer();
    sb.append(" This class implements the SubsetLister interface with an ");
    sb.append("algorithm for identifying the flats of a matroid that will ");
    sb.append("work for any Matroid implementation.  It begins by taking ");
    sb.append("the closures of the single-element subsets of the matroid, ");
    sb.append("and then builds up the list of flats by taking pairwise ");
    sb.append("unions of flats, taking closures, and throwing new flats onto");
    sb.append(" the list.  It is not very efficient, since the closure ");
    sb.append("operation (at least in the GF(p) representation) is not very ");
    sb.append("efficient.");
    return sb.toString();
  }

  /**
   * returns this algorithm's name 
   *
   * @return algorithm name
   */
  public String getName() 
  {
    return new String("flat calculator");
  }

  /*
   * Determines the flats for the matroid 
   */
  private void determineFlats()
  {    
    Vector allFlats;
    Subset s1, s2, sU, sCU;
    SubsetEnumeration pSub;
    int n = mMatroid.getSize();
    int r = mMatroid.getRank();
    int theRank;
    int nCheck, nCompare;
    int i;
    
    //
    // dimension the array and instantiate the linked lists
    //
    mFlats = new Vector();
    for (i=0; i<r+1; i++) {
      mFlats.add(new Vector());
    }
    allFlats = new Vector();
    
    try {
      //
      // put closure of empty set into alFlats[0]
      //
      s1 = new Subset(n);
      s2 = mMatroid.closure(s1);
      ((Vector)mFlats.get(0)).add(s2);

      //
      // add closures of single-element subsets
      //
      for (pSub = new SubsetEnumeration(n, 1); pSub.hasMoreElements();) {
        s1 = (Subset)pSub.nextElement();
        s2 = mMatroid.closure(s1);
        if (checkSet(mMatroid, s2, allFlats)) {
          theRank = mMatroid.subsetRank(s1);
          ((Vector)mFlats.get(theRank)).add(s2);
        }
      }
      
      //
      // main loop goes through flats, takes union with all preceding flats
      // and checks the resulting subset
      //
      nCheck = 1;
      while (nCheck < allFlats.size()) {
        s1 = (Subset)allFlats.get(nCheck);
        for (nCompare=0; nCompare<nCheck; nCompare++) {
          s2 = (Subset)allFlats.get(nCompare);
          if (!(s1.subsetContains(s2) || s2.subsetContains(s1))) {
            sU = (Subset)s1.clone();
            sU.or(s2);
            sCU = mMatroid.closure(sU);
            if (checkSet(mMatroid, sCU, allFlats)) {
              theRank = mMatroid.subsetRank(sU);
              ((Vector)mFlats.get(theRank)).add(sCU);
            }
          }
        }
        nCheck++;
      }
    }
    catch (Throwable e) {
      // probably dangerous to catch everything here, but nothing bad should
      // happen that gets thrown upward, as we have no parameters other than
      // a Matroid
      throw new InternalError("Internal error in flats calculation");
    }
  }
  
  private boolean checkSet(Matroid M, Subset s, Vector lFlats)
  {
    boolean bAns = false;
    
    if (!lFlats.contains(s)) {
      lFlats.add(s);
      bAns = true;
    }
    
    return bAns;
  }

  private void determineChain()
  {
    Vector v;
    int i;

    mChain = new Vector();
    for (i=0; i<mFlats.size(); i++) {
      v = (Vector)mFlats.get(i);
      mChain.add(i, new Integer(v.size()));
    }
  }

  //
  // ComplexityProvider interface methods
  //
  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name, or -1 if the method cannot
   * currently be executed
   * 
   * @param methodName name of method
   * @return complexity estimate for method
   */
  public long getComplexityEstimate(String methodName)
  {
    long answer = -1;
    ComplexityProvider cp;

    if ((mMatroid instanceof Matroid) && 
	(mMatroid instanceof ComplexityProvider) &&
	(methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain"))) {
      cp = (ComplexityProvider)mMatroid;
      answer = estimateComplexity(mMatroid.getRank(),
				  cp.getComplexityEstimate("closure"));
    }

    return answer;
  }
  
  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   *
   * @param methodName name of method
   * @param argument argument for method
   * @return complexity estimate for method
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    long answer = -1;

    if ((argument instanceof Matroid) && 
	(argument instanceof ComplexityProvider) &&
	(methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain"))) {
      ComplexityProvider cp = (ComplexityProvider)argument;
      Matroid m = (Matroid)argument;
      answer = estimateComplexity(m.getRank(),
				  cp.getComplexityEstimate("closure"));
    }
    
    return answer;
  }

  /**
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name and collection of arguments
   * (which must be placed in correct order for iteration), or -1 if the
   * method cannot be executed with the passed arguments
   *
   * @param methodName name of method
   * @param arguments arguments for method
   * @return complexity estimate for method
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    long answer = -1;

    if (arguments.size() > 0) {
      Object argument = arguments.iterator().next();
      if ((argument instanceof Matroid) && 
	  (argument instanceof ComplexityProvider) &&
	  (methodName.equals("getSubsetList") ||
	   methodName.equals("getSubsetListVector") ||
	   methodName.equals("getSubsetCountChain"))) {
	ComplexityProvider cp = (ComplexityProvider)argument;
	Matroid m = (Matroid)argument;
	answer = estimateComplexity(mMatroid.getRank(),
				    cp.getComplexityEstimate("closure"));
      }
    }
    return answer;
  }
  
  //
  // private method that estimates the complexity in terms of the
  // rank of the matroid and the number of steps required to get the
  // closure of a set
  //
  private long estimateComplexity(long rank, long steps)
  {
    long p = (long)Math.pow(2, rank);
    return steps * p + p * (p-1) / 2;
  }

}

