package Oid;
import java.util.*;
import java.math.*;

/**
 * This class is an implementation of SubsetLister which can determine the
 * independent sets of any Matroid implementation.  It works by simply 
 * getting the bases of the matroid and then enumerating their nonempty 
 * subsets. 
 * 
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 25 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class DefaultIndependentSetAlgorithm implements SubsetLister, 
						       ComplexityProvider
{
  //
  // private data members
  // 
  private Matroid mMatroid;
  private Vector  mIndSets;
  private Vector  mChain;

  /**
   * sets the reference object for this SubsetLister 
   *
   * @param arg reference object
   * @throws IllegalArgumentException if <code>arg</code> is not a 
   * <code>Matroid</code>
   */
  public void setInstance(Object arg)
    throws IllegalArgumentException
  {
    if (!(arg instanceof Matroid)) {
      throw new IllegalArgumentException("Matroid required by "
					 + "DefaultIndependentSetAlgorithm");
    }
    mMatroid = (Matroid)arg;
    determineIndependentSets();
  }

  /**
   * returns the reference object for this SubsetLister 
   * 
   * @return reference object
   */
  public Object getInstance()
  {
    return mMatroid;
  }

  /**
   * generates a list of the relevant Subsets for the reference object 
   *
   * @return <code>Collection</code> of subsets
   * @throws NullPointerException if no matroid has been set
   */
  public Collection getSubsetList()
    throws NullPointerException
  {
    Vector allIndSets = new Vector();
    Vector v;
    Iterator p;

    for (p = mIndSets.iterator(); p.hasNext();) {
      allIndSets.addAll((Vector)p.next());
    }
//      for (i=0; i<mIndSets.size(); i++) {
//        v = (Vector)mIndSets.get(i);
//        allIndSets.addAll(v);
//      }
    return allIndSets;
  }

  /** 
   * generates a Vector containing indexed collections of Subsets for 
   * the reference object
   *
   * @return <code>Vector</code> of collections of subsets
   * @throws NullPointerException if no matroid has been set
   */
  public Vector getSubsetListVector()
    throws NullPointerException
  {
    if (!(mMatroid instanceof Matroid)) {
      throw new NullPointerException("No matroid set");
    }
    return mIndSets;
  }

  /** 
   * generates a Vector containing the number of Subsets at each index
   * for the reference object
   *
   * @return <code>Vector</code> of subset counts
   * @throws NullPointerException if no matroid has been set
   */
  public Vector getSubsetCountChain()
    throws NullPointerException
  {
    if (!(mMatroid instanceof Matroid)) {
      throw new NullPointerException("No matroid set");
    }
    return mChain;
  }

  /**
   * returns the name of the property by which the subsets are indexed
   *
   * @return name of indexed property
   * @throws NullPointerException if no matroid has been set
   */
  public String getArrayIndexName()
    throws NullPointerException
  {
    return new String("size");
  }

  /**
   * returns a description of the algorithm used to generate subsets 
   *
   * @return algorithm description
   */
  public String getAlgorithmDescription()
  {
    String answer = new String("");
    answer += 
      "This class, an implementation of SubsetLister, can determine the";
    answer += 
      "independent sets of any Matroid implementation.  It works by simply ";
    answer += 
      "getting the bases of the matroid and then enumerating their nonempty ";
    answer += 
      "subsets. ";
    return answer;
  }

  /**
   * returns this algorithm's name
   *
   * @return algorithm name 
   */
  public String getName() 
  {
    return new String("default independent set calculator");
  }

  // determines the matroid's circuits and places them into Vector mCircuits
  private void determineIndependentSets()
  {
    DefaultBasisAlgorithm basisAlg = new DefaultBasisAlgorithm();
    SubsetEnumeration subEnum;
    Subset s;
    boolean found;
    Vector bases;
    Vector indSets;
    Iterator pBasis;
    int i, k, m = mMatroid.getRank(), n = mMatroid.getSize();

    mIndSets = new Vector();
    for (i=0; i<m; i++) {
      mIndSets.add(new Vector());
    }

    // get bases and put them into place
    basisAlg.setInstance(mMatroid);
    bases = new Vector(basisAlg.getSubsetList());
    mIndSets.add(m, bases);

    // loop down through smaller sizes and check each subset--is it a 
    // subset of a basis?
    for (k=m-1; k>0; k--) {
      indSets = (Vector)mIndSets.get(k);
      try {
	subEnum = new SubsetEnumeration(n, k);
      }
      catch (Exception e) {
	throw new InternalError("Internal error in ind. set algorithm: " + 
				e.toString());
      }
      for (;subEnum.hasMoreElements(); ) {
	s = (Subset)subEnum.nextElement();
	found = false;
	for (pBasis = bases.iterator(); pBasis.hasNext(); ) {
	  if (((Subset)pBasis.next()).subsetContains(s)) {
	    found = true;
	    break;
	  }
	}
	if (found) {
	  indSets.add(s);
	}
      }
    }

    determineChain();
  }

  private void determineChain()
  {
    Vector v;
    int i;

    mChain = new Vector();
    for (i=0; i<mIndSets.size(); i++) {
      v = (Vector)mIndSets.get(i);
      mChain.add(i, new Integer(v.size()));
    }
  }

  //
  // ComplexityProvider interface methods
  //
  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name, or -1 if the method cannot
   * currently be executed
   *
   * @param methodName name of method
   * @return complexity estimate
   */
  public long getComplexityEstimate(String methodName)
  {
    long answer = -1;
    ComplexityProvider cp;

    if ((mMatroid instanceof Matroid) && 
	(mMatroid instanceof ComplexityProvider) &&
	(methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain"))) {
      cp = (ComplexityProvider)mMatroid;
      answer = estimateComplexity(mMatroid);
    }

    return answer;
  }
  
  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   *
   * @param methodName name of method
   * @param argument argument for method
   * @return complexity estimate
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    long answer = -1;
    ComplexityProvider cp;
    Matroid m;

    if ((argument instanceof Matroid) && 
	(argument instanceof ComplexityProvider) &&
	(methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain"))) {
      cp = (ComplexityProvider)argument;
      m = (Matroid)argument;
      answer = estimateComplexity(m);
    }

    return answer;
  }

  /**
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name and collection of arguments
   * (which must be placed in correct order for iteration), or -1 if the
   * method cannot be executed with the passed arguments
   *
   * @param methodName name of method
   * @param arguments arguments for method
   * @return complexity estimate
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    long answer = -1;
    ComplexityProvider cp;
    Matroid m;
    Object argument;

    if (arguments.size() > 0) {
      argument = arguments.iterator().next();
      if ((argument instanceof Matroid) && 
	  (argument instanceof ComplexityProvider) &&
	  (methodName.equals("getSubsetList") ||
	   methodName.equals("getSubsetListVector") ||
	   methodName.equals("getSubsetCountChain"))) {
	cp = (ComplexityProvider)argument;
	m = (Matroid)argument;
	answer = estimateComplexity(m);
      }
    }
    return answer;
  }

  //
  // private method that estimates the complexity in terms of the
  // rank of the matroid and the number of steps required to check
  // independence of a set
  //
  private long estimateComplexity(Matroid m)
  {
    // currently this class "knows" the complexity for finding the list
    // of bases -- need to find a way for it to ask the basis algorithm
    // later
    ComplexityProvider cp = (ComplexityProvider)m;
    long steps = cp.getComplexityEstimate("isIndependent");
    long n = m.getSize();
    long r = m.getRank();
    steps *= nCk(n, r);
    steps += Math.pow(2, r-1);
    return steps;
  }

  //
  // private method to get combinations -- only uses BigInteger if 
  // it has to
  //
  private long nCk(long n, long k)
  {
    long answer = 0;
    long i;

    if ((0 == k) || (n == k)) {
      answer = 1;
    }
    else if ((1 == k) || ((n-1) == k)) {
      answer = n;
    }
    else if ((k > 1) && (k < n-1)) {
      if (n > 20) {
	BigInteger bigAns = BigInteger.valueOf(1);
	for (i=n; i>n-k; i--) {
	  bigAns = bigAns.multiply(BigInteger.valueOf((long)i));
	}
	for (i=n-k; i>1; i--) {
	  bigAns = bigAns.divide(BigInteger.valueOf((long)i));
	}
	answer = bigAns.longValue();
      }
      else {
	answer = 1;
	for (i=n; i>n-k; i--) {
	  answer *= i;
	}
	for (i=n-k; i>1; i--) {
	  answer /= i;
	}
      }
    }

    return answer;
  }
}
