package Oid;
import java.util.*;

/**
 * <code>DisjointFamilyEnumeration</code> generates, for a family of
 * <code>Subset</code> references representing subsets of a common
 * ground set, all of the subfamilies whose members are pairwise
 * disjoint, beginning with the empty subfamily.  It is based on the
 * general sequence algorithm presented in (Nijenhuis and Wilf, 1978)
 * page 102.  It implements the <code>Enumeration</code> interface
 * from the Java SDK.
 *
 * Source: Nijenhuis, Albert and Wilf, Herbert S. (1978)
 * <em>Combinatorial Algorithms for Computers and Calculators, Second
 * Edition. Orlando, FL: Academic Press, Inc.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 2003-06-15
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class DisjointFamilyEnumeration implements Enumeration
{

  private Vector mFamily;
  private boolean[] mNextSubFamily;
  private int[] mU;
  private int mM; // ground set size
  private boolean mHasMoreElements;
  private boolean mReturnArray;

  /**
   * Creates a disjoint family enumeration with the specified family
   * of subsets of a common ground set.
   *
   * @param family Vector containing Subset references, all with the
   * same ground set size
   * @param returnArray if true, <code>nextElement</code> returns an
   * array of booleans indicating subfamily membership instead of a
   * Vector of Subsets
   * @throws IllegalArgumentException if the members of <code>family</code>
   * are not all Subset references, or if they do not have a common ground
   * set.
   */
  public DisjointFamilyEnumeration(Vector family, boolean returnArray)
    throws IllegalArgumentException
  {
    Subset sub;
    Vector newVector = new Vector();
    Object oSub;
    int m=-1;

    for (Iterator pFamily = family.iterator(); pFamily.hasNext(); ) {
      oSub = pFamily.next();
      if (!(oSub instanceof Subset)) {
	throw new IllegalArgumentException("Vector of subset references "
					   + "required");
      }
      sub = (Subset)oSub;
      if (-1 == m) {
	m = sub.getSize();
      }
      else if (m != sub.getSize()) {
	throw new IllegalArgumentException("Inconsistent ground set sizes");
      }
      
      newVector.add(sub);
    }
    mFamily = newVector;
    mNextSubFamily = new boolean[mFamily.size()];
    mU = new int[m];
    mM = m;
    mHasMoreElements = true;
    mReturnArray = returnArray;
  }

  public boolean hasMoreElements()
  {
    return mHasMoreElements;
  }

  public Object nextElement()
    throws NoSuchElementException
  {
    int n = mFamily.size();
    int m = mM;
    int i, j;
    Subset F;
    boolean select;
    Object result;

    // make sure there is a next element
    if (!mHasMoreElements) {
      throw new NoSuchElementException("no more elements");
    }
   
    if (!mReturnArray) {
      // build vector to return before generating new selection
      Vector vectorResult = new Vector();
      for (i=0; i<n; i++) {
	if (mNextSubFamily[i]) {
	  vectorResult.add((Subset)mFamily.get(i));
	}
      }
      result = (Object)vectorResult;
    }
    else {
      boolean[] arrayResult = new boolean[n];
      for (i=0; i<n; i++) {
	arrayResult[i] = mNextSubFamily[i];
      }
      result = (Object)arrayResult;
    }

    // now initialize U
    for (j=0; j<m; j++) {
      mU[j] = 0;
    }

    for (i=0; i<n; i++) {
      if (mNextSubFamily[i]) {
	F = (Subset)mFamily.get(i);

	// increment members of U corresponding to elements of F
	for (j=0; j<m; j++) {
	  if (F.get(j)) {
	    ++mU[j];
	  }
	}
      }
    }

    i = n-1;
    select = false;
    while (!select && (i >= 0)) {
      F = (Subset)mFamily.get(i);
      if (mNextSubFamily[i]) {
	//
	// this subset is not a candidate to add to selection, 
	// decrement its corresponding members of U
	//
	for (j=0; j<m; j++) {
	  if (F.get(j)) {
	    --mU[j];
	  }
	}
	select = false;
      }
      else {
	//
	// this may be a candidate to put in the next family;
	// check to see if it is disjoint from all the subsets before
	// it in the current family (in other words, all of its corresponding
	// values in U are 0)
	//
	select = true;

	for (j=0; j<m; j++) {
	  if (F.get(j) && (mU[j] > 0)) {
	    select = false;
	    break;
	  }
	}
      }

      // if a subset has not been selected, decrement i
      if (!select) {
	--i;
      }
    }

    if (select) {
      // 
      // set the boolean value to indicate the newly selected subset, and
      // deselect all following subsets
      //
      mNextSubFamily[i] = true;
      for (j=i+1; j<n; j++) {
	mNextSubFamily[j] = false;
      }
    }
    else {
      // 
      // there are no more changes to make
      // 
      mHasMoreElements = false;
    }

    // return the current subfamily
    return result;
  }
}
