package Oid;
/**
 * An instance of FiniteFieldFactory, given a prime <em>p</em> and an 
 * exponent <em>k > 0</em> will return an object that represents the field
 * GF(<em>p</em>^<em>k</em>).
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 14 Aug 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class FiniteFieldFactory
{
  /**
   * returns an implementation of <code>FiniteField</code> with the 
   * specified field size
   *
   * @param q field size (must be a prime power)
   * @return finite field reference
   * @throws IllegalArgumentException if <code>q</code> is not a prime power
   */
  public static FiniteField createFiniteField(int q)
    throws IllegalArgumentException
  {
    PrimeFactorization pf = new PrimeFactorization();
    int[][] factors = pf.getFactors(q);

    if (factors.length != 1) {
      throw new IllegalArgumentException(q + " is not a prime power");
    }

    int p = factors[0][0];
    int k = factors[0][1];

    return FiniteFieldFactory.createFiniteField(p, k);
  }

  /**
   * returns an implementation of <code>FiniteField</code> with the 
   * specified characteristic and size
   * 
   * @param p field characteristic (must be prime)
   * @param k power of p that will be size of field (must be positive)
   * @return finite field object
   * @throws IllegalArgumentException if p is not prime or k is not positive
   */
  public static FiniteField createFiniteField(int p, int k) 
    throws IllegalArgumentException
  {
    FiniteField answer = (FiniteField)null;

    if (k <= 0) {
      throw new IllegalArgumentException("Invalid exponent");
    }

    try {
      if (1 == k) {
	answer = new GFpCalculator(p);
      }
      else if (2 == p) {
	answer = new GF2kCalculator(k);
      }
      else {
	answer = new GFpkCalculator(p, k);
      }
    }
    catch (Exception ex) {
      throw new IllegalArgumentException(ex.getMessage());
    }

    return answer;
  }

}
