package Oid;
import java.util.*;

/**
 * An instance of FiniteFieldVectorEnumeration produces all of the vectors over
 * GF(q) (in the form of FiniteFieldVector instances) of a given size, in 
 * lexicographic order.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 27 Oct 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class FiniteFieldVectorEnumeration implements Enumeration
{
  //
  // private data members
  // 
  private FiniteFieldVector mCurrentVec;
  private int               mFieldSize;
  private FiniteField       mField;
  private int               mSize;
  private boolean           mHasMoreElements;
  private static final int  mDEFAULT_FIELDSIZE =  2;
  private static final int  mDEFAULT_SIZE    = 10;

  /** 
   * creates an instance with the vector size set to 10 and the field
   * set to GF(2)
   */
  public FiniteFieldVectorEnumeration()
  {
    mFieldSize = mDEFAULT_FIELDSIZE;
    try {
      mField = new GFpCalculator(mFieldSize);
    }
    catch (IllegalArgumentException e) {
      // this "can't happen"
      throw new InternalError("Internal error in "
			      + "FiniteFieldVectorEnumeration");
    }
    mSize = mDEFAULT_SIZE;
    mCurrentVec = new FiniteFieldVector(mSize, mField);
    mCurrentVec.multiply(0);
    mHasMoreElements = true;
  }

  /** 
   * creates an instance with the field size set to <code>q</code> and 
   * the vector size set to 10
   *
   * @param q field size
   * @throws IllegalArgumentException if <code>q</code> is not a prime power
   */
  public FiniteFieldVectorEnumeration(int q)
    throws IllegalArgumentException
  {
    mFieldSize = q;
    mField = FiniteFieldFactory.createFiniteField(mFieldSize);
    mSize = mDEFAULT_SIZE;
    mCurrentVec = new FiniteFieldVector(mSize, mField);
    mCurrentVec.multiply(0);
    mHasMoreElements = true;
  }

  /**
   * sets the size to 10 and uses <code>field</code> (and its field size)
   *
   * @param calc field to use for vectors
   */
  public FiniteFieldVectorEnumeration(FiniteField field)
  {
    mField = field;
    mFieldSize = field.getFieldSize();
    mSize = mDEFAULT_SIZE;
    mCurrentVec = new FiniteFieldVector(mSize, mField);
    mCurrentVec.multiply(0);
    mHasMoreElements = true;
  }

  /**
   * sets the size and modulus from the parameters <code>size</code> and
   * <code>q</code> for the size of the vectors and the field size
   *
   * @param size size of vectors to generate
   * @param q field size for vectors (must be a prime power)
   * @throws IllegalArgumentException if size isn't positive or q isn't a 
   * prime power
   */
  public FiniteFieldVectorEnumeration(int size, int q)
    throws IllegalArgumentException
  {
    mFieldSize = q;
    mField = FiniteFieldFactory.createFiniteField(q);
    mSize = size;
    mCurrentVec = new FiniteFieldVector(size, mField);    
    mCurrentVec.multiply(0);
    mHasMoreElements = true;
  }

  /**
   * sets the size and modulus from size and <code>field</code>
   * @param size size of vectors to generate
   * @param field provides field size for vectors
   */
  public FiniteFieldVectorEnumeration(int size, FiniteField field)
  {
    mField = field;
    mFieldSize = mField.getFieldSize();
    mSize = size;
    mCurrentVec = new FiniteFieldVector(size, mField);    
    mCurrentVec.multiply(0);
    mHasMoreElements = true;
  }

  /** 
   * indicates whether the enumeration has more elements 
   * @return boolean value indicating whether there are more vectors
   */
  public boolean hasMoreElements()
  {
    return mHasMoreElements;
  }

  /** 
   * returns the next FiniteFieldVector in the list 
   * @return next vector (actually a FiniteFieldVector instance)
   */
  public Object nextElement()
  {
    FiniteFieldVector answer = new FiniteFieldVector(mCurrentVec);
    int i;

    // find the last element that is less than mFieldSize-1
    for (i=mSize-1; i>=0; i--) {
      if (mCurrentVec.get(i) < mFieldSize-1) {
	break;
      }
    }

    if (i >= 0) {
      // increment that element and set everything beyond it to 0
      mCurrentVec.set(i, mCurrentVec.get(i)+1);
      for (i++; i<mSize; i++) {
	mCurrentVec.set(i, 0);
      }
    }
    else {
      mHasMoreElements = false;
    }
    return (Object)answer;
  }
}
