package Oid;
import java.util.*;
import java.io.*;

/**
 * This class implements the SubsetLister interface with an algorithm 
 * for identifying the flats of a matroid that will work for any Matroid
 * implementation.  It begins by taking the closures of the single-element
 * subsets of the matroid, and then builds up the list of flats by taking
 * pairwise unions of flats, taking closures, and throwing new flats onto
 * the list.  It is not very efficient, since the closure operation (at 
 * least in the GF(p) representation) is not very efficient.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 25 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class FlatAlgorithm implements SubsetLister,
				      ComplexityProvider
{
  //
  // private data members
  //
  private Matroid mMatroid;
  private Vector  mFlats;
  private Vector  mChain;

  /**
   * sets the reference object for this SubsetLister
   *
   * @param arg reference object
   * @throws IllegalArgumentException if <code>arg</code> is not a 
   * <code>Matroid</code> 
   */
  public void setInstance(Object arg)
    throws IllegalArgumentException
  {
    if (!(arg instanceof Matroid)) {
      throw new IllegalArgumentException("Matroid required in " + 
					 "FlatAlgorithm");
    }
    mMatroid = (Matroid)arg;
    determineFlats();
    determineChain();
  }

  /**
   * returns the reference object for this SubsetLister 
   *
   * @return reference object
   */
  public Object getInstance()
  {
    return mMatroid;
  }

  /**
   * generates a list of the relevant Subsets for the reference object 
   *
   * @return <code>Collection</code> of subsets
   * @throws NullPointerException if no matroid was set for the algorithm
   */
  public Collection getSubsetList()
    throws NullPointerException 
  {
    if (!(mMatroid instanceof Matroid)) {
      throw new NullPointerException("No matroid set in FlatAlgorithm");
    }
    
    int r, rank=mMatroid.getRank();
    Vector v;
    Vector allFlats = new Vector();
    Flat f;
    Iterator pF;

    for (r=0; r<=rank; r++) {
      v = (Vector)mFlats.get(r);
      for (pF = v.iterator(); pF.hasNext(); ) {
	f = (Flat)pF.next();
	allFlats.add(new Subset(f.getSubset()));
      }
    }

    return allFlats;
  }

  /** 
   * generates a Vector containing indexed collections of Subsets for 
   * the reference object
   *
   * @return <code>Vector</code> of collections of subsets
   * @throws NullPointerException if no matroid was set for the algorithm
   */
  public Vector getSubsetListVector()
    throws NullPointerException
  {
    if (!(mMatroid instanceof Matroid)) {
      throw new NullPointerException("No matroid set in FlatAlgorithm");
    }
    
    Vector slv = new Vector();
    Vector level1, level2;
    int r, rank=mMatroid.getRank();
    Flat f;
    Iterator pF;

    for (r=0; r<=rank; r++) {
      level1 = (Vector)mFlats.get(r);
      level2 = new Vector();
      for (pF = level1.iterator(); pF.hasNext(); ) {
	level2.add(new Subset(((Flat)pF.next()).getSubset()));
      }
      slv.add(level2);
    }

    return slv;
  }

  /** 
   * generates a Vector containing the number of Subsets at each index
   * for the reference object
   *
   * @return <code>Vector</code> of subset counts
   * @throws NullPointerException if no matroid was set for the algorithm
   */
  public Vector getSubsetCountChain()
    throws NullPointerException
  {
    if (!(mMatroid instanceof Matroid)) {
      throw new NullPointerException("No matroid set in FlatAlgorithm");
    }
    return mChain;
  }

  /**
   * returns the name of the property by which the subsets are indexed
   * 
   * @return name of indexed property
   * @throws NullPointerException if no matroid was set for the algorithm
   */
  public String getArrayIndexName()
    throws NullPointerException
  {
    return new String("Rank");
  }

  /**
   * returns a description of the algorithm used to generate subsets 
   *
   * @return algorithm description
   */
  public String getAlgorithmDescription()
  {
    StringBuffer sb = new StringBuffer();
    sb.append(" This class implements the SubsetLister interface with an ");
    sb.append("algorithm for identifying the flats of a matroid that will ");
    sb.append("work for any Matroid implementation.");
    return sb.toString();
  }

  /**
   * returns this algorithm's name 
   *
   * @return algorithm name
   */
  public String getName() 
  {
    return new String("flat calculator");
  }

  /**
   * returns the Vector of Vectors of Flat objects obtained 
   *
   * @return Vector of Vectors of flats, by rank
   * @throws NullPointerException if no matroid has been set
   */
  public Vector getFlats()
    throws NullPointerException
  {
    if (!(mMatroid instanceof Matroid)) {
      throw new NullPointerException("no matroid has been set");
    }

    return mFlats;
  }

  /*
   * Determines the flats for the matroid
   */
  private void determineFlats()
  {
    Vector curFlats, subFlats;
    Subset s1, s2, cl;
    int rank = mMatroid.getRank();
    int size = mMatroid.getSize();
    boolean[] D = new boolean[size];
    int e,e1,r;
    Flat curFlat, subFlat, subFlat2;
    Iterator pF1, pF2;

    //
    // set up the Vectors
    //
    mFlats = new Vector();
    for (r=0; r<=rank; r++) {
      mFlats.add(new Vector());
    }

    //
    // closure of empty set goes into zero level
    //
    curFlats = (Vector)mFlats.get(0);
    s1 = new Subset(size);
    cl = mMatroid.closure(s1);
    curFlat = new Flat(0, cl);
    curFlats.add(curFlat);

    //
    // loop through other ranks
    //
    for(r=1; r<rank; r++) {
      subFlats = (Vector)mFlats.get(r-1);
      curFlats = (Vector)mFlats.get(r);
      
      //
      // loop through lower rank flats
      //
      for (pF1 = subFlats.iterator(); pF1.hasNext(); ) {
	subFlat = (Flat)pF1.next();
	s1 = subFlat.getSubset();

	// 
	// now set up array of excluded elements
	// 
	for (e=0; e<size; e++) {
	  D[e] = s1.get(e);
	}

	// 
	// also exclude elements of all previous superflats
	//
	for (pF2 = subFlat.getSuperFlats().iterator(); pF2.hasNext(); ) {
	  subFlat2 = (Flat)pF2.next();
	  s2 = subFlat2.getSubset();
	  for (e=0; e<size; e++) {
	    D[e] = D[e] || (s2.get(e));
	  }
	}

	// 
	// now find the first element not in the excluded list
	// 
	e = -1;
	for (e1=0; e1<size; e1++) {
	  if (!D[e1]) {
	    e = e1;
	    break;
	  }
	}

	//
	// keep going as long as there are non-excluded elements
	//
	while (e >= 0) {
	  s2 = new Subset(s1);
	  s2.set(e);
	  cl = mMatroid.closure(s2);
	  curFlat = new Flat(r, cl);
	  
	  curFlats.add(curFlat);
	  	    
	  // 
	  // now set up current flat's subflats and superflats
	  //
	  for (pF2 = subFlats.iterator(); pF2.hasNext(); ) {
	    subFlat2 = (Flat)pF2.next();
	    if (cl.subsetContains(subFlat2.getSubset())) {
	      curFlat.addSubFlat(subFlat2);
	      subFlat2.addSuperFlat(curFlat);
	    }
	  }

	  //
	  // add elements of new flat to excluded list and find
	  // another non-excluded element, if there is one
	  //
	  e = -1;
	  for (e1=size-1; e1>=0; e1--) {
	    D[e1] = D[e1] || (cl.get(e1));
	    if (!D[e1]) {
	      e = e1;
	    }
	  }
	}
      }
    }

    // 
    // finally add entire ground set at top
    //
    s1 = new Subset(size);
    for (e=0; e<size; e++) {
      s1.set(e);
    }
    curFlats = (Vector)mFlats.get(rank);
    subFlats = (Vector)mFlats.get(rank-1);
    
    curFlat = new Flat(rank, s1);

    for (pF1 = subFlats.iterator(); pF1.hasNext(); ) {
      curFlat.addSubFlat((Flat)pF1.next());
    }
    
    curFlats.add(curFlat);
  }
  
  private void determineChain()
  {
    Vector v;
    int i;

    mChain = new Vector();
    for (i=0; i<mFlats.size(); i++) {
      v = (Vector)mFlats.get(i);
      mChain.add(i, new Integer(v.size()));
    }
  }

  //
  // ComplexityProvider interface methods
  //
  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name, or -1 if the method cannot
   * currently be executed
   * 
   * @param methodName name of method
   * @return complexity estimate for method
   */
  public long getComplexityEstimate(String methodName)
  {
    long answer = -1;
    ComplexityProvider cp;

    if ((mMatroid instanceof Matroid) && 
	(mMatroid instanceof ComplexityProvider) &&
	(methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain"))) {
      cp = (ComplexityProvider)mMatroid;
      answer = estimateComplexity(mMatroid.getRank(),
				  cp.getComplexityEstimate("closure"));
    }

    return answer;
  }
  
  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   *
   * @param methodName name of method
   * @param argument argument for method
   * @return complexity estimate for method
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    long answer = -1;

    if ((argument instanceof Matroid) && 
	(argument instanceof ComplexityProvider) &&
	(methodName.equals("getSubsetList") ||
	 methodName.equals("getSubsetListVector") ||
	 methodName.equals("getSubsetCountChain"))) {
      ComplexityProvider cp = (ComplexityProvider)argument;
      Matroid m = (Matroid)argument;
      answer = estimateComplexity(m.getRank(),
				  cp.getComplexityEstimate("closure"));
    }
    
    return answer;
  }

  /**
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name and collection of arguments
   * (which must be placed in correct order for iteration), or -1 if the
   * method cannot be executed with the passed arguments
   *
   * @param methodName name of method
   * @param arguments arguments for method
   * @return complexity estimate for method
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    long answer = -1;

    if (arguments.size() > 0) {
      Object argument = arguments.iterator().next();
      if ((argument instanceof Matroid) && 
	  (argument instanceof ComplexityProvider) &&
	  (methodName.equals("getSubsetList") ||
	   methodName.equals("getSubsetListVector") ||
	   methodName.equals("getSubsetCountChain"))) {
	ComplexityProvider cp = (ComplexityProvider)argument;
	Matroid m = (Matroid)argument;
	answer = estimateComplexity(mMatroid.getRank(),
				    cp.getComplexityEstimate("closure"));
      }
    }
    return answer;
  }
  
  //
  // private method that estimates the complexity in terms of the
  // rank of the matroid and the number of steps required to get the
  // closure of a set
  //
  private long estimateComplexity(long rank, long steps)
  {
    long p = (long)Math.pow(2, rank);
    return steps * p + p * (p-1) / 2;
  }

}

