package Oid;
import java.util.*;

/**
 * An instance of <code>FlatMatroid</code> implements the operations
 * for a <code>Matroid</code> using the lattice of flats for
 * reference.  It must be constructed from another
 * <code>FlatMatroid</code> or from another <code>Matroid</code>
 * implementation via the <code>FlatAlgorithm</code>.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 26 May 2003
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class FlatMatroid implements Matroid
{
  //
  // private data members
  //
  private Vector mFlats;
  private int mRank;
  private int mSize;
  private String mName;
  private String[] mLabel;

  /**
   * copy constructor
   * 
   * @param fm another <code>FlatMatroid</code>
   */
  public FlatMatroid(FlatMatroid fm)
  {
    Vector rankFlats;
    Vector newFlats;
    Iterator pFlat;
    Flat oldFlat;
    int i;

    mSize = fm.getSize();
    mRank = fm.getRank();
    mName = new String(fm.getName());
    mLabel = new String[mSize];
    for (i=0; i<mSize; i++) {
      mLabel[i] = new String(fm.getLabel(i));
    }
    mFlats = new Vector();

    // copy flats over
    for (i=0; i<=mRank; i++) {
      rankFlats = fm.getFlatsOfRank(i);
      newFlats = new Vector();
      for (pFlat = rankFlats.iterator(); pFlat.hasNext(); ) {
	oldFlat = (Flat)pFlat.next();
	newFlats.add(new Flat(i, oldFlat.getSubset()));
      }
      mFlats.add(newFlats);
    }
    
    // easier to check subsets than sort out pointers
    findSubSupFlats();
  }

  /**
   * constructs this matroid using matroid and last set of flats generated by 
   * <code>FlatAlgorithm</code>.
   * 
   * @param flatAlg reference to <code>FlatAlgorithm</code>
   * @throws NullPointerException if the algorithm doesn't have an instance set
   */
  public FlatMatroid(FlatAlgorithm flatAlg)
    throws NullPointerException
  {
    int i;
    Matroid M = (Matroid)flatAlg.getInstance();
    if (!(M instanceof Matroid)) {
      throw new NullPointerException("no matroid to obtain flats from");
    }
    mName = new String(M.getName());
    mSize = M.getSize();
    mRank = M.getRank();
    mFlats = flatAlg.getFlats();
    mLabel = new String[mSize];
    for (i=0; i<mSize; i++) {
      mLabel[i] = new String(M.getLabel(i));
    }
  }

  /**
   * constructs this matroid from another matroid by generating its flats
   * using <code>FlatAlgorithm</code>.
   *
   * @param M reference to an object implementing <em>Matroid</em>
   */
  public FlatMatroid(Matroid M)
  {
    int i;
    FlatAlgorithm flatAlg = new FlatAlgorithm();
    flatAlg.setInstance(M);

    mSize = M.getSize();
    mRank = M.getRank();
    mName = new String(M.getName());
    mLabel = new String[mSize];
    for (i=0; i<mSize; i++) {
      mLabel[i] = new String(M.getLabel(i));
    }
    mFlats = flatAlg.getFlats();
  }

  //
  // Matroid interface methods
  //
  /** 
   * returns the number of elements in matroid 
   *
   * @return element count
   */
  public int getSize()
  {
    return mSize;
  }

  /** 
   * returns the rank of the matroid 
   *
   * @return matroid rank
   */
  public int getRank()
  {
    return mRank;
  }

  /** 
   * sets the matroid's name 
   *
   * @param newName new name for matroid
   */
  public void setName(String newName)
  {
    mName = new String(newName);
  }

  /** 
   * returns the matroid's name 
   *
   * @return name of matroid
   */
  public String getName() 
  {
    return new String(mName);
  }

  /** 
   * sets the label for one of the matroid's elements 
   *
   * @param element element number (starts at zero)
   * @param label label for element
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is out
   * of bounds
   */
  public void setLabel(int element, String label)
    throws ArrayIndexOutOfBoundsException
  {
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("invalid element index: " +
					       String.valueOf(element));
    }
    
    mLabel[element] = new String(label);
  }

  /** 
   * returns the label for one of the matroid's elements 
   *
   * @param element element number (starts at zero)
   * @return label of element
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is out
   * of bounds
   */
  public String getLabel(int element) 
    throws ArrayIndexOutOfBoundsException
  {
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("invalid element index: " +
					       String.valueOf(element));
    }
    return new String(mLabel[element]);
  }

  /** 
   * copies the internal state of this matroid to a string in the
   * specified format
   *
   * @param format format string ("text" is currently supported)
   * @return matroid's state in string form
   * @throws IllegalArgumentException if format is not supported
   */
  public String getState(String format)
  throws IllegalArgumentException
  {
    StringBuffer labels = new StringBuffer("");
    StringBuffer answer = new StringBuffer("");
    Vector flats;
    Iterator pFlat;
    Flat f;

    addLine(answer, "FlatMatroid", mName);
    addLine(answer, "size", String.valueOf(mSize));
    addLine(answer, "rank", String.valueOf(mRank));
    for (int i = 0; i<mSize; i++) {
      labels.append(mLabel[i]);
      labels.append(" ");
    }
    addLine(answer, "labels", labels.toString());
    for (int rank=0; rank<=mRank; rank++) {
      addLine(answer, "flatRank", String.valueOf(rank));
      flats = (Vector)mFlats.get(rank);
      for (pFlat = flats.iterator(); pFlat.hasNext(); ) {
	f = (Flat)pFlat.next();
	addLine(answer, "flat", f.getSubset().toString(true));
      }
    }

    return answer.toString();
  }

  private void addLine(StringBuffer buf, String label, String value)
  {
    buf.append(label);
    buf.append(" ");
    buf.append(value);
    buf.append("\n");
  }

  /** 
   * sets state of this matroid from the specified string using 
   * the specified format
   *
   * @param state state string for matroid
   * @param format format state string is in
   * @throws IllegalArgumentException if format is not supported
   */
  public void setState(String state, String format)
  throws IllegalArgumentException
  {
    String line;

    int rank=-1, size=-1, i;
    String name;
    String[] label;

    int flatRank=0;
    Vector flats=(Vector)null, rankFlats=(Vector)null;
    Flat newFlat;

    String lineLabel, token;
    StringBuffer subsetString;
    Subset flatSubset;

    StringTokenizer lineTokens, wordTokens;

    try {
      for (lineTokens = new StringTokenizer(state, "\n");
	   lineTokens.hasMoreElements(); ) {
	wordTokens = new StringTokenizer((String)lineTokens.nextElement());
	if (wordTokens.countTokens() > 0) {
	  lineLabel = (String)wordTokens.nextElement();
	
	  if (lineLabel.equals("FlatMatroid")) {
	    name = (String)wordTokens.nextElement();
	  }
	  else if (lineLabel.equals("size")) {
	    size = Integer.parseInt((String)wordTokens.nextElement());
	    if (size <= 0) {
	      throw new IllegalArgumentException("matroid size must be" +
						 " positive");
	    }
	  }
	  else if (lineLabel.equals("rank")) {
	    rank = Integer.parseInt((String)wordTokens.nextElement());
	    if (rank <= 0) {
	      throw new IllegalArgumentException("matroid rank must be" + 
						 " positive");
	    }
	    flats = new Vector();
	    for (i=0; i<=rank; i++) {
	      flats.add(new Vector());
	    }
	  }
	  else if (lineLabel.equals("labels")) {
	    if (size < 0) {
	      throw new IllegalArgumentException("size must precede labels" + 
						 " in FlatMatroid state" + 
						 " string");
	    }
	    label = new String[size];
	    for (i=0; i<size; i++) {
	      if (!wordTokens.hasMoreElements()) {
		throw new IllegalArgumentException("not enough labels");
	      }
	      label[i] = (String)wordTokens.nextElement();
	    }
	  }
	  else if (lineLabel.equals("flatRank")) {
	    flatRank = Integer.parseInt((String)wordTokens.nextElement());
	    if ((flatRank < 0) || (flatRank > rank)) {
	      throw new IllegalArgumentException("invalid flat rank: " + 
						 String.valueOf(flatRank));
	    }
	    rankFlats = (Vector)flats.get(flatRank);
	  }
	  else if (lineLabel.equals("flat")) {
	    if ((size < 0) || (rank < 0) || (!(rankFlats instanceof Vector))) {
	      throw new IllegalArgumentException("flat in invalid location");
	    }
	    subsetString = new StringBuffer("");
	    for(;wordTokens.hasMoreElements();) {
	      subsetString.append(wordTokens.nextElement());
	      subsetString.append(' ');
	      flatSubset = new Subset(subsetString.toString(), size, true);
	      newFlat = new Flat(flatRank, flatSubset);
	      rankFlats.add(newFlat);
	    }
	  }
	}
      }
    }
    catch (NumberFormatException nex) {
      throw new IllegalArgumentException(nex.toString());
    }

    findSubSupFlats();
    
  }

  /** 
   * determines the rank of a subset of the matroid's elements 
   *
   * @param s subset of base set of matroid
   * @return rank of <code>s</code>in this matroid
   */
  public int subsetRank(Subset s)
  {
    int answer = 0;

    if ((s.getSize() == mSize) && (s.numElements() > 0)) {
      answer = cl(s).getRank();
    }

    return answer;
  }

  /** 
   * determines whether a subset of the matroid is a circuit 
   *
   * @param s subset of base set of matroid
   * @return boolean indicating whether <code>s</code>is a circuit in
   * this matroid
   */
  public boolean isCircuit(Subset s)
  {
    int n = s.numElements();
    int i;
    int r;
    Subset t;

    if ((0 == n) || (n > mRank+1) || (s.getSize() != mSize)) {
      return false;
    }

    // set must have rank equal to its number of elements, minus one
    r = subsetRank(s);
    if (r != n-1) {
      return false;
    }

    // every subset (only need to check single-element deletions) must be ind.
    for (i=0; i<mSize; i++) {
      if (s.get(i)) {
	t = new Subset(s);
	t.clear(i);
	if (subsetRank(t) != n-1) {
	  return false;
	}
      }
    }

    return true;
  }

  /** 
   * determines whether a subset of the matroid is a flat 
   *
   * @param s subset of base set of matroid
   * @return boolean indicating whether <code>s</code>is a flat in this matroid
   */
  public boolean isFlat(Subset s)
  {
    boolean answer = false;

    if (s.getSize() == mSize) {
      answer = s.subsetEquals(cl(s).getSubset());
    }

    return answer;
  }

  /** 
   * determines whether a subset of the matroid is a basis 
   *
   * @param s subset of base set of matroid
   * @return boolean indicating whether <code>s</code>is a basis in
   * this matroid
   */
  public boolean isBasis(Subset s)
  {
    boolean answer = false;
    
    if ((s.getSize() != mSize) || (s.numElements() != mRank)) {
      return false;
    }

    answer = (subsetRank(s) == mRank);

    return answer;
  }

  /** 
   * determines whether a subset of the matroid is independent 
   *
   * @param s subset of base set of matroid
   * @return boolean indicating whether <code>s</code>is independent
   * in this matroid
   */
  public boolean isIndependent(Subset s)
  {
    boolean answer = false;
    int n = s.numElements();

    if ((s.getSize() != mSize) || (n > mRank)) {
      return false;
    }

    answer = (n == subsetRank(s));

    return answer;
  }

  /** 
   * determines whether a subset of the matroid is a hyperplane 
   *
   * @param s subset of base set of matroid
   * @return boolean indicating whether <code>s</code>is a hyperplane
   * in this matroid
   */
  public boolean isHyperplane(Subset s)
  {
    boolean answer = false;
    Flat cls;

    if (s.getSize() != mSize) {
      return false;
    }

    cls = cl(s);
    answer = (cls.getRank() == mRank-1) && (s.subsetEquals(cls.getSubset()));
    
    return answer;
  }

  /** 
   * determines whether an element is in the closure of a set 
   *
   * @param s subset of base set of matroid
   * @param e element to check (elements start with zero)
   * @return boolean indicating whether <code>e</code> is in
   * cl(<code>s</code>) in this matroid
   */
  public boolean isInClosure(Subset s, int e)
  { 
    boolean answer = false;
    
    if (s.getSize() != mSize) {
      return false;
    }

    if ((e<0) || (e >= mSize)) {
      throw new IllegalArgumentException("invalid element index: " + 
					 String.valueOf(e));
    }

    answer = cl(s).getSubset().get(e);

    return answer;
  }

  /** 
   * determines the closure of a subset of the matroid 
   *
   * @param s subset of base set of matroid
   * @param subset with closure of <code>s</code>in this matroid
   */
  public Subset closure(Subset s)
  {
    Subset answer = (Subset)null;

    if (s.getSize() != mSize) {
      return (Subset)null;
    }

    answer = new Subset(cl(s).getSubset());

    return answer;
  }

  // 
  // public methods specific to this Matroid representation
  //
  
  /**
   * returns a reference to the collection of flats for this matroid
   *
   * @return Vector of Vectors of Flat references indexed by rank
   */
  public Vector getFlats()
  {
    return mFlats;
  }

  /**
   * returns a reference to the top flat in the lattice; i.e., the entire
   * ground set
   *
   * @return reference to Flat
   */
  public Flat getTopFlat()
  {
    Flat answer = (Flat)((Vector)mFlats.get(mRank)).get(0);

    return answer;
  }

  /**
   * returns a reference to the bottom flat in the lattice; i.e., the 
   * closure of the empty set, or the set of loops of the matroid
   *
   * @return reference to Flat
   */
  public Flat getBottomFlat()
  {
    Flat answer = (Flat)((Vector)mFlats.get(0)).get(0);

    return answer;
  }

  /** 
   * returns a reference to the Vector of flats of the specified rank
   *
   * @param rank rank of flats to return
   * @return Vector of Flat references
   * @throws IllegalArgumentException if rank is less than zero or greater
   * than the rank of the matroid
   */
  public Vector getFlatsOfRank(int rank) 
    throws IllegalArgumentException
  {
    if ((rank < 0) || (rank > mRank)) {
      throw new IllegalArgumentException("rank out of range: " + 
					 String.valueOf(rank));
    }

    Vector answer = (Vector)mFlats.get(rank);

    return answer;
  }

  private void findSubSupFlats()
  {
    Vector flats, subFlats;
    Flat rankFlat, subFlat;
    Iterator pF1, pF2;
    int rank;

    // search for super- subflats for remaining ranks
    for (rank=1; rank<=mRank; rank++) {
      flats = (Vector)mFlats.get(rank);
      subFlats = (Vector)mFlats.get(rank-1);
      for (pF1 = flats.iterator(); pF1.hasNext(); ) {
	rankFlat = (Flat)pF1.next();
	for (pF2 = subFlats.iterator(); pF2.hasNext(); ) {
	  subFlat = (Flat)pF2.next();
	  if (rankFlat.getSubset().subsetContains(subFlat.getSubset())) {
	    rankFlat.addSubFlat(subFlat);
	    subFlat.addSuperFlat(rankFlat);
	  }
	}
      }
    }
  }

  private Flat cl(Subset s)
  {
    Vector C0, C1=(Vector)null;
    Flat f, sub;
    Iterator pF, pSub;
    
    C0 = new Vector();
    C0.add(getTopFlat());
    
    while (C0.size() > 0) {
      C1 = C0;
      C0 = new Vector();
      for (pF = C1.iterator(); pF.hasNext(); ) {
	f = (Flat)pF.next();
	for (pSub = f.getSubFlats().iterator(); pSub.hasNext(); ) {
	  sub = (Flat)pSub.next();
	  if (sub.getSubset().subsetContains(s) && 
	      !(containsFlat(C0, sub))) {
	    C0.add(sub);
	  }
	}
      }
    }

    if (C1.size() != 1) {
      throw new InternalError(String.valueOf(C1.size()) + 
			      " closures found for set " +
			      s.toString(true));
    }
    
    return (Flat)C1.get(0);
  }

  private boolean containsFlat(Vector v, Flat f)
  {
    boolean answer = false;
    Flat f2;
    for (Iterator pF = v.iterator(); pF.hasNext(); ) {
      f2 = (Flat)pF.next();
      if (f2.getSubset().subsetEquals(f.getSubset())) {
	answer = true;
	break;
      }
    }

    return answer;
  }
}
