package Oid;
import java.math.*;
import java.util.*;

/**
 * A <code>GF2kCalculator</code> instance can perform arithmetic operations 
 * in GF(2^k) where 1 < k < 31.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 18 March 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class GF2kCalculator implements FiniteField
{
  //
  // private data members
  //
  private int mK;
  private int mFieldSize;
  private int mPolynomial;
  private int mMask;
  private int[] mPower;
  private static final int DEFAULT_K = 2;
  private static final char DEFAULT_X = 'x';

  /**
   * default constructor sets k = 2
   */
  public GF2kCalculator()
  {
    mK = DEFAULT_K;
    initArrays();
  }

  /**
   * constructor accepts a value for k
   * @param k power of 2 for size of field
   * @throws IllegalArgumentException thrown if k <= 1
   */
  public GF2kCalculator(int k) 
  throws IllegalArgumentException
  {
    if (k <= 0) {
      throw new IllegalArgumentException("k must be greater than one");
    }

    if (k > 30) {
      throw new IllegalArgumentException("k must not be greater than 30");
    }

    mK = k;
    initArrays();
  }

  /**
   * returns the value of k used to initialize the instance
   * @return power of 2 that is size of the field
   */
  public int getExponent() {
    return mK;
  }

  /**
   * returns the characteristic of this field (2)
   * @return field characteristic as integer
   */
  public int getCharacteristic() 
  {
    return 2;
  }

  /**
   * returns the size of the field (2^k)
   *
   * @return field size
   */
  public int getFieldSize()
  {
    return mFieldSize;
  }

  /**
   * returns the sum of two field elements, expressed as ints
   * @param x int with bits representing one field element
   * @param y int with bits representing second field element
   * @return sum of x and y in field
   */
  public int sum(int x, int y)
  {
    int answer = 0;
    x = checkValue(x);
    y = checkValue(y);
    answer = x ^ y;

    return answer;
  }
  
  /**
   * returns the difference of two field elements, expressed as ints
   * (included for completeness' sake, since with characteristic 2 addition
   * and subtraction are the same thing).
   * @param x int with bits representing one field element
   * @param y int with bits representing second field element
   * @return difference of x and y in field
   */
  public int difference(int x, int y)
  {
    return sum(x,y);
  }

  /** 
   * returns the product of two field elements, expressed as ints
   * @param x int with bits representing one field element
   * @param y int with bits representing second field element
   * @return product of x and y in field
   * @throws IllegalArgumentException if x or y has a bit beyond the kth pos'n
   */
  public int product(int x, int y)
  {
    int answer = 0;
    int logX, logY, logAns;

    x = checkValue(x);
    y = checkValue(y);

    if ((0 != x) && (0 != y)) {
      logX = lookupPower(x);
      logY = lookupPower(y);
      logAns = (logX + logY) % ((1 << mK) - 1);
      answer = mPower[logAns];
    }

    return answer;
  }

  /**
   * returns the quotient of two field elements, expressed as ints
   * @param x int with bits representing one field element
   * @param y int with bits representing second field element
   * @return quotient of x and y in field
   * @throws ArithmeticException if y is zero
   */
  public int quotient(int x, int y)
  throws ArithmeticException
  {
    int answer = 0;
    int logX, logY, logAns;

    if (0 == y) {
      throw new ArithmeticException("Attempt to divide by 0 in GF(2^k)");
    }
 
    checkValue(x);
    checkValue(y);

    if (0 != x) {
      logX = lookupPower(x);
      logY = lookupPower(y);
      logAns = logX - logY;
      if (logAns < 0) {
	logAns += (1 << mK) - 1;
      }
      answer = mPower[logAns];
    }

    return answer;
  }

  /** 
   * returns the additive inverse of the passed field element (included for
   * completeness' sake, but with characteristic 2 each element is its own
   * negative).
   * @param x int with field element
   * @return additive inverse of x in field
   */
  public int negative(int x)
  {
    return checkValue(x);
  }

  /**
   * returns the multiplicative inverse of the passed field element
   * @param x int with field element
   * @return multiplicative inverse of x 
   * @throws ArithmeticException if x is zero
   */
  public int inverse(int x)
    throws ArithmeticException
  {
    if (0 == x) {
      throw new ArithmeticException("Attempt to divide by 0 in GF(2^k)");
    }
    x = checkValue(x);
    
    int logX = lookupPower(x);
    return mPower[(1 << mK) - 1 - logX];
  }

  /**
   * returns a field element raised to a specified power
   * 
   * @param x field element
   * @param k power
   * @return x^k in this field
   * @throws ArithmeticException if x = 0 and k < 0
   */
  public int power(int x, int k) 
    throws ArithmeticException
  {
    if ((0 == x) && (k < 0)) {
      throw new ArithmeticException("Divide by 0 in GF(2^k)");
    }

    //
    // take advantage of Fermat's Theorem if necessary to reduce exponent
    //
    int j = (k < 0) ? ((-k) % ((1 << mK)-1)) : (k % ((1 << mK)-1));
    int y = (k < 0) ? this.inverse(x) : checkValue(x);
    int answer = 1;

    for (int i=0; i<j; i++) {
      answer = this.product(answer, y);
    }

    return answer;
  }

  /**
   * returns a String describing the passed field entry in terms of a 
   * polynomial representative in GF(2)[x] / p(x) (where p(x) is the
   * primitive polynomial in use).
   * @param x int representing a field element
   * @return String with polynomial description
   */
  public String eltToString(int x)
  {
    StringBuffer answer = new StringBuffer("");
    boolean flag = false;
    int i;

    checkValue(x);
    if (0 == x) {
      answer.append("0");
    }
    else {
      for (i = mK; i >= 1; i--) {
	if (1 == (x >> i) % 2) {
	  if (flag) {
	    answer.append(" + ");
	  }
	  else {
	    flag = true;
	  }
	  answer.append("x^");
	  answer.append(String.valueOf(i));
	}
      }
      if (1 == x % 2) {
	if (flag) {
	  answer.append(" + ");
	}
	answer.append("1");
      }
    }
      
    return answer.toString();
  }

  /**
   * returns a String describing the passed field entry in terms of a
   * sum of powers of a primitive element, where the passed character
   * indicates the primitive element
   * @param x int representing field element
   * @param fmt character code representing primitive element
   * @return String with description
   */
  public String eltToString(int x, int fmt) 
  {
    StringBuffer answer = new StringBuffer("");
    boolean flag = false;
    int i;

    x = checkValue(x);
    if (0 == x) {
      answer.append("0");
    }
    else {
      for (i = mK; i >= 1; i--) {
	if (1 == (x >> i) % 2) {
	  if (flag) {
	    answer.append(" + ");
	  }
	  else {
	    flag = true;
	  }
	  answer.append((char)fmt);
	  answer.append("^");
	  answer.append(String.valueOf(i));
	}
      }
      if (1 == x % 2) {
	if (flag) {
	  answer.append(" + ");
	}
	answer.append("1");
      }
    }
    
    return answer.toString();
  }

  /**
   * returns the integer representing a field element given a String
   * representation of the element in the form returned by elt2String
   *
   * @param s String with representation of element
   * @return integer representation of element
   * @throws IllegalArgumentException if s is not in correct form
   */
  public int stringToElt(String s)
    throws IllegalArgumentException
  {
    String token;
    int ans = 0;
    int exp;

    for (StringTokenizer st = new StringTokenizer(s, "+"); 
	 st.hasMoreElements();) {
      token = st.nextElement().toString().trim();
      if (token.equals("1")) {
	ans |= 1;
      }
      else if (!(token.equals("0"))) {
	exp = getExponent(token, 'x');
	if ((exp >= 0) && (exp < mK)) {
	  ans |= (1 << exp);
	}
	else {
	  throw new IllegalArgumentException("Invalid expression: '" +
					     token + "'");
	}
      }
    }

    return ans;
  }

  /**
   * returns the integer representing a field element given a String
   * representation of the element in the form returned by elt2String with
   * an integer representing the character code to use to represent the 
   * equivalence class of x
   *
   * @param s String with representation of element
   * @param fmt character code of character to use in place of x
   * @return integer representation of element
   * @throws IllegalArgumentException if s is not in correct form
   */
  public int stringToElt(String s, int fmt)
    throws IllegalArgumentException
  {
    String token;
    int ans = 0;
    int exp;

    for (StringTokenizer st = new StringTokenizer(s, "+"); 
	 st.hasMoreElements();) {
      token = st.nextElement().toString().trim();
      if (token.equals("1")) {
	ans |= 1;
      }
      else if (!(token.equals("0"))) {
	exp = getExponent(token, (char)fmt);
	if ((exp >= 0) && (exp < mK)) {
	  ans |= (1 << exp);
	}
	else {
	  throw new IllegalArgumentException("Invalid expression: '" +
					     token + "'");
	}
      }
    }

    return ans;
  }


  private int getExponent(String s, char c)
  {
    int answer = -1;
    if (s.length() > 2) {
      if ((s.charAt(0) == c) && (s.charAt(1) == '^')) {
	try {
	  answer = (Integer.valueOf(s.substring(2))).intValue();
	}
	catch (NumberFormatException ex) { 
	  answer = -1;
	}
      }
    }
    return answer;
  }

  /**
   * returns a String with a short description of the passed field element
   * @param x integer with bits representing coefficients
   * @return String with description
   */
  public String elt2ShortString(int x)
  {
    StringBuffer answer = new StringBuffer("");
    int i;
    boolean flag=false;

    x = checkValue(x);

    if (0 == x) {
      answer.append("0");
    }
    else {
      answer.append("(");
      for (i = mK; i >= 1; i--) {
	if (1 == (x >> i) % 2) {
	  if (flag) {
	    answer.append(",");
	  }
	  else {
	    flag = true;
	  }
	  answer.append(String.valueOf(i));
	}
      }
      if (1 == x % 2) {
	if (flag) {
	  answer.append(",");
	}
	answer.append("0");
      }      
      answer.append(")");
    }

    return answer.toString();
  }

  /**
   * returns a String with the bit-representation of the passed field element
   * (basically, the number expressed in binary)
   * @param x integer representing a field element
   * @return string with binary representation of x
   */
  public String elt2BitString(int x) 
  {
    int i;
    StringBuffer answer = new StringBuffer("");

    x = checkValue(x);

    for (i = mK; i>=0; i--) {
      if (1 == (x >> i) % 2) {
	answer.append("1");
      }
      else {
	answer.append("0");
      }
    }

    return answer.toString();
  }

  /**
   * returns an integer representing a field element given a string of 
   * ones and zeroes representing coefficients as returned by elt2BitString
   * @param s String representing a field element
   * @return integer representation of element
   * @throws IllegalArgumentException if string format is incorrect or value
   *         is out of range
   */
  public int bitString2Elt(String s)
  {
    int answer = 0;
    int i, k=1, d=0;
    String ts = s.trim();
    int n = ts.length();

    k = 1;
    for (i=n-1; i>=0; i--) {
      switch (s.charAt(i)) {
      case '0':
	break;
      case '1':
	answer += k;
      default:
	throw new IllegalArgumentException("Illegal character in string: '"
					   + s.charAt(i) + "'");
      }
      k *= 2;
      d++;
      if (d > mK) {
	throw new IllegalArgumentException("Value is out of range");
      }
    }
    
    return answer;
  }
  
  /**
   * returns a String with a representation of a field element either as 
   * a 0 or 1 or as a power of the primitive element in the form (k), where
   * the passed number represents w^k, where w is the primitive element
   * @param x integer representation of a field element
   * @return string with representation of x
   * @throws IllegalArgumentException if x doesn't represent a field element
   */
  public String eltToPowerString(int x)
    throws IllegalArgumentException
  {
    int logX;
    StringBuffer answer = new StringBuffer("");

    if (0 == x) {
      answer.append("0");
    }
    else if (1 == x) {
      answer.append("1");
    }
    else {
      checkValue(x);
      logX = lookupPower(x);
      answer.append("(");
      answer.append(String.valueOf(logX));
      answer.append(")");
    }
    return answer.toString();
  }

  /**
   * returns an integer representing a field element, given its representation
   * in the form expressed in eltToPowerString; that is, either 0 or (j), where
   * the field element is w^j, where w generates the group of units of the
   * field. If j is less than 0 or greater than 2^k-2 (where the field is
   * of size 2^k), it is reduced modulo 2^k-1.
   * @param s String with representation of field element
   * @return integer representation of field element
   * @throws NumberFormatException if the string is not in the correct fmt
   */
  public int powerStringToElt(String s)
    throws NumberFormatException
  {
    int answer = 0;
    String ts = s.trim();
    String ns;
    int n = ts.length();
    int nUnits = (1 << mK) - 1;
    int p;

    if (ts.equals("0")) {
      answer = 0;
    }
    else if (ts.equals("1")) {
      answer = 1;
    }
    else {
      if (n < 2) {
	throw new NumberFormatException("Non-zero or one values must be in the"
					+ " form (k)");
      }

      if ((ts.charAt(0) != '(') || (ts.charAt(n-1) != ')')) {
	throw new NumberFormatException("Non-zero or one values must be"
					+ " surrounded by parentheses");
      }

      ns = ts.substring(1,n-1);

      p = Integer.valueOf(ns).intValue();
      if (p % nUnits == 0) {
	p = 0;
      }
      else if (p < 0) {
	p += nUnits * (-p / nUnits + 1);
      }
      else {
	p = p % nUnits;
      }
      answer = mPower[p];
    }
    return answer;
  }

  //
  // private sub initializes lookup arrays
  //
  private void initArrays()
  {
    findPolynomial();

    int n = (1 << mK);
    mFieldSize = n;
    int i;
    mPower = new int[n];

    mPower[0] = 1;
    for (i=1; i<n; i++) {
      mPower[i] = mPower[i-1] << 1;
      if ((mPower[i] & n) != 0) {
	mPower[i] ^= mPolynomial;
      }
    }

    if (mK < 31) {
      mMask = (1 << mK) - 1;
    }
    else {
      mMask = 0x7fffffff;
    }
  }

  //
  // private sub finds a primitive polynomial -- several are hardcoded, 
  // but it will do a search if it is not one of those cases
  //
  private void findPolynomial()
  {
    int k=0;
    switch (mK) {
    case 2: // x^2 + x + 1
      k += 1;
      k += 1 << 1;
      k += 1 << 2;
      break;
    case 3:
    case 4:
    case 6:
    case 15: // x^k + x + 1
      k += 1;
      k += 1 << 1;
      k += 1 << mK;
      break;      
    case 5: // x^5 + x^2 + 1
      k += 1;
      k += 1 << 2;
      k += 1 << 5;
      break;
    case 7: // x^7 + x + 1
      k += 1;
      k += 1 << 1;
      k += 1 << 7;
      break;
    case 8: // x^8 + x^4 + x^3 + x^2 + 1
      k += 1;
      k += 1 << 2;
      k += 1 << 3;
      k += 1 << 4;
      k += 1 << 8;
      break;
    case 9: // x^9 + x^4 + 1
      k += 1;
      k += 1 << 4;
      k += 1 << 9;
      break;
    case 10: // x^10 + x^3 + 1
      k += 1;
      k += 1 << 3;
      k += 1 << 10;
      break;
    case 11: // x^11 + x^2 + 1
      k += 1 << 11;
      k += 1 << 2;
      k += 1;
      break;
    case 12: // x^12 + x^6 + x^4 + x + 1
      k += 1 << 12;
      k += 1 << 6;
      k += 1 << 4;
      k += 1 << 1;
      k += 1;
      break;
    case 13: // x^13 + x^4 + x^3 + x + 1
      k += 1 << 13;
      k += 1 << 4;
      k += 1 << 3;
      k += 1 << 1;
      k += 1;
      break;
    case 14: // x^14 + x^5 + x^3 + x + 1
      k += 1 << 14;
      k += 1 << 5;
      k += 1 << 3;
      k += 1 << 1;
      k += 1;
      break;
    default:
      k = seekPolynomial();
    }   
    mPolynomial = k;
  }

  //
  // private function seeks polynomial in non-hard-coded cases
  //
  private int seekPolynomial()
  {
    int answer=0;
    int poly=0;
    int step=0;
    int power=1;
    boolean found=false;
    int first, last;
    int check;
    int i,k,n=mK;

    first = (1 << n);
    last = (1 << (n+1));

    /* main loop */
    for (poly = first+1; (poly < last) && !found; poly++) {
      step = 0;
      power = 1;

      do {
	/* step register */
	power = power << 1;
	if (1 == (power >> n) % 2) {
	  power = power ^ poly;
	}
	step++;
      } while ((1 != power) && (step < first));

      if (step == first-1) {
	found = true;
	for (i=n; i>=0; i--) {
	  if (1 == (poly >> i) % 2) {
	    answer += 1 << i;
	  }
	}
      }
    }

    if (!found) {
      throw new InternalError("Primitive polynomial search failed");
    }

    return answer;
  }

  //
  // private method converts passed value to a valid representation 
  // of a field element
  //
  private int checkValue(int b)
  {
    return b & mMask;
  }

  //
  // private function looks up the power of an element -- this is 
  // very non-optimal
  //
  private int lookupPower(int b)
  {
    int answer = -1;
    int i;
    int n = 1 << mK;

    for (i=0; i<n; i++) {
      if (mPower[i] == b) {
	answer = i;
	break;
      }
    }
    
    if (answer < 0) {
      throw new InternalError("Power not found");
    }

    return answer;
  }
}
