package Oid;
/**
 * An instance of <code>IntPoly</code> stores a polynomial in one variable 
 * with integer coefficients.  It can do basic polynomial operations (add, 
 * subtract, multiply with other instances and with integers) and can evaluate 
 * itself at a value.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 15 Jun 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class IntPoly
{
  //
  // private data members
  //
  private int mDegree;
  private int[] mCoeff;

  /** 
   * the default constructor creates the zero polynomial
   */
  public IntPoly()
  {
    mDegree = -1;
  }

  /**
   * the copy constructor copies the coefficients from another 
   * <code>IntPoly</code> instance
   * @param poly another <code>IntPoly</code> instance
   */
  public IntPoly(IntPoly poly)
  {
    mDegree = poly.getDegree();    
    
    if (mDegree >= 0) {
      int[] newCoeff = poly.getCoefficientArray();
      int i;
      
      mCoeff = new int[mDegree+1];
      for (i=0; i<=mDegree; i++) {
	mCoeff[i] = newCoeff[i];
      }
    }
  }

  /**
   * constructs an instance from an array of integers with the 
   * coefficients
   * @param coeff array of coefficients
   */
  public IntPoly(int[] coeff)
  {

    int n = coeff.length-1;
    int i;

    // find last nonzero coefficient
    for (i=n; i>=0; i--) {
      if (coeff[i] != 0) {
	break;
      }
    }

    mDegree = i;

    if (mDegree >= 0) {
      mCoeff = new int[mDegree+1];
      for (i=0; i<=mDegree; i++) {
	mCoeff[i] = coeff[i];
      }
    }
  }

  /** 
   * returns this polynomial's array of coefficients
   * @return integer array of coefficients
   */
  public int[] getCoefficientArray()
  {
    return mCoeff;
  }

  /**
   * returns this polynomial's degree
   * @return integer with degree
   */
  public int getDegree()
  {
    return mDegree;
  }

  /**
   * returns the coefficient of x to the specified power
   * @param k exponent
   * @return coefficient of x^k
   */
  public int getCoefficient(int k) 
  {
    int answer = 0;
    if ((k >= 0) && (k <= mDegree)) {
      answer = mCoeff[k];
    }
    return answer;
  }

  /**
   * returns a <code>String</code> expression of this polynomial using the 
   * passed character as the variable
   *
   * @param v character to use as the variable
   * @return polynomial in <code>String</code> form
   */
  public String toString(char v)
  {
    StringBuffer answer = new StringBuffer();
    boolean first=true;

    if (mDegree < 0) {
      answer.append("0");
    }
    else {
      for (int i=mDegree; i>=0; i--) {
	if (i > 0) {
	  if (mCoeff[i] != 0) {
	    if (mCoeff[i] > 0) {
	      if (!first) {
		answer.append(" + ");
	      }
	      if (mCoeff[i] > 1) {
		answer.append(mCoeff[i]);
	      }
	    }
	    else {
	      answer.append(" - ");
	      if (mCoeff[i] < -1) {
		answer.append(-mCoeff[i]);
	      }
	    }
	    answer.append(v);
	    if (i > 1) {
	      answer.append("^");
	      answer.append(i);
	    }
	    first = false;
	  }
	}
	else {
	  if (mCoeff[0] > 0) {
	    if (!first) {
	      answer.append(" + ");
	    }
	    answer.append(mCoeff[0]);
	  }
	  else {
	    answer.append(" - ");
	    answer.append(-mCoeff[0]);
	  }
	  first = false;
	}
      }
    }
    return answer.toString();
  }

  /**
   * returns a <code>String</code> expression of this polynomial in terms of x
   * @return <code>String</code> with polynomial
   */
  public String toString() 
  {
    return this.toString('x');
  }

  /**
   * evaluates this polynomial at an integer value and returns the result
   * @param x integer value to use
   * @return value of this polynomial evaluated at x
   */
  public int evaluate(int x) 
  {
    int answer = 0;
    int i;
    int p = 1;

    for (i=0; i<=mDegree; i++) {
      answer += mCoeff[i] * p;
      p *= x;
    }
    return answer;
  }

  /**
   * substitutes another polynomial for the variable in this one and 
   * returns the result
   * @param poly polynomial to substitute
   * @return resulting polynomial
   */
  public IntPoly substitute(IntPoly poly) 
  {
    IntPoly answer = new IntPoly();
    IntPoly p = new IntPoly();
    IntPoly q;
    int i;

    p.add(1);
    for (i=0; i<=mDegree; i++) {
      answer.add(p, mCoeff[i]);
      p.multiply(poly);
    }
    return answer;
  }

  /** 
   * adds another polynomial, times an integer constant to this one
   * @param poly another IntPoly instance
   * @param c integer constant
   */
  public void add(IntPoly poly, int c)
  {
    int degree = poly.getDegree();
    int[] coeff = poly.getCoefficientArray();
    int i;

    if ((c != 0) && (degree >= 0)) {
      if (degree > mDegree) {
	// make me a bigger coefficient array
	int[] newCoeff = new int[degree+1];
	for (i=0; i<=mDegree; i++) {
	  newCoeff[i] = mCoeff[i];
	}
	mCoeff = (int[])null;
	mCoeff = newCoeff;
	mDegree = degree;
      }
      
      // now add 'em up
      for (i=0; i<=mDegree; i++) {
	mCoeff[i] = (i > degree) ? mCoeff[i] : (mCoeff[i] + c * coeff[i]);
      }
      
      // check whether leading coefficient(s) were cancelled out
      adjustDegree();
    }
  }

  /** 
   * adds an integer to this polynomial
   * @param c integer to add
   */
  public void add(int c) 
  {
    if (c != 0) {
      if (mDegree <= 0) {
	mDegree = 0;
	mCoeff = new int[1];
	mCoeff[0] = c;
      }
      else {
	mCoeff[0] += c;
	// what if this is a constant and we just cancelled the constant term?
	if ((mDegree == 0) && (mCoeff[0] == 0)) {
	  mDegree = -1;
	}
      }
    }
  }

  /**
   * adds another polynomial to this one
   * @param poly another <code>IntPoly</code> instance
   */
  public void add(IntPoly poly) 
  {
    this.add(poly, 1);
  }

  /**
   * subtracts another polynomial from this one
   * @param poly the other polynomial
   */
  public void subtract(IntPoly poly)
  {
    this.add(poly, -1);
  }

  /**
   * subtracts an integer constant from this polynomial (for completness' sake)
   * @param c constant
   */
  public void subtract(int c) 
  {
    this.add(-c);
  }

  /**
   * multiplies this polynomial by an integer constant
   * @param c constant
   */
  public void multiply(int c)
  {
    if (c == 0) {
      mCoeff = (int[])null;
      mDegree = -1;
    }
    else {
      for (int i=0; i<=mDegree; i++) {
	mCoeff[i] *= c;
      }
    }
  }

  /**
   * multiplies this polynomial by another one
   * @param poly another <code>IntPoly</code>
   */
  public void multiply(IntPoly poly)
  {
    int degree = poly.getDegree();
    int[] coeff = poly.getCoefficientArray();

    if (degree < 0) {
      mDegree = -1;
      mCoeff = (int[])null;
    }
    else if (degree == 0) {
      this.multiply(coeff[0]);
    }
    else if (mDegree >= 0) {
      int newDegree = mDegree + degree;
      int[] newCoeff = new int[newDegree+1];
      int i,j;

      for (i=0; i<=newDegree; i++) {
	newCoeff[i] = 0;
      }

      for (i=0; i<=mDegree; i++) {
	for (j=0; j<=degree; j++) {
	  newCoeff[i+j] += mCoeff[i] * coeff[j];
	}
      }

      mDegree = newDegree;
      mCoeff = (int[])null;
      mCoeff = newCoeff;
    }
  }

  /**
   * checks this polynomial for equality with another one (or with an integer)
   * @param obj object to compare
   */
  public boolean equals(Object obj)
  {
    boolean answer = false;

    if (obj instanceof IntPoly) {
      IntPoly poly = (IntPoly)obj;
      int d = poly.getDegree();
      int[] coeff = poly.getCoefficientArray();
      int i;

      answer = (d == mDegree);

      if (answer) {
	for (i=0; i<=mDegree; i++) {
	  if (coeff[i] != mCoeff[i]) {
	    answer = false;
	    break;
	  }
	}
      }
    }
    else if (obj instanceof Integer) {
      answer = (mDegree == 0) && (mCoeff[0] == ((Integer)obj).intValue());
    }
    
    return answer;
  }

  //
  // private method adjusts the degree in case the leading coefficient is 0
  //
  private void adjustDegree()
  {
    int i;

    // find last nonzero coefficient
    for (i=mDegree; i>=0; i--) {
      if (mCoeff[i] != 0) {
	break;
      }
    }

    mDegree = i;
  }
}
