package Oid;
/**
 * An instance of <code>IntPoly2</code> stores a polynomial in two
 * variables with integer coefficients.  It can do basic polynomial
 * operations (add, subtract and multiply with integers, other
 * <code>IntPoly2</code> instances and <code>IntPoly</code> instances)
 * and can evaluate itself at a point, or at a value to produce a
 * polynomial in one variable.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 16 June 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class IntPoly2
{
  // 
  // private data members
  // 
  private int mDegree1;
  private int mDegree2;
  private int[][] mCoeff;
  
  /**
   * the default constructor creates the zero polynomial
   */
  public IntPoly2()
  {
    mDegree1 = -1;
    mDegree2 = -1;
  }

  /**
   * constructs an <code>IntPoly2</code> instance from a
   * two-dimensional array of integers containing the coefficients
   *
   * @param coeff array of coefficients
   */
  public IntPoly2(int[][] coeff)
  {
    int m = coeff.length;
    int n = coeff[0].length;
    int i, j;
    boolean found;

    // find real limits
    found = false;
    for (i=m-1; i>=0; i--) {
      for (j=0; j<n; j++) {
	if (coeff[i][j] != 0) {
	  found = true;
	  break;
	}
      }
      if (found) {
	m = i;
	break;
      }
    }

    if (m >= 0) {
      found = false;
      for (j=n-1; j>=0; j--) {
	for (i=0; i<=m; i++) {
	  if (coeff[i][j] != 0) {
	    found = true;
	    break;
	  }
	}
	if (found) {
	  n = j;
	  break;
	}
      }
    }

    mDegree1 = m;
    mDegree2 = n;

    if ((mDegree1 >= 0) && (mDegree2 >= 0)) {
      mCoeff = new int[mDegree1+1][mDegree2+1];
      for (i=0; i<=mDegree1; i++) {
	for (j=0; j<=mDegree2; j++) {
	  mCoeff[i][j] = coeff[i][j];
	}
      }
    }
  }

  /**
   * the copy constructor copies the coefficients from another
   * <code>IntPoly2</code> instance
   *
   * @param poly an existing <code>IntPoly2</code> instance to copy
   */
  public IntPoly2(IntPoly2 poly)
  {
    mDegree1 = poly.getDegree1();
    mDegree2 = poly.getDegree2();

    if ((mDegree1 >= 0) && (mDegree2 >= 0)) {
      int[][] coeff = poly.getCoefficientArray();
      int i,j;
      mCoeff = new int[mDegree1+1][mDegree2+1];

      for (i=0; i<=mDegree1; i++) {
	for (j=0; j<=mDegree2; j++) {
	  mCoeff[i][j] = coeff[i][j];
	}
      }
    }
  }

  /**
   * checks this polynomial for equality with another
   * <code>IntPoly2</code> or an integer
   *
   * @param obj object to check
   * @return boolean evaluation of equality
   */
  public boolean equals(Object obj)
  {
    boolean answer = false;
    IntPoly2 poly2;
    IntPoly poly;
    int i,j;

    if (obj instanceof IntPoly2) {
      poly2 = (IntPoly2)obj;
      
      if ((poly2.getDegree1() == mDegree1) && 
	  (poly2.getDegree2() == mDegree2)) {
	int[][] coeff = poly2.getCoefficientArray();
	answer = true;
	for (i=0; i<=mDegree1; i++) {
	  for (j=0; j<=mDegree2; i++) {
	    if (mCoeff[i][j] != coeff[i][j]) {
	      answer = false;
	      break;
	    }
	  }
	  if (!answer) {
	    break;
	  }
	}
      }
    }
    else if (obj instanceof IntPoly) {
      // obj could be equal to this polynomial in one variable if this
      // polynomial's degree is 0 in the other variable

      poly = (IntPoly)obj;
      int[] coeff2 = poly.getCoefficientArray();

      if ((poly.getDegree() == mDegree1) && (0 == mDegree2)) {
	answer = true;
	for (i=0; i<=mDegree1; i++) {
	  if (coeff2[i] != mCoeff[i][0]) {
	    answer = false;
	    break;
	  }
	}
      }
      else if ((poly.getDegree() == mDegree2) && (0 == mDegree1)) {
	answer = true;
	for (i=0; i<=mDegree2; i++) {
	  if (coeff2[i] != mCoeff[0][i]) {
	    answer = false;
	    break;
	  }
	}
      }
    }
    else if (obj instanceof Integer) {
      if ((0 == mDegree1) && (0 == mDegree2)) {
	answer = (mCoeff[0][0] == ((Integer)obj).intValue());
      }
    }

    return answer;
  }

  /**
   * returns the coefficient of x^i * y^j in the polynomial
   *
   * @param i power of first variable to seek
   * @param j power of second variable to seek
   * @return coefficient of x^i * y^j
   */
  public int getCoefficient(int i, int j)
  {
    int answer = 0;

    if ((i >= 0) && (i <= mDegree1) && (j >= 0) && (j <= mDegree2)) {
      answer = mCoeff[i][j];
    } 

    return answer;
  }

  /**
   * returns the two-dimensional coefficient array for this polynomial
   *
   * @return two-dimensional array of integers
   */
  public int[][] getCoefficientArray()
  {
    return mCoeff;
  }

  /**
   * returns the degree of this polynomial in its first variable
   *
   * @return degree as an integer (-1 for the zero polynomial)
   */
  public int getDegree1()
  {
    return mDegree1;
  }

  /**
   * returns the degree of this polynomial in its second variable
   *
   * @return degree as an integer (-1 for the zero polynomial)
   */
  public int getDegree2()
  {
    return mDegree2;
  }

  /**
   * returns a string expression of this polynomial using the specified
   * characters to represent the variables
   *
   * @param v character for first variable
   * @param w character for second variable
   * @return string expression in terms of v and w
   */
  public String toString(char v, char w) 
  {
    StringBuffer answer = new StringBuffer();
    boolean first = true;
    int i,j;

    if ((mDegree1 < 0) && (mDegree2 < 0)) {
      answer.append("0");
    }
    else {
      for (i=mDegree1; i>=0; i--) {
	for (j=mDegree2; j>=0; j--) {
	  if (mCoeff[i][j] != 0) {
	    // leading sign
	    if (mCoeff[i][j] < 0) {
	      answer.append(" - ");
	      if (((i == 0) && (j == 0)) || (mCoeff[i][j] < -1)) {
		answer.append(-mCoeff[i][j]);
	      }
	    }
	    else {
	      if (!first) {
		answer.append(" + ");
	      }
	      if (((i == 0) && (j == 0)) || (mCoeff[i][j] > 1)) {
		answer.append(mCoeff[i][j]);
	      }
	    }

	    // first variable
	    if (i > 0) {
	      answer.append(v);
	      if (i > 1) {
		answer.append("^");
		answer.append(i);
	      }
	    }

	    // second variable
	    if (j > 0) {
	      answer.append(w);
	      if (j > 1) {
		answer.append("^");
		answer.append(j);
	      }
	    }

	    first = false;
	  }
	}
      }
    }

    return answer.toString();
  }

  /**
   * returns a string expression of this polynomial in terms of x and y
   *
   * @return string expression
   */
  public String toString()
  {
    return this.toString('x','y');
  }

  /**
   * adds an integer to the constant term of this polynomial
   *
   * @param c integer to add
   */
  public void add(int c)
  {
    if (c != 0) {
      if ((mDegree1 >= 0) && (mDegree2 >= 0)) {
	mCoeff[0][0] += c;
	if ((0 == mDegree1) && (0 == mDegree2) && (0 == mCoeff[0][0])) {
	  mDegree1 = -1;
	  mDegree2 = -1;
	}
      }
      else {
	expandDegrees(0, 0);
	mCoeff[0][0] = c;
      }
    }
  }

  /**
   * adds an <code>IntPoly</code>in this polynomial's first variable
   * to this polynomial
   *
   * @param poly <code>IntPoly</code>instance
   */
  public void add(IntPoly poly)
  {
    this.add(poly, 1);
  }

  /**
   * adds an <code>IntPoly</code>in this polynomial's second variable
   * to this polynomial
   *
   * @param poly <code>IntPoly</code>instance
   */
  public void add2(IntPoly poly)
  {
    this.add2(poly, 1);
  }

  /**
   * adds another <code>IntPoly2</code> to this one
   *
   * @param poly <code>IntPoly2</code> instance
   */
  public void add(IntPoly2 poly)
  {
    this.add(poly, 1);
  }

  /**
   * adds an <code>IntPoly</code>in this polynomial's first variable,
   * times a constant, to this polynomial
   *
   * @param poly <code>IntPoly</code>instance
   * @param c constant
   */
  public void add(IntPoly poly, int c)
  {
    if (0 != c) {
      int n = poly.getDegree();
      int i;
      int[] coeff = poly.getCoefficientArray();
      
      if (n > mDegree1) {
	expandDegrees(n, mDegree2);
      }
      
      for (i=0; i<=n; i++) {
	mCoeff[i][0] += c * coeff[i];
      }
      
      // leading term(s) may be cancelled out
      adjustDegrees();
    }
  }

  /**
   * adds an <code>IntPoly</code>in this polynomial's second variable,
   * times a constant, to this polynomial
   *
   * @param poly <code>IntPoly</code>instance
   * @param c constant
   */
  public void add2(IntPoly poly, int c)
  {
    if (0 != c) {
      int n = poly.getDegree();
      int i;
      int[] coeff = poly.getCoefficientArray();
      
      if (n > mDegree2) {
	expandDegrees(mDegree1, n);
      }
      
      for (i=0; i<=n; i++) {
	mCoeff[0][i] += c * coeff[i];
      }
      
      // leading term(s) may be cancelled out
      adjustDegrees();
    }
  }

  /**
   * adds another <code>IntPoly2</code> instance, times a constant, to this one
   *
   * @param poly <code>IntPoly2</code> instance
   * @param c constant
   */
  public void add(IntPoly2 poly, int c)
  {
    if (0 != c) {
      int m = poly.getDegree1();
      int n = poly.getDegree2();
      int i,j;
      int[][] coeff = poly.getCoefficientArray();

      if ((m > mDegree1) || (n > mDegree2)) {
	expandDegrees(m, n);
      }

      for (i=0; i<=m; i++) {
	for (j=0; j<=n; j++) {
	  mCoeff[i][j] += c * coeff[i][j];
	}
      }

      // leading term(s) may be cancelled out
      adjustDegrees();
    }
  }

  /**
   * subtracts an integer from the constant term of this polynomial
   *
   * @param c integer to subtract
   */
  public void subtract(int c)
  {
    this.add(-c);
  }

  /**
   * subtracts an <code>IntPoly</code>in this polynomial's first
   * variable from this polynomial
   *
   * @param poly <code>IntPoly</code>instance
   */
  public void subtract(IntPoly poly)
  {
    this.add(poly, -1);
  }

  /**
   * subtracts an <code>IntPoly</code>in this polynomial's second
   * variable from this polynomial
   *
   * @param poly <code>IntPoly</code>instance
   */
  public void subtract2(IntPoly poly)
  {
    this.add2(poly, -1);
  }

  /**
   * subtracts another <code>IntPoly2</code> from this one
   *
   * @param poly <code>IntPoly2</code> instance
   */
  public void subtract(IntPoly2 poly)
  {
    this.add(poly, -1);
  }

  /**
   * multiplies this polynomial by an integer constant
   *
   * @param c constant
   */
  public void multiply(int c)
  {
    if (0 == c) {
      mDegree1 = -1;
      mDegree2 = -1;
      mCoeff = (int[][])null;
    }
    else if ((mDegree1 >= 0) && (mDegree2 >= 0)) {
      int i,j;
      for (i=0; i<=mDegree1; i++) {
	for (j=0; j<=mDegree2; j++) {
	  mCoeff[i][j] *= c;
	}
      }
    }
  }

  /**
   * multiplies this polynomial by an <code>IntPoly</code>in its first variable
   *
   * @param poly <code>IntPoly</code>instance
   */
  public void multiply(IntPoly poly)
  {
    int n = poly.getDegree();
    int[] coeff = poly.getCoefficientArray();
    int newDeg1;
    int i,j,k;

    if (n == 0) {
      // multiply by a constant only
      this.multiply(coeff[0]);
    }
    else {
      newDeg1 = mDegree1 + n;
      int[][] newCoeff = new int[newDeg1+1][mDegree2+1];

      for (i=0; i<=newDeg1; i++) {
	for (j=0; j<=mDegree2; j++) {
	  newCoeff[i][j] = 0;
	}
      }

      for (i=0; i<=mDegree1; i++) {
	for (j=0; j<=mDegree2; j++) {
	  for (k=0; k<=n; k++) {
	    newCoeff[i+k][j] += mCoeff[i][j] * coeff[k];
	  }
	}
      }
      mDegree1 = newDeg1;
      mCoeff = newCoeff;
    }
  }

  /**
   * multiplies this polynomial by an <code>IntPoly</code>in its
   * second variable
   *
   * @param poly <code>IntPoly</code>instance
   */
  public void multiply2(IntPoly poly)
  {
    int n = poly.getDegree();
    int[] coeff = poly.getCoefficientArray();
    int newDeg2;
    int i,j,k;

    if (n < 0) {
      mDegree1 = -1;
      mDegree2 = -1;
      mCoeff = (int[][])null;
    }
    else if (n == 0) {
      // multiply by a constant only
      this.multiply(coeff[0]);
    }
    else {
      newDeg2 = mDegree2 + n;
      int[][] newCoeff = new int[mDegree1+1][newDeg2+1];

      for (i=0; i<=mDegree1; i++) {
	for (j=0; j<=newDeg2; j++) {
	  newCoeff[i][j] = 0;
	}
      }

      for (i=0; i<=mDegree1; i++) {
	for (j=0; j<=mDegree2; j++) {
	  for (k=0; k<=n; k++) {
	    newCoeff[i][j+k] += mCoeff[i][j] * coeff[k];
	  }
	}
      }
      mDegree2 = newDeg2;
      mCoeff = newCoeff;
    }
  }

  /**
   * multiplies this polynomial by another polynomial in two variables
   *
   * @param poly <code>IntPoly2</code> instance
   */
  public void multiply(IntPoly2 poly)
  {
    int m = poly.getDegree1();
    int n = poly.getDegree2();
    int[][] coeff = poly.getCoefficientArray();
    int newDeg1, newDeg2;
    int i,j,k,l;

    if ((m < 0) || (n < 0)) {
      mDegree1 = -1;
      mDegree2 = -1;
      mCoeff = (int[][])null;
    }
    else {
      newDeg1 = mDegree1 + m;
      newDeg2 = mDegree2 + n;
      int[][] newCoeff = new int[newDeg1+1][newDeg2+1];

      for (i=0; i<=newDeg1; i++) {
	for (j=0; j<=newDeg2; j++) {
	  newCoeff[i][j] = 0;
	}
      }
     
      for (i=0; i<=mDegree1; i++) {
	for (j=0; j<=mDegree2; j++) {
	  for (k=0; k<=m; k++) {
	    for (l=0; l<=n; l++) {
	      newCoeff[i+k][j+l] += mCoeff[i][j] * coeff[k][l];
	    }
	  }
	}
      }

      mDegree1 = newDeg1;
      mDegree2 = newDeg2;
      mCoeff = newCoeff;
    }
  }

  /**
   * evaluates this polynomial at an integer pair and returns the result
   *
   * @param x first part of pair
   * @param y second part of pair
   * @return evaluation of polynomial at (x,y)
   */
  public int evaluate(int x, int y)
  {
    int answer = 0;
    int px = 1, py;
    int i,j;

    for (i=0; i<=mDegree1; i++) {
      py = 1;
      for (j=0; j<=mDegree2; j++) {
	answer += mCoeff[i][j] * px * py;
	py *= y;
      }
      px *= x;
    }

    return answer;
  }

  /**
   * evaluates this polynomial by substituting the passed value for the 
   * first variable, and returns the resulting polynomial in one variable
   *
   * @param x value for first variable
   * @return <code>IntPoly</code>reference with resulting polynomial
   */
  public IntPoly evaluate1(int x)
  {
    int[] coeff = new int[mDegree2+1];
    int i,j;
    int px=1;

    for (i=0; i<=mDegree2; i++) {
      coeff[i] = 0;
    }

    for (i=0; i<=mDegree1; i++) {
      for (j=0; j<=mDegree2; j++) {
	coeff[j] += mCoeff[i][j] * px;
      }
      px *= x;
    }

    return new IntPoly(coeff);
  }

  /**
   * evaluates this polynomial by substituting the passed value for the
   * second variable, and returns the resulting polynomial in one variable
   *
   * @param y value for second variable
   * @return <code>IntPoly</code>reference with resulting polynomial
   */
  public IntPoly evaluate2(int y) 
  {
    int[] coeff = new int[mDegree1+1];
    int i,j;
    int py = 1;

    for (i=0; i<mDegree1; i++) {
      coeff[i] = 0;
    }

    for (j=0; j<=mDegree2; j++) {
      for (i=0; i<=mDegree1; i++) {
	coeff[i] += mCoeff[i][j] * py;
      }
      py *= y;
    }

    return new IntPoly(coeff);
  }

  /**
   * substitutes the passed polynomial in one variable for the first variable
   * of this polynomial, and returns the result
   *
   * @param poly polynomial to substitute
   * @return resulting polynomial
   */
  public IntPoly2 substitute1(IntPoly poly) 
  {
    IntPoly2 answer = new IntPoly2();
    IntPoly px = new IntPoly();
    IntPoly2 one = new IntPoly2();
    IntPoly2 py;
    IntPoly2 py2;
    int[][] yCoeff = new int[][] {{0, 1}};
    IntPoly2 y = new IntPoly2(yCoeff);
    int i,j;

    px.add(1);
    one.add(1);
    
    for (i=0; i<=mDegree1; i++) {
      py = new IntPoly2(one);
      for (j=0; j<=mDegree2; j++) {
	if (mCoeff[i][j] != 0) {
	  py2 = new IntPoly2(py);
	  py2.multiply(px);
	  py2.multiply(mCoeff[i][j]);
	  answer.add(py2);
	}
	py.multiply(y);
      }
      px.multiply(poly);
    }
    return answer;
  }

  /**
   * substitutes the passed polynomial in one variable for the second variable
   * of this polynomial, and returns the result
   *
   * @param poly polynomial to substitute
   * @return resulting polynomial
   */
  public IntPoly2 substitute2(IntPoly poly)
  {
    IntPoly2 answer = new IntPoly2();
    IntPoly py = new IntPoly();
    IntPoly2 one = new IntPoly2();
    IntPoly2 px;
    IntPoly2 px2;
    int[][] xCoeff = new int[][] {{0}, {1}};
    IntPoly2 x = new IntPoly2(xCoeff);
    int i,j;

    py.add(1);
    one.add(1);
    
    for (i=0; i<=mDegree2; i++) {
      px = new IntPoly2(one);
      for (j=0; j<=mDegree1; j++) {
	if (mCoeff[j][i] != 0) {
	  px2 = new IntPoly2(px);
	  px2.multiply2(py);
	  px2.multiply(mCoeff[j][i]);
	  answer.add(px2);
	}
	px.multiply(x);
      }
      py.multiply(poly);
    }
    return answer;
  }

  // 
  // private method adjusts the degrees of the polynomial in case 
  // coefficients have been cancelled
  //
  private void adjustDegrees()
  {
    int i,j;
    boolean found;

    // check first degree
    found = false;
    for (i=mDegree1; i>=0; i--) {
      for (j=0; j<=mDegree2; j++) {
	if (mCoeff[i][j] != 0) {
	  found = true;
	  mDegree1 = i;
	  break;
	}
      }
      if (found) {
	break;
      }
    }
    
    // check second degree
    found = false;
    for (j=mDegree2; j>=0; j--) {
      for (i=0; i<=mDegree1; i++) {
	if (mCoeff[i][j] != 0) {
	  found = true;
	  mDegree2 = j;
	  break;
	}
      }
      if (found) {
	break;
      }
    }

    if ((mDegree1 < 0) && (mDegree2 < 0)) {
      mCoeff = (int[][])null;
    }
  }

  //
  // private method ensures degrees are at least as large as specified
  //
  private void expandDegrees(int deg1, int deg2) {
    if ((deg1 > mDegree1) || (deg2 > mDegree2)) {
      int d1 = (deg1 > mDegree1) ? deg1 : mDegree1;
      int d2 = (deg2 > mDegree2) ? deg2 : mDegree2;
      int[][] newCoeff = new int[d1+1][d2+1];
      int i, j;

      for (i=0; i<=d1; i++) {
	for (j=0; j<=d2; j++) {
	  newCoeff[i][j] = 0;
	}
      }

      for (i=0; i<=mDegree1; i++) {
	for (j=0; j<=mDegree2; j++) {
	  newCoeff[i][j] = mCoeff[i][j];
	}
      }

      mCoeff = newCoeff;
      mDegree1 = d1;
      mDegree2 = d2;
    }
  }
}
