package Oid;
import java.util.*;
import java.io.*;

/** 
 * MatrixFileReader reads matrices from an ASCII file with a specific
 * format: for each matrix, the file lists the matroid's: 
 * base field modulus 
 * size 
 * rank 
 * matrix 
 *
 * It generates FiniteFieldMatroids from these matrices and returns
 * them using the Enumeration interface.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 21 Jul 2002
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class MatrixFileReader implements Enumeration
{
  //
  // private data members
  //
  private Vector mFileTokens;
  private boolean mHasMoreElements;
  private int mCurElement;
  private Vector mElements;
  private static final int BUF_SIZE = 16384;
  
  /** 
   * constructor opens and reads the file, and prepares the first matroid
   * @param filename String with name of file containing matrices
   * @throws FileNotFoundException if unable to open the file
   * @throws IOException if unable to read/format the file
   */
  public MatrixFileReader(String filename)
    throws FileNotFoundException, IOException
  {
    readFile(filename, false);
  }

  public MatrixFileReader(String filename, boolean useNames)
    throws FileNotFoundException, IOException
  {
    readFile(filename, useNames);
  }

  private void readFile(String filename, boolean useNames)
    throws FileNotFoundException, IOException
  {
    // open and read the file
    FileReader fr = new FileReader(filename);
    char[] buffer = new char[BUF_SIZE];
    StringBuffer contents = new StringBuffer();
    int bytesRead;

    bytesRead = fr.read(buffer, 0, BUF_SIZE);
    while (bytesRead >= 0) {
      contents.append(buffer, 0, bytesRead);
      bytesRead = fr.read(buffer, 0, BUF_SIZE);
    }
    fr.close();

    // now tokenize the contents of the file by lines, then by white space
    String sContents = contents.toString();
    String token;
    sContents = sContents.replace((char)(0x0d), ' ');
    StringTokenizer strToken = new StringTokenizer(sContents, "\n");
    StringTokenizer linToken;

    Vector tokens = new Vector();
    Vector lineNum = new Vector();
    int lnum = 1;
    
    for (; strToken.hasMoreElements(); ) {
      token = strToken.nextElement().toString();
      if (token.length() > 0) {
	linToken = new StringTokenizer(token);
	for (; linToken.hasMoreElements(); ) {
	  tokens.add(linToken.nextElement().toString());
	  lineNum.add(new Integer(lnum));
	}
      }
      lnum++;
    }

    // now make matrices and matroids
    mElements = new Vector();
    Iterator pToken = tokens.iterator();
    Iterator pLNum = lineNum.iterator();
    FiniteField calc;
    int i,j,p,r,n;
    String matroidName = new String("");
    int[][] matrix;
    FiniteFieldVector[] col;
    FiniteFieldMatroid matroid;
    token = new String("");

    try {
      calc = FiniteFieldFactory.createFiniteField(2);
      while (pToken.hasNext()) {
	// get name if included
	if (useNames) {
	  lnum = ((Integer)(pLNum.next())).intValue();
	  matroidName = pToken.next().toString();
	}

	// get modulus
	lnum = ((Integer)(pLNum.next())).intValue();
	token = pToken.next().toString();
	p = Integer.valueOf(token).intValue();
	if (p != calc.getFieldSize()) {
	  calc = FiniteFieldFactory.createFiniteField(p);
	}
	
	// get column count of matrix
	lnum = ((Integer)(pLNum.next())).intValue();
	token = pToken.next().toString();
	n = Integer.valueOf(token).intValue();
	if (n <= 0) {
	  throw new IOException("Error in line " + lnum
				+ ": size must be positive");
	}
	
	// get row count of matrix 
	lnum = ((Integer)(pLNum.next())).intValue();
	token = pToken.next().toString();
	r = Integer.valueOf(token).intValue();
	if (r <= 0) {
	  throw new IOException("Error in line " + lnum
				+ ": rank must be positive");
	}

	matrix = new int[n][r];
	for (i=0; i<r; i++) {
	  for (j=0; j<n; j++) {
	    // get i,j entry of matrix
	    lnum = ((Integer)(pLNum.next())).intValue();
	    token = pToken.next().toString();
	    // add to zero in calculator to ensure value is in range
	    matrix[j][i] = calc.sum(Integer.valueOf(token).intValue(),0);
	  }
	}   

	// now form array of FiniteFieldVectors
	col = new FiniteFieldVector[n];
	for (i=0; i<n; i++) {
	  col[i] = new FiniteFieldVector(matrix[i], r, calc);
	}

	// finally, form the matroid
	matroid= new FiniteFieldMatroid(col);
	if (useNames) {
	  matroid.setName(matroidName);
	}
	mElements.add(matroid);
      }
      mCurElement = 0;
      mHasMoreElements = true;
    }
    catch (NoSuchElementException exNoElt) {
      throw new IOException("Error in line " + lnum + ": not enough entries");
    }
    catch (NumberFormatException exNumFmt) {
      throw new IOException("Error in line " + lnum
			    + ": invalid number format: " + token);
    }
    catch (IllegalArgumentException exIllArg) {
      exIllArg.printStackTrace();
      throw new IOException("Error in line " + lnum);
    }
  }

  /**
   * tests if this enumeration has more elements
   * @return <code>true</code> if and only if there is at least one more matrix
   */
  public boolean hasMoreElements()
  {
    return mHasMoreElements;
  }

  /**
   * returns the next matrix as a FiniteFieldMatroid, if there is one
   * @return next element (matroid)
   * @throws NoSuchElementException if there are no more elements
   */
  public Object nextElement()
    throws NoSuchElementException
  {
    Object elt;

    try {
      elt = mElements.get(mCurElement);
    }
    catch (ArrayIndexOutOfBoundsException ex) {
      throw new NoSuchElementException();
    }

    mHasMoreElements = (++mCurElement < mElements.size());
    return elt;
  }
}
