package Oid;
import java.util.*;

/** 
 * All concrete matroid representations and containers for various 
 * representations of the same matroid should support this interface.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 25 Aug 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public interface Matroid {
  /** 
   * returns the number of elements in matroid 
   *
   * @return element count
   */
  int getSize();

  /** 
   * returns the rank of the matroid 
   *
   * @return matroid rank
   */
  int getRank();

  /** 
   * sets the matroid's name 
   *
   * @param newName new name for matroid
   */
  void setName(String newName);

  /** 
   * returns the matroid's name 
   *
   * @return name of matroid
   */
  String getName();

  /** 
   * sets the label for one of the matroid's elements 
   *
   * @param element element number (starts at zero)
   * @param label label for element
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is out
   * of bounds
   */
  void setLabel(int element, String label)
    throws ArrayIndexOutOfBoundsException;

  /** 
   * returns the label for one of the matroid's elements 
   *
   * @param element element number (starts at zero)
   * @return label of element
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is out
   * of bounds
   */
  String getLabel(int element) 
    throws ArrayIndexOutOfBoundsException;

  /** 
   * copies the internal state of this matroid to a string in the
   * specified format
   *
   * @param format format string ("text" is currently supported)
   * @return matroid's state in string form
   * @throws IllegalArgumentException if format is not supported
   */
  String getState(String format)
  throws IllegalArgumentException;

  /** 
   * sets state of this matroid from the specified string using 
   * the specified format
   *
   * @param state state string for matroid
   * @param format format state string is in
   * @throws IllegalArgumentException if format is not supported
   */
  void setState(String state, String format)
  throws IllegalArgumentException;

  /** 
   * determines the rank of a subset of the matroid's elements 
   *
   * @param s subset of base set of matroid
   * @return rank of <code>s</code>in this matroid
   */
  int subsetRank(Subset s);

  /** 
   * determines whether a subset of the matroid is a circuit 
   *
   * @param s subset of base set of matroid
   * @return boolean indicating whether <code>s</code>is a circuit in
   * this matroid
   */
  boolean isCircuit(Subset s);

  /** 
   * determines whether a subset of the matroid is a flat 
   *
   * @param s subset of base set of matroid
   * @return boolean indicating whether <code>s</code>is a flat in this matroid
   */
  boolean isFlat(Subset s);

  /** 
   * determines whether a subset of the matroid is a basis 
   *
   * @param s subset of base set of matroid
   * @return boolean indicating whether <code>s</code>is a basis in
   * this matroid
   */
  boolean isBasis(Subset s);

  /** 
   * determines whether a subset of the matroid is independent 
   *
   * @param s subset of base set of matroid
   * @return boolean indicating whether <code>s</code>is independent
   * in this matroid
   */
  boolean isIndependent(Subset s);

  /** 
   * determines whether a subset of the matroid is a hyperplane 
   *
   * @param s subset of base set of matroid
   * @return boolean indicating whether <code>s</code>is a hyperplane
   * in this matroid
   */
  boolean isHyperplane(Subset s);

  /** 
   * determines whether an element is in the closure of a set 
   *
   * @param s subset of base set of matroid
   * @param e element to check (elements start with zero)
   * @return boolean indicating whether <code>e</code> is in
   * cl(<code>s</code>) in this matroid
   */
  boolean isInClosure(Subset s, int e);

  /** 
   * determines the closure of a subset of the matroid 
   *
   * @param s subset of base set of matroid
   * @param subset with closure of <code>s</code>in this matroid
   */
  Subset closure(Subset s);
}


  
