package Oid;
import java.util.*;

/**
 * An instance of MatroidIsomorphismChecker accepts as input two 
 * Matroid instances and determines whether they are isomorphic.  It first
 * performs some common-sense checks, making sure that both matroids have
 * the same size and rank.
 *
 * If the matroids pass these checks, then the algorithm finds their 
 * independent set degree sequences and compares them.  If there is a match,
 * it adds in circuit degrees, then spanning set degrees, then cocircuit 
 * degrees.  If there is still a match, the algorithm seeks a mapping based
 * on the degree sequences that preserves the circuit structure of the
 * matroids.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 22 Feb 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class MatroidIsomorphismChecker implements Combiner
{
  //
  // private data members
  //
  private Matroid m1, m2;  // last matroids checked
  private String mExpl;       // explanation of steps in last check
  private boolean mUseExpl;
  private int mMaxDepth; // maximum number of mappings to check (-1 for all)
  private boolean mFailResult; // conclusion if no result after mMaxDepth maps
  private boolean mMaxDepthReached;
  private DefaultCircuitAlgorithm mCCTAlg;
  private DefaultIndependentSetAlgorithm mIndAlg;
  private SpanningSetAlgorithm mSpnAlg;
  private DefaultBasisAlgorithm mBasAlg;
  private DefaultFlatAlgorithm mFltAlg;
  private int[] mMap;
  private Vector mDegL;
  private Vector mChainL;
  private Vector mDegR;
  private Vector mChainR;
  private Vector mBasesL;
  private Vector mBasesR;
  private boolean mCriteriaSetL;
  private boolean mCriteriaSetR;

  /**
   * constructor initializes algorithms for checks
   */
  public MatroidIsomorphismChecker()
  {
    mCCTAlg = new DefaultCircuitAlgorithm();
    mIndAlg = new DefaultIndependentSetAlgorithm();
    mSpnAlg = new SpanningSetAlgorithm();
    mBasAlg = new DefaultBasisAlgorithm();
    mFltAlg = new DefaultFlatAlgorithm();
    mUseExpl = true;
    mMaxDepth = -1;
    mFailResult = false;
  }

  /**
   * sets a value indicating the maximum number of candidate mappings the
   * checker should review before making a conclusion, and sets the 
   * conclusion the checker should take if the maximum number has been 
   * checked without discovering an isomorphism; if the maximum number is
   * -1, the checker always examines all candidate maps
   *
   * @param maxDepth maximum number of candidate maps to check
   * @param failResult boolean value indicating conclusion to return if
   *        maxDepth maps have been checked with no result
   * @throws IllegalArgumentException if maxDepth &lt; -1
   */
  public void setMaxDepth(int maxDepth, boolean failResult)
  {
    if (mMaxDepth < -1) {
      throw new IllegalArgumentException("Negative maximum depth");
    }

    mMaxDepth = maxDepth;
    mFailResult = failResult;
  }

  /**
   * returns the current maximum search depth
   * 
   * @return maximum number of maps to check or -1 to check all
   */
  public int getMaxDepth()
  {
    return mMaxDepth;
  }
   
  /**
   * returns a boolean value indicating whether the maximum search
   * depth was reached on the last check
   *
   * @return boolean value
   */
  public boolean maxDepthReached()
  {
    return mMaxDepthReached;
  }
 
  /**
   * returns the conclusion the checker will make if there are no
   * isomorphisms in the first maxDepth maps checked
   *
   * @return boolean value
   */
  public boolean getFailResult()
  {
    return mFailResult;
  }

  /**
   * sets value indicating whether checker should generate an explanation
   * each time it checks two matroids
   *
   * @param makeExplanation boolean value
   */
  public void setMakeExplanation(boolean makeExplanation)
  {
    mUseExpl = makeExplanation;
  }

  /** 
   * returns the explanation generated during the last check
   *
   * @return String with explanation (null if no check has been done)
   */
  public String getExplanation()
  {
    String answer = (String)null;
    if (mExpl instanceof String) {
      answer = new String(mExpl);
    }
    return answer;
  }

  /**
   * returns the first isomorphism found in the last check, if there was one
   *
   * @return array of integers with mapping
   */
  public int[] getMap()
  {
    return mMap;
  }

  /**
   * returns the first matroid used for the last check
   *
   * @return Matroid reference (null if no check has been done)
   */
  public Matroid getMatroid1()
  {
    Matroid answer = (Matroid)null;
    if (m1 instanceof Matroid) {
      answer = m1;
    }
    return answer;
  }

  /** 
   * returns a vector of degree sequences for the first matroid
   *
   * @return Vector with circuit, ind set, spanning set, and basis deg seqs
   */
  public Vector getDegrees1()
  {
    Vector answer = (Vector)null;
    if (mDegL instanceof Vector) {
      answer = mDegL;
    }
    return answer;
  }

  /**
   * returns a vector of chains for the first matroid
   *
   * @return Vector with circuit, ind set, spanning set and bases chains
   */
  public Vector getChains1()
  {
    Vector answer = (Vector)null;
    if (mChainL instanceof Vector) {
      answer = mChainL;
    }
    return answer;
  }

  /**
   * returns a vector containing a list of bases for the first matroid
   *
   * @return Vector of Subsets with bases
   */
  public Vector getBases1()
  {
    Vector answer = (Vector)null;
    if (mBasesL instanceof Vector) {
      answer = mBasesL;
    }
    return answer;
  }

  /**
   * returns the second matroid used for the last check
   *
   * @return Matroid reference (null if no check has been done)
   */
  public Matroid getMatroid2()
  {
    Matroid answer = (Matroid)null;
    if (m2 instanceof Matroid) {
      answer = m2;
    }
    return answer;
  }

  /** 
   * returns a vector of degree sequences for the second matroid
   *
   * @return Vector with circuit, ind set, spanning set, and basis deg seqs
   */
  public Vector getDegrees2()
  {
    Vector answer = (Vector)null;
    if (mDegR instanceof Vector) {
      answer = mDegR;
    }
    return answer;
  }

  /**
   * returns a vector of chains for the second matroid
   *
   * @return Vector with circuit, ind set, spanning set and bases chains
   */
  public Vector getChains2()
  {
    Vector answer = (Vector)null;
    if (mChainR instanceof Vector) {
      answer = mChainR;
    }
    return answer;
  }

  /**
   * returns a vector containing a list of bases for the second matroid
   *
   * @return Vector of Subsets with bases
   */
  public Vector getBases2()
  {
    Vector answer = (Vector)null;
    if (mBasesR instanceof Vector) {
      answer = mBasesR;
    }
    return answer;
  }

  /**
   * sets reference to first matroid (without accompanying chains, degrees
   * and bases)
   *
   * @param m Matroid reference
   */
  public void setMatroid1(Matroid m)
  {
    m1 = m;
    int n = m1.getSize();
    mChainL = new Vector();
    mDegL = new Vector();
      
    mIndAlg.setInstance(m1);
    mChainL.add(mIndAlg.getSubsetCountChain());
    mDegL.add(getTuple((Vector)mIndAlg.getSubsetList(), n));
      
    mCCTAlg.setInstance(m1);
    mChainL.add(mCCTAlg.getSubsetCountChain());
    mDegL.add(getTuple((Vector)mCCTAlg.getSubsetList(), n));
      
    mBasAlg.setInstance(m1);
    mChainL.add(mBasAlg.getSubsetCountChain());
    mBasesL = (Vector)mBasAlg.getSubsetList();
    mDegL.add(getTuple(mBasesL, n));

    mSpnAlg.setInstance(m1);
    mChainL.add(mSpnAlg.getSubsetCountChain());
    mDegL.add(getTuple((Vector)mSpnAlg.getSubsetList(), n));

    if (m.getRank() == 3) {
      mFltAlg.setInstance(m1);
      mChainL.add(mFltAlg.getSubsetCountChain());
      mDegL.add(getTuple((Vector)mFltAlg.getSubsetList(), n));
    }

    mCriteriaSetL = true;
  }

  /**
   * sets references to first matroid and Vectors of chains, degree sequences
   * and bases
   *
   * @param m Matroid reference
   * @param chains Vector with chains of circuits, ind sets, spanning sets and
   * bases
   * @param degrees Vector with degree sequences for cct, ind, span, bases
   * @param bases Vector with list of bases
   */
  public void setMatroid1(Matroid m,
			  Vector chains, 
			  Vector degrees, 
			  Vector bases)
  {
    m1 = m;
    mCriteriaSetL = true;
    mDegL = degrees;
    mChainL = chains;
    mBasesL = bases;
  }

  /**
   * sets reference to second matroid (without accompanying chains, degrees
   * and bases)
   *
   * @param m Matroid reference
   */
  public void setMatroid2(Matroid m)
  {
    m2 = m;
    int n = m2.getSize();
    mChainR = new Vector();
    mDegR = new Vector();
      
    mIndAlg.setInstance(m2);
    mChainR.add(mIndAlg.getSubsetCountChain());
    mDegR.add(getTuple((Vector)mIndAlg.getSubsetList(), n));
      
    mCCTAlg.setInstance(m2);
    mChainR.add(mCCTAlg.getSubsetCountChain());
    mDegR.add(getTuple((Vector)mCCTAlg.getSubsetList(), n));
      
    mBasAlg.setInstance(m2);
    mChainR.add(mBasAlg.getSubsetCountChain());
    mBasesR = (Vector)mBasAlg.getSubsetList();
    mDegR.add(getTuple(mBasesR, n));

    mSpnAlg.setInstance(m2);
    mChainR.add(mSpnAlg.getSubsetCountChain());
    mDegR.add(getTuple((Vector)mSpnAlg.getSubsetList(), n));

    if (m.getRank() == 3) {
      mFltAlg.setInstance(m2);
      mChainR.add(mFltAlg.getSubsetCountChain());
      mDegR.add(getTuple((Vector)mFltAlg.getSubsetList(), n));
    }

    mCriteriaSetR = true;
  }

  /**
   * sets references to second matroid and Vectors of chains, degree sequences
   * and bases
   *
   * @param m Matroid reference
   * @param chains Vector with chains of circuits, ind sets, spanning sets and
   * bases
   * @param degrees Vector with degree sequences for cct, ind, span, bases
   * @param bases Vector with list of bases
   */
  public void setMatroid2(Matroid m,
			  Vector chains, 
			  Vector degrees, 
			  Vector bases)
  {
    m2 = m;
    mCriteriaSetR = true;
    mDegR = degrees;
    mChainR = chains;
    mBasesR = bases;
  }


  /**
   * Performs check on matroids, and returns a boolean value indicating
   * whether any isomorphisms were found
   *
   * @param m1 first matroid to check
   * @param m2 second matroid to check
   * @return boolean value indicating result of check
   * @throws NullPointerException if matroids aren't set
   */
  public boolean checkMatroids()
  throws NullPointerException
  {
    mMaxDepthReached = false;

    if (!(m1 instanceof Matroid)) {
      throw new NullPointerException("No left-hand matroid set");
    }

    if (!(m2 instanceof Matroid)) {
      throw new NullPointerException("No right-hand matroid set");
    }

    boolean answer = false;
    int i,j;
    int n=0, r=0, p=0;
    int index;
    StringBuffer expl=(StringBuffer)null;

    if (mUseExpl) {
      expl = new StringBuffer();
    }

    // 
    // check sizes, ranks and moduli of base fields
    //
    n = m1.getSize();
    r = m1.getRank();
    if ((m2.getSize() != n) || (m2.getRank() != r)) {
      if (mUseExpl) {
	expl.append("matroids have inconsistent sizes and/or, ranks");
	mExpl = expl.toString();
      }
      return answer;
    }

    Vector degrees1 = new Vector(); // holds degree sequences for m1
    Vector degrees2 = new Vector(); // holds degree sequences for m2
    Vector chain1, chain2, sets1, sets2;
    
    //
    // get chains, degree sequences and basis lists if necessary
    //
    if (!mCriteriaSetL) {
      mChainL = new Vector();
      mDegL = new Vector();
      
      mIndAlg.setInstance(m1);
      mChainL.add(mIndAlg.getSubsetCountChain());
      mDegL.add(getTuple((Vector)mIndAlg.getSubsetList(), n));
      
      mCCTAlg.setInstance(m1);
      mChainL.add(mCCTAlg.getSubsetCountChain());
      mDegL.add(getTuple((Vector)mCCTAlg.getSubsetList(), n));
      
      mBasAlg.setInstance(m1);
      mChainL.add(mBasAlg.getSubsetCountChain());
      mBasesL = (Vector)mBasAlg.getSubsetList();
      mDegL.add(getTuple(mBasesL, n));

      mSpnAlg.setInstance(m1);
      mChainL.add(mSpnAlg.getSubsetCountChain());
      mDegL.add(getTuple((Vector)mSpnAlg.getSubsetList(), n));

      if (m1.getRank() == 3) {
	mFltAlg.setInstance(m1);
	mChainL.add(mFltAlg.getSubsetCountChain());
	mDegL.add(getTuple((Vector)mFltAlg.getSubsetList(), n));
      }
    }

    if (!mCriteriaSetR) {
      mChainR = new Vector();
      mDegR = new Vector();
      
      mIndAlg.setInstance(m2);
      mChainR.add(mIndAlg.getSubsetCountChain());
      mDegR.add(getTuple((Vector)mIndAlg.getSubsetList(), n));
      
      mCCTAlg.setInstance(m2);
      mChainR.add(mCCTAlg.getSubsetCountChain());
      mDegR.add(getTuple((Vector)mCCTAlg.getSubsetList(), n));
      
      mBasAlg.setInstance(m2);
      mChainR.add(mBasAlg.getSubsetCountChain());
      mBasesR = (Vector)mBasAlg.getSubsetList();
      mDegR.add(getTuple(mBasesR, n));

      mSpnAlg.setInstance(m2);
      mChainR.add(mSpnAlg.getSubsetCountChain());
      mDegR.add(getTuple((Vector)mSpnAlg.getSubsetList(), n));

      if (m2.getRank() == 3) {
	mFltAlg.setInstance(m2);
	mChainR.add(mFltAlg.getSubsetCountChain());
	mDegR.add(getTuple((Vector)mFltAlg.getSubsetList(), n));
      }
    }

    // check independent set chains
    index = 0;
    chain1 = (Vector)mChainL.get(index);
    degrees1.add((int [])mDegL.get(index));

    chain2 = (Vector)mChainR.get(index);
    degrees2.add((int [])mDegR.get(index));

    if (mUseExpl) {
      expl.append("independent set chains:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain1, 0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain2, 0));
      expl.append("\n");
    }
    if (compareChains(chain1, chain2)) {
      if (mUseExpl) {
	expl.append("independent set chains match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("independent set chains do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // check independent set degrees
    if (mUseExpl) {
      expl.append("independent set degree sequences:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees1.get(index),0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees2.get(index),0));
      expl.append("\n");
    }

    if (degreesMatch(degrees1, degrees2,n)) {
      if (mUseExpl) {
	expl.append("independent set degree sequences match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("independent set degree sequences do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }
      
    // now check circuits
    ++index;
    chain1 = (Vector)mChainL.get(index);
    degrees1.add((int [])mDegL.get(index));

    chain2 = (Vector)mChainR.get(index);
    degrees2.add((int [])mDegR.get(index));

    if (mUseExpl) {
      expl.append("circuit chains:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain1, 0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain2, 0));
      expl.append("\n");
    }


    if (compareChains(chain1, chain2)) {
      if (mUseExpl) {
	expl.append("circuit chains match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("circuit chains do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // check degree sequences
    if (mUseExpl) {
      expl.append("circuit degree sequences:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees1.get(index),0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees2.get(index),0));
      expl.append("\n");
    }

    if (degreesMatch(degrees1, degrees2, n)) {
      if (mUseExpl) {
	expl.append("circuit degree sequences match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("circuit degree sequences do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // now check bases -- save bases to use when checking mappings
    ++index;
    chain1 = (Vector)mChainL.get(index);
    degrees1.add((int [])mDegL.get(index));

    chain2 = (Vector)mChainR.get(index);
    degrees2.add((int [])mDegR.get(index));

    if (mUseExpl) {
      expl.append("basis count:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain1, 0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain2, 0));
      expl.append("\n");
    }

    if (compareChains(chain1, chain2)) {
      if (mUseExpl) {
	expl.append("basis counts match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("basis counts do not match");
	mExpl = expl.toString();
      }
      return answer;
    }

    // check degree sequences
    if (mUseExpl) {
      expl.append("basis degree sequences:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees1.get(index),0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees2.get(index),0));
      expl.append("\n");
    }

    if (degreesMatch(degrees1, degrees2,n)) {
      if (mUseExpl) {
	expl.append("basis degree sequences match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("basis degree sequences do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // now check spanning sets
    ++index;
    chain1 = (Vector)mChainL.get(index);
    degrees1.add((int [])mDegL.get(index));

    chain2 = (Vector)mChainR.get(index);
    degrees2.add((int [])mDegR.get(index));

    if (mUseExpl) {
      expl.append("spanning set chains:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain1, 0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain2, 0));
      expl.append("\n");
    }

    if (compareChains(chain1, chain2)) {
      if (mUseExpl) {
	expl.append("spanning set chains match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("spanning set chains do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // check degree sequences
    if (mUseExpl) {
      expl.append("spanning set degree sequences:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees1.get(index),0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees2.get(index),0));
      expl.append("\n");
    }

    if (degreesMatch(degrees1, degrees2, n)) {
      if (mUseExpl) {
	expl.append("spanning set degree sequences match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("spanning set degree sequences do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // check flats if rank is 3
    if (3 == r) {
      ++index;
      chain1 = (Vector)mChainL.get(index);
      degrees1.add((int [])mDegL.get(index));
      
      chain2 = (Vector)mChainR.get(index);
      degrees2.add((int [])mDegR.get(index));
      
      if (mUseExpl) {
	expl.append("flat chains:\n");
	expl.append(m1.getName());
	expl.append(": ");
	expl.append(getChainDescription(chain1, 0));
	expl.append("\n");
	expl.append(m2.getName());
	expl.append(": ");
	expl.append(getChainDescription(chain2, 0));
	expl.append("\n");
      }
      
      
      if (compareChains(chain1, chain2)) {
	if (mUseExpl) {
	  expl.append("flat chains match\n\n");
	}
      }
      else {
	if (mUseExpl) {
	  expl.append("flat chains do not match\n");
	  mExpl = expl.toString();
	}
	return answer;
      }
    }



    // now, all degree sequences have matched, so determine orbits
    Vector orbits = getOrbits(degrees1, degrees2, n);
    Vector[] orbitArray;
    Vector orbit1, orbit2;
    StringBuffer s1, s2;
    Iterator pOrbit, p1, p2;
    if (mUseExpl) {
      expl.append("Orbits:\n");
      for (pOrbit = orbits.iterator(); pOrbit.hasNext();) {
	orbitArray = (Vector[])pOrbit.next();
	orbit1 = orbitArray[0];
	orbit2 = orbitArray[1];
	s1 = new StringBuffer("(");
	s2 = new StringBuffer("(");
	for (p1 = orbit1.iterator(), p2 = orbit2.iterator(); p1.hasNext();) {
	  s1.append(((Integer)p1.next()).intValue()); s1.append(" ");
	  s2.append(((Integer)p2.next()).intValue()); s2.append(" ");
	}
	s1.append(")");
	s2.append(")");
	expl.append(s1.toString()); expl.append("\n");
	expl.append(s2.toString()); expl.append("\n");
	expl.append("\n");
      }
    }

    // if we're supposed to stop on one orbit, do so here
    if (0 == mMaxDepth) {
      if (mUseExpl) {
	expl.append("stopping with orbits");
	mExpl = expl.toString();
      }
      mMaxDepthReached = true;
      return mFailResult;
    }

    // generate permutation enumerators for orbits
    Vector enumerators = new Vector();
    Vector perms = new Vector();
    PermutationEnumeration perm;
    enumerators.setSize(orbits.size());
    perms.setSize(orbits.size());
    int[] orbitSize = new int[orbits.size()];
    i = 0;
    for (pOrbit = orbits.iterator(); pOrbit.hasNext();) {
      orbitArray = (Vector[])pOrbit.next();
      orbitSize[i] = orbitArray[0].size();
      if (orbitSize[i] > 1) {
	perm = new PermutationEnumeration(orbitSize[i]);
	enumerators.setElementAt(perm, i);
	perms.setElementAt(perm.nextElement(), i);
      }
      i++;
    }
    
    // loop through all permutation combinations
    boolean hasMore = true;
    boolean found;
    Iterator pPerm;
    int mapCount = 0;
    int[] theMap = new int[n];
    mMap = new int[n];

    while (hasMore) {
      //
      // do we stop here anyway?
      // 
      if ((mMaxDepth > 0) && (mapCount >= mMaxDepth)) {
	if (mUseExpl) {
	  expl.append("maximum search length reached");
	}
	mMaxDepthReached = true;
	return mFailResult;
      }

      //
      // generate a map using current permutations
      //
      makeMap(perms, orbits);
      //
      // check whether mapping preserves bases
      //
      // for now, always stop if we find an isomorphism
      if (checkMap(mBasesL, mBasesR)) {
	// this is an isomorphism
	if (!answer) {
	  // this is the first isomorphism, so save it
	  for (i=0; i<n; i++) {
	    theMap[i] = mMap[i];
	  }
	}
	answer = true;
	if (mUseExpl) {
	  expl.append("isomorphism discovered: ");
	  for (i=0; i<n; i++) {
	    expl.append(String.valueOf(mMap[i]));
	    expl.append(" ");
	  } 
	  expl.append("\n");
	  mExpl = expl.toString();
	  return answer;
	}
	else {
	  // if we don't need all the info, just stop now
	  return answer;
	}
      }
      
      //
      // move ahead through permutation loop -- find the first 
      // enumeration that is not complete, move it up one, and reset
      // all of its predecessors to their beginnings
      //
      hasMore = false;
      for (i=0; i<enumerators.size(); i++) {
	if (orbitSize[i] > 1) {
	  perm = (PermutationEnumeration)enumerators.get(i);
	  if (perm.hasMoreElements()) {
	    perms.setElementAt(perm.nextElement(), i);
	    for (j=0; j<i; j++) {
	      if (orbitSize[j] > 1) {
		perm = (PermutationEnumeration)enumerators.get(j);
		perm.reset();
		perms.setElementAt(perm.nextElement(), j);
	      }
	    }
	    hasMore = true;
	    break;
	  }
	}
      }

      System.gc();
      ++mapCount;
    }

    if (mUseExpl) {
      mExpl = expl.toString();
    }

    if (answer) {
      // if there was an isomorphism, save the first one found
      for (i=0; i<n; i++) {
	mMap[i] = theMap[i];
      }
    }
    else {
      // make sure there is no map 
      mMap = (int [])null;
    }

    return answer;
  }
  
  /**
   * Performs check on matroids, and returns a boolean value indicating
   * whether degree sequences and chains matched
   *
   * @param m1 first matroid to check
   * @param m2 second matroid to check
   * @return boolean value indicating result of check
   * @throws NullPointerException if matroids aren't set
   */
  public boolean checkMatroidsShallow()
  throws NullPointerException
  {

    if (!(m1 instanceof Matroid)) {
      throw new NullPointerException("No left-hand matroid set");
    }

    if (!(m2 instanceof Matroid)) {
      throw new NullPointerException("No right-hand matroid set");
    }

    boolean answer = false;
    int i,j;
    int n=0, r=0, p=0;
    int index;
    StringBuffer expl=(StringBuffer)null;

    if (mUseExpl) {
      expl = new StringBuffer();
    }

    // 
    // check sizes, ranks and moduli of base fields
    //
    n = m1.getSize();
    r = m1.getRank();
    if ((m2.getSize() != n) || (m2.getRank() != r)) {
      if (mUseExpl) {
	expl.append("matroids have inconsistent sizes and/or, ranks");
	mExpl = expl.toString();
      }
      return answer;
    }

    Vector degrees1 = new Vector(); // holds degree sequences for m1
    Vector degrees2 = new Vector(); // holds degree sequences for m2
    Vector chain1, chain2, sets1, sets2;
    
    //
    // get chains, degree sequences and basis lists if necessary
    //
    if (!mCriteriaSetL) {
      mChainL = new Vector();
      mDegL = new Vector();
      
      mIndAlg.setInstance(m1);
      mChainL.add(mIndAlg.getSubsetCountChain());
      mDegL.add(getTuple((Vector)mIndAlg.getSubsetList(), n));
      
      mCCTAlg.setInstance(m1);
      mChainL.add(mCCTAlg.getSubsetCountChain());
      mDegL.add(getTuple((Vector)mCCTAlg.getSubsetList(), n));
      
      mBasAlg.setInstance(m1);
      mChainL.add(mBasAlg.getSubsetCountChain());
      mBasesL = (Vector)mBasAlg.getSubsetList();
      mDegL.add(getTuple(mBasesL, n));

      mSpnAlg.setInstance(m1);
      mChainL.add(mSpnAlg.getSubsetCountChain());
      mDegL.add(getTuple((Vector)mSpnAlg.getSubsetList(), n));
    }

    if (!mCriteriaSetR) {
      mChainR = new Vector();
      mDegR = new Vector();
      
      mIndAlg.setInstance(m2);
      mChainR.add(mIndAlg.getSubsetCountChain());
      mDegR.add(getTuple((Vector)mIndAlg.getSubsetList(), n));
      
      mCCTAlg.setInstance(m2);
      mChainR.add(mCCTAlg.getSubsetCountChain());
      mDegR.add(getTuple((Vector)mCCTAlg.getSubsetList(), n));
      
      mBasAlg.setInstance(m2);
      mChainR.add(mBasAlg.getSubsetCountChain());
      mBasesR = (Vector)mBasAlg.getSubsetList();
      mDegR.add(getTuple(mBasesR, n));

      mSpnAlg.setInstance(m2);
      mChainR.add(mSpnAlg.getSubsetCountChain());
      mDegR.add(getTuple((Vector)mSpnAlg.getSubsetList(), n));
    }

    // check independent set chains
    index = 0;
    chain1 = (Vector)mChainL.get(index);
    degrees1.add((int [])mDegL.get(index));

    chain2 = (Vector)mChainR.get(index);
    degrees2.add((int [])mDegR.get(index));

    if (mUseExpl) {
      expl.append("independent set chains:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain1, 0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain2, 0));
      expl.append("\n");
    }
    if (compareChains(chain1, chain2)) {
      if (mUseExpl) {
	expl.append("independent set chains match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("independent set chains do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // check independent set degrees
    if (mUseExpl) {
      expl.append("independent set degree sequences:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees1.get(index),0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees2.get(index),0));
      expl.append("\n");
    }

    if (degreesMatch(degrees1, degrees2,n)) {
      if (mUseExpl) {
	expl.append("independent set degree sequences match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("independent set degree sequences do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }
      
    // now check circuits
    ++index;
    chain1 = (Vector)mChainL.get(index);
    degrees1.add((int [])mDegL.get(index));

    chain2 = (Vector)mChainR.get(index);
    degrees2.add((int [])mDegR.get(index));

    if (mUseExpl) {
      expl.append("circuit chains:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain1, 0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain2, 0));
      expl.append("\n");
    }


    if (compareChains(chain1, chain2)) {
      if (mUseExpl) {
	expl.append("circuit chains match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("circuit chains do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // check degree sequences
    if (mUseExpl) {
      expl.append("circuit degree sequences:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees1.get(index),0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees2.get(index),0));
      expl.append("\n");
    }

    if (degreesMatch(degrees1, degrees2, n)) {
      if (mUseExpl) {
	expl.append("circuit degree sequences match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("circuit degree sequences do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // now check bases -- save bases to use when checking mappings
    ++index;
    chain1 = (Vector)mChainL.get(index);
    degrees1.add((int [])mDegL.get(index));

    chain2 = (Vector)mChainR.get(index);
    degrees2.add((int [])mDegR.get(index));

    if (mUseExpl) {
      expl.append("basis count:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain1, 0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain2, 0));
      expl.append("\n");
    }

    if (compareChains(chain1, chain2)) {
      if (mUseExpl) {
	expl.append("basis counts match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("basis counts do not match");
	mExpl = expl.toString();
      }
      return answer;
    }

    // check degree sequences
    if (mUseExpl) {
      expl.append("basis degree sequences:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees1.get(index),0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees2.get(index),0));
      expl.append("\n");
    }

    if (degreesMatch(degrees1, degrees2,n)) {
      if (mUseExpl) {
	expl.append("basis degree sequences match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("basis degree sequences do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // now check spanning sets
    ++index;
    chain1 = (Vector)mChainL.get(index);
    degrees1.add((int [])mDegL.get(index));

    chain2 = (Vector)mChainR.get(index);
    degrees2.add((int [])mDegR.get(index));

    if (mUseExpl) {
      expl.append("spanning set chains:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain1, 0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getChainDescription(chain2, 0));
      expl.append("\n");
    }

    if (compareChains(chain1, chain2)) {
      if (mUseExpl) {
	expl.append("spanning set chains match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("spanning set chains do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    // check degree sequences
    if (mUseExpl) {
      expl.append("spanning set degree sequences:\n");
      expl.append(m1.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees1.get(index),0));
      expl.append("\n");
      expl.append(m2.getName());
      expl.append(": ");
      expl.append(getTupleDescription((int[])degrees2.get(index),0));
      expl.append("\n");
    }

    if (degreesMatch(degrees1, degrees2, n)) {
      if (mUseExpl) {
	expl.append("spanning set degree sequences match\n\n");
      }
    }
    else {
      if (mUseExpl) {
	expl.append("spanning set degree sequences do not match\n");
	mExpl = expl.toString();
      }
      return answer;
    }

    answer = true; // shallow check is positive
    return answer;
  }

  //
  // private method compares two chains and returns true if they have
  // the same numbers
  //
  private boolean compareChains(Vector v1, Vector v2)
  {
    boolean answer = true;
    int i,j;
    Iterator p1, p2;
    for (p1=v1.iterator(), p2=v2.iterator(); p1.hasNext() && p2.hasNext();) {
      i = ((Integer)p1.next()).intValue();
      j = ((Integer)p2.next()).intValue();
      if (i != j) {
	answer = false;
	break;
      }
    }

    if (p1.hasNext() || p2.hasNext()) {
      answer = false;
    }

    return answer;
  }

  // 
  // private method generates a descriptive string for a tuple,
  // starting at the passed index
  //
  private String getTupleDescription(int[] tuple, int start)
  {
    int i, n=tuple.length;
    StringBuffer ansBuf = new StringBuffer("(");
    boolean flag = false;

    for (i=start; i<n; i++) {
      if (!flag) {
	flag = true;
      }
      else {
	ansBuf.append(", ");
      }
      ansBuf.append(String.valueOf(tuple[i]));
    }
    ansBuf.append(")");

    return ansBuf.toString(); 
  }

  //
  // private method generates a descriptive string for a chain,
  // starting at the passed index and preceded by a total
  //
  private String getChainDescription(Vector chain, int start)
  {
    int i, n=chain.size(), total = 0;
    boolean flag = false;
    StringBuffer ansBuf;
    
    // get total
    for (i=0; i<n; i++) {
      total += ((Integer)(chain.get(i))).intValue();
    }
    
    ansBuf = new StringBuffer(String.valueOf(total));
    ansBuf.append(" = ");
    for (i=start; i<n; i++) {
      if (!flag) {
	flag = true;
      }
      else {
	ansBuf.append(" + ");
      }
      ansBuf.append(String.valueOf(((Integer)(chain.get(i))).intValue()));
    }
    return ansBuf.toString();
  }
  
  //
  // private method returns a matrix with the number of occurrences
  // of each value in the passed vector of sets, up to the specified
  // maximum
  //
  private int[] getTuple(Vector sets, int n)
  {
    int[] tuple = new int[n];
    int i, j;
    Subset s;

    for (i=0; i<n; i++) {
      tuple[i] = 0;
    }

    // 
    // loop through all sets
    //
    for (Iterator p = sets.iterator(); p.hasNext(); ) {
      s = (Subset)(p.next());
      try {
	for (i=0; i<n; i++) {
	  tuple[i] += (s.get(i)) ? 1 : 0;
	}
      }
      catch (Exception e) {
	// this "can't happen"
	throw new InternalError();
      }
    }
    return tuple;
  }

  //
  // private method returns a vector with the number of elements in 
  // no sets, the number of elements in one set, etc. 
  //
  private Vector getPointChain(int[] tuple)
  {
    int i, j, n = tuple.length;
    Vector answer = new Vector();

    // we need a vector with element counts at the various indices
    for (i=0; i<n; i++) {
      j = tuple[i];
      if (answer.size() <= j) {
	answer.setSize(j+1);
      }
      
      // set the correct value to 1 or increment it
      if (!(answer.get(j) instanceof Integer)) {
	answer.set(j, new Integer(1));
      }
      else {
	answer.set(j, new Integer(((Integer)answer.get(j)).intValue()+1));
      }
    }

    // fill in the remaining values with zeroes
    for (i=0; i<answer.size(); i++) {
      if (!(answer.get(i) instanceof Integer)) {
	answer.set(i, new Integer(0));
      }
    }
    return answer;
  }

  //
  // private function determines whether vectors of degree sequences
  // have a match
  //
  private boolean degreesMatch(Vector degrees1, Vector degrees2, int n)
  {
    boolean answer = true;
    int i, j;
    Iterator p1, p2;
    boolean[] matched = new boolean[n];
    boolean found;
    boolean flag;
    int[] deg1;
    int[] deg2;

    for (i=0; i<n; i++) {
      matched[i] = false;
    }

    // go through first array and find matches in second
    for (i=0; i<n; i++) {
      found = false;
      for (j=0; j<n; j++) {
	if (!matched[j]) {
	  flag = true;
	  for (p1 = degrees1.iterator(), p2 = degrees2.iterator(); 
	       p1.hasNext();) {
	    deg1 = (int[])p1.next();
	    deg2 = (int[])p2.next();
	    if (deg1[i] != deg2[j]) {
	      flag = false;
	      break;
	    }
	  }
	  if (flag) {
	    matched[j] = true;
	    found = true;
	    break;
	  }
	}
      }
      if (!found) {
	answer = false;
	break;
      }
    }

    return answer;
  }

  //
  // private function determines orbits and returns them as a Vector of
  // 2-long arrays of Vectors with the orbit elements for the two 
  // matroids
  //
  private Vector getOrbits(Vector degrees1, Vector degrees2, int n)
  {
    int i, j, k;
    Iterator p1, p2;
    int[] orbitAssignment1 = new int[n];
    int[] orbitAssignment2 = new int[n];
    boolean found;
    boolean flag;
    int[] deg1;
    int[] deg2;
    Vector answer = new Vector();
    Vector[] orbit;
    Vector v1, v2;
    int maxOrbit = 0;

    for (i=0; i<n; i++) {
      orbitAssignment1[i] = -1;
      orbitAssignment2[i] = -1;
    }
    
    for (i=0; i<n; i++) {
      // get orbit assignment for degree list in matroid1
      found = false;
      for (j=0; j<i; j++) {
	flag = true;
	for (p1 = degrees1.iterator(); p1.hasNext(); ) {
	  deg1 = (int[])p1.next();
	  if (deg1[i] != deg1[j]) {
	    flag = false;
	    break;
	  }
	}
	if (flag) {
	  found = true;
	  break;
	}
      }
      if (found) {
	orbitAssignment1[i] = orbitAssignment1[j];
      }
      else {
	orbitAssignment1[i] = maxOrbit++;
	// found a new orbit on the left, so find all matching 
	// degree lists on the right
	for (j=0; j<n; j++) {
	  if (-1 == orbitAssignment2[j]) {
	    flag = true;
	    for (p1 = degrees1.iterator(), p2 = degrees2.iterator();
		 p1.hasNext();) {
	      deg1 = (int[])p1.next();
	      deg2 = (int[])p2.next();
	      if (deg1[i] != deg2[j]) {
		flag = false;
		break;
	      }
	    }
	    if (flag) {
	      orbitAssignment2[j] = orbitAssignment1[i];
	    }
	  }
	}
      }
    }

    // now build vectors with orbit lists
    for (i=0; i<maxOrbit; i++) {
      orbit = new Vector[2];
      orbit[0] = new Vector();
      orbit[1] = new Vector();
      for (j=0; j<n; j++) {
	if (orbitAssignment1[j] == i) {
	  orbit[0].add(new Integer(j));
	}
	if (orbitAssignment2[j] == i) {
	  orbit[1].add(new Integer(j));
	}
      }
      answer.add(orbit);
    }

    return answer;
  }

  // 
  // private method generates a mapping using a vector of permutations
  // and a vector containing the orbits
  //
  private void makeMap(Vector perms, Vector orbits)
  {
    Iterator pPerm, pOrbit;
    Vector[] orbitArray;
    Vector orbit1, orbit2;
    int[] perm;
    int i, j, from, to;

    for (i=0; i<orbits.size(); i++) {
      orbitArray = (Vector[])orbits.get(i);
      orbit1 = orbitArray[0];
      orbit2 = orbitArray[1];
      if (orbit1.size() > 1) {
	perm = (int[])perms.get(i);
	for (j=0; j<orbit1.size(); j++) {
	  from = ((Integer)orbit1.get(j)).intValue();
	  to = ((Integer)orbit2.get(perm[j])).intValue();
	  mMap[from] = to;
	}
      }
      else {
	from = ((Integer)orbit1.get(0)).intValue();
	to = ((Integer)orbit2.get(0)).intValue();
	mMap[from] = to;
      }
    }
  }

  //
  // private method checks a mapping to determine whether it preserves
  // all the sets in the passed vector
  // 
  private boolean checkMap(Vector bases1, Vector bases2)
  {
    boolean answer = true;
    boolean found;
    Subset source, mapped, target;
    int i, n=m1.getSize();
    Iterator p1, p2;

    try {
      for (p1 = bases1.iterator(); p1.hasNext();) {
	// run basis through map
	source = (Subset)p1.next();
	mapped = new Subset(n);
	for (i=0; i<n; i++) {
	  if (source.get(i)) {
	    mapped.set(mMap[i]);
	  }
	}
	// search for image of basis in bases2
	found = false;
	for (p2 = bases2.iterator(); p2.hasNext();) {
	  target = (Subset)p2.next();
	  if (mapped.equals(target)) {
	    found = true;
	    break;
	  }
	}
	if (!found) {
	  answer = false;
	  break;
	}
      }
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError("Subset exception in checkMap");
    }

    return answer;
  }

  // Combiner interface implementation

  /** 
   * sets the reference collection for this combiner 
   *
   * @param ref reference collection
   * @throws IllegalArgumentException if <code>ref</code> isn't two matroids
   */
  public void setCollection(Collection ref) 
    throws IllegalArgumentException
  {
    if (ref.size() == 0) {
      throw new IllegalArgumentException("No matroids passed");
    }
    
    Object[] matroids = ref.toArray();
    
    if (matroids.length == 1) {
      if (!(matroids[0] instanceof Matroid)) {
	throw new IllegalArgumentException("Not a matroid");
      }
      m1 = (Matroid)matroids[0];
      m2 = (Matroid)matroids[0];
    }
    else {
      if (!((matroids[0] instanceof Matroid) && 
	    (matroids[1] instanceof Matroid)) ) {
	throw new IllegalArgumentException("Not a matroid");
      }
      m1 = (Matroid)matroids[0];
      m2 = (Matroid)matroids[1];
    }
    mUseExpl = true;
//      mStopWithOrbits = true; // don't check all n! if using as Combiner
    checkMatroids();
  }

  /** 
   * returns the reference collection for this constructor 
   *
   * @return reference collection
   */
  public Object getCollection()
  {
    Vector answer = new Vector();
    answer.add(m1);
    answer.add(m2);
    return answer;
  }

  /** 
   * returns the constructed object from the reference collection 
   *
   * @return constructed object
   */
  public Object getConstruction()
  {
    return mExpl;
  }

  /** 
   * returns a description of the algorithm used for the construction 
   *
   * @return algorithm description
   */
  public String getAlgorithmDescription()
  {
    StringBuffer answer = new StringBuffer();
    answer.append("An instance of MatroidIsomorphismChecker ");
    answer.append("accepts as input two ");
    answer.append("Matroid instances and determines whether ");
    answer.append("they are isomorphic.  It first");
    answer.append("performs some common-sense checks, making sure ");
    answer.append("that both matroids have");
    answer.append("the same size and rank.");
    answer.append("If the matroids pass these checks, ");
    answer.append("then the algorithm finds their ");
    answer.append("independent set degree sequences and compares them.  ");
    answer.append("If there is a match,");
    answer.append("it adds in circuit degrees, then spanning set degrees, ");
    answer.append("then cocircuit ");
    answer.append("degrees.  If there is still a match, the algorithm ");
    answer.append("seeks a mapping based");
    answer.append("on the degree sequences that preserves the circuit ");
    answer.append("structure of the");
    answer.append("matroids.");
    return answer.toString();
  }

  /** 
   * returns the algorithm's name 
   *
   * @return algorithm name
   */
  public String getName()
  {
    return new String("Matroid Isomorphism Checker");
  }
}
