package Oid;
import java.util.*;

/**
 * <code>MinorCalculator</code> deletes and contracts elements from a
 * represented matroid and returns the resulting minor.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 2003-06-13
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class MinorCalculator implements SingleConstructor,
					ComplexityProvider
{
  //
  // private data members
  //
  private FiniteFieldMatroid mMatroid;
  private FiniteFieldMatroid mMinor;
  private String mParams;
  private int[] mElement;
  private boolean[] mDelete;

  /** 
   * sets the reference object for this constructor 
   *
   * @param ref reference object
   * @throws IllegalArgumentException if ref is not of the right type
   */
  public void setInstance(Object ref) 
    throws IllegalArgumentException
  {
    if (!(ref instanceof FiniteFieldMatroid)) {
      throw new IllegalArgumentException("This operation requires a " + 
					 "Finite Field Matroid");
    }
    mMatroid = (FiniteFieldMatroid)ref;
    if (mParams instanceof String) {
      calcMinor();
    }
  }

  /** 
   * returns the reference object for this constructor 
   * 
   * @return reference to object passed to this algorithm
   */
  public Object getInstance()
  {
    return mMatroid;
  }

  /** 
   * indicates whether the algorithm requires a parameter
   *
   * @return boolean answer
   */
  public boolean requiresParameter()
  {
    return true;
  }

  /** 
   * sets the algorithm's parameters
   *
   * @param parameters String containing parameters for algorithm
   * @throws IllegalArgumentException if parameters are not correct
   */
  public void setParameters(String parameters)
    throws IllegalArgumentException
  {
    mParams = new String(parameters);
    if (mMatroid instanceof FiniteFieldMatroid) {
      calcMinor();
    }
  }

  /**
   * returns the prompt to display when asking the user to enter the 
   * parameters required for the algorithm
   * 
   * @return String with prompt
   */
  public String getPrompt()
  {
    return new String("Enter the elements to be deleted/contracted " +
		      "in the form d1 d2 ... di / c1 c2 ... cj");
  }

  /** 
   * returns the algorithm's parameters
   *
   * @return String with parameters
   */
  public String getParameters()
  {
    String answer = new String("");

    if (mParams instanceof String) {
      answer = new String(mParams);
    }
    return answer;
  }

  /** 
   * returns the constructed object from the reference object
   *
   * @return object constructed by algorithm
   */
  public Object getConstruction()
  {
    return mMinor;
  }

  /** 
   * returns a description of the algorithm used for the construction
   *
   * @return string description of algorithm
   */
  public String getAlgorithmDescription()
  {
    return new String("Calculates minor of a finite field matroid");
  }

  /** 
   * returns this algorithm's name
   *
   * @return string with algorithm's name
   */
  public String getName()
  {
    return new String("Minor Calculator");
  }

  // private routine calculates minor
  private void calcMinor()
    throws IllegalArgumentException
  {
    // since deleting and contracting elements rearranges the
    // elements, but the routines rearrange the labels accordingly, we
    // start by making a local copy of the matroid; we then delete and
    // contract the elements one by one

    int i, j, index, e, n, size;
    String label;

    mMinor = new FiniteFieldMatroid(mMatroid);
    size = mMinor.getSize();
    
    buildArray();

    n = mElement.length;
    for (i=0; i<n && size>0; i++) {
      // search for element to be deleted/contracted in matroid
      label = String.valueOf(mElement[i]);
      index = -1;
      for (j=0; j<size; j++) {
	if (mMinor.getLabel(j).equals(label)) {
	  index = j;
	  break;
	}
      }
      if (index < 0) {
	throw new IllegalArgumentException(label + " occurs twice in list");
      }

      // delete or contract element
      if (mDelete[i]) {
	mMinor.deleteElement(index);
      }
      else {
	mMinor.contractElement(index);
      }

      // size should now be smaller
      size = mMinor.getSize();
    }
  }

  // private routine builds array of elements and array of booleans 
  // indicating whether to delete (or if false contract) each one
  private void buildArray()
    throws IllegalArgumentException
  {
    boolean bDelete = true;
    String token;
    int e;
    int index=0;
    int size = mMatroid.getSize();
    int n;

    StringTokenizer tokens = new StringTokenizer(mParams, " ");
    n = tokens.countTokens();
    int[] element = new int[n];
    boolean[] delete = new boolean[n];

    for (; tokens.hasMoreElements(); ) {
      token = (String)tokens.nextElement();
      if (token.equals("/")) {
	bDelete = false;
      }
      else {
	try {
	  e = Integer.parseInt(token);
	  if ((e > 0) && (e <= size)) {
	    element[index] = e;
	    delete[index] = bDelete;
	    ++index;
	  }
	  else {
	    throw new IllegalArgumentException("Invalid element index: " +
					       token);
	  }
	}
	catch (NumberFormatException ex) {
	  throw new IllegalArgumentException("Invalid element index: " +
					     token);
	}
      }
    }

    // now copy element and delete into mElement and mDelete
    mElement = new int[index];
    mDelete = new boolean[index];
    
    for (int i=0; i<index; i++) {
      mElement[i] = element[i];
      mDelete[i] = delete[i];
    }
  }

  /** 
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name, or -1 if the method cannot
   * currently be executed
   *
   * @param methodName name of method to check
   * @return estimate of complexity of method
   */
  public long getComplexityEstimate(String methodName)
  {
    return (long)1;
  }
  
  /**
   * returns a value estimating the number of steps required to
   * execute the method with the passed name and argument, or -1 if the 
   * method cannot be executed with the passed argument
   *
   * @param methodName name of method to check
   * @param argument single argument to be passed to method
   * @return estimate of complexity of method with this argument
   */
  public long getComplexityEstimate(String methodName, Object argument)
  {
    return (long)1;
  }

  /**
   * returns a value estimating the number of steps required to 
   * execute the method with the passed name and collection of arguments
   * (which must be placed in correct order for iteration), or -1 if the
   * method cannot be executed with the passed arguments
   *
   * @param methodName name of method to check
   * @param arguments collection of arguments to be passed to method
   * @return estimate of complexity of method with these arguments
   */
  public long getComplexityEstimate(String methodName, Collection arguments)
  {
    return (long)1;
  }

}
