package Oid;
import java.util.*;
import java.io.*;

/**
 * This class enumerates candidate sets of flats from a matroid for
 * modification to produce modular cuts. Specifically, it enumerates
 * the subsets of the set of flats of a matroid that are closed under
 * containment upward; i.e., if a flat is in the set, then every flat
 * containing it is in the set also.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 1 Jul 2003
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class ModCutCandidateEnumeration implements Enumeration
{
  // 
  // private data members
  // 
  private FlatMatroid mMatroid;
  private Enumeration[] mLevelEnum;
  private int mRank;
  private int[] mRankFlatCount;
  private Subset[] mSelFlats;
  private Subset[] mCovered;
  private boolean mHasMoreElements;

  /**
   * Constructs an instance with the passed FlatMatroid
   */ 
  public ModCutCandidateEnumeration(FlatMatroid M)
  {
    mMatroid = M;
    initEnumeration();
  }

  /**
   * returns a value indicating whether there are any more candidates
   * to return
   *
   * @return true if and only if more candidates remain
   */
  public boolean hasMoreElements()
  {
    return mHasMoreElements;
  }

  /**
   * returns the next candidate as a Vector of Flat references
   *
   * @return next candidate
   */
  public Object nextElement()
  throws NoSuchElementException
  {
    Vector answer = (Vector)null;

    if (!mHasMoreElements) {
      throw new NoSuchElementException("There are no more " +
				       "modular cut candidates");
    }

    answer = buildFlatVector();
    findNextElement();

    return (Object)answer;
  }

  //
  // private function initializes enumeration
  //
  private void initEnumeration()
  {
    int i;

    i = mMatroid.getRank();
    mHasMoreElements = (i > 1);
    
    if (mHasMoreElements) {
      mRank = i;
      mLevelEnum = new Enumeration[mRank];
      mRankFlatCount = new int[mRank];
      mSelFlats = new Subset[mRank];
      mCovered = new Subset[mRank];
      
      for (i=mRank-1; i>=2; i--) {
	mRankFlatCount[i] = mMatroid.getFlatsOfRank(i).size();
	mSelFlats[i] = new Subset(mRankFlatCount[i]);
	mCovered[i] = new Subset(mRankFlatCount[i]);
      }

      for (i=0; i<mRankFlatCount[mRank-1]; i++) {
	mCovered[mRank-1].set(i);
      }
      
      for (i=mRank-1; i>=2; i--) {
	mLevelEnum[i] = new SubSubsetEnumeration(mCovered[i]);
      }
    }
  }

  //
  // private function determines next element
  // 
  private void findNextElement()
  {
    int i, j, k;
    
    j = -1;
    for (i=2; i<mRank; i++) {
      if (mLevelEnum[i].hasMoreElements()) {
	j = i;
	break;
      }
    }

    if (j >= 0) {
      mSelFlats[j] = (Subset)mLevelEnum[i].nextElement();
      determineCovered(j);
    }
    else {
      mHasMoreElements = false;
    }
  }

  //
  // private function determines currently covered flats from given
  // (rank-1) down
  //
  private void determineCovered(int rank)
  {
    int i, j, k, r;
    Vector flats;
    Flat f1, f2;
    Iterator p;

    //
    // first identify all the covered (rank-1) flats
    //
    if (rank > 2) {
      mCovered[rank-1].clearAll();
      flats = mMatroid.getFlatsOfRank(rank-1);
      for (i=0; i<mRankFlatCount[rank]; i++) {
	if (mSelFlats[rank].get(i)) {
	  for (p=((Flat)mMatroid.getFlatsOfRank(rank).get(i)).
		 getSubFlats().iterator();
	       p.hasNext(); ) {
	    j = -1;
	    f1 = (Flat)p.next();
	    for (k=0; k<flats.size(); k++) {
	      f2 = (Flat)flats.get(k);
	      if (f1.equals(f2)) {
		j = k;
		break;
	      }
	    }

	    if (j >= 0) {
	      mCovered[rank-1].set(j);
	    }
	    else {
	      throw new InternalError("j is negative");
	    }
	  }
	}
      }
      
      //
      // now reset all the selected flat subsets and covered flat subsets
      // below
      //
      for (r=rank-2; r>=2; r--) {
	mCovered[r].clearAll();
      }

      for (r=rank-1; r>=2; r--) {
	//
	// we want a SubSubsetEnumeration instance, even though 
	// all but one will be empty
	//
	mLevelEnum[r] = new SubSubsetEnumeration(mCovered[r]);
	if (mLevelEnum[r].hasMoreElements()) {
	  mSelFlats[r] = (Subset)mLevelEnum[r].nextElement();
	}
	else {
	  mSelFlats[r].clearAll();
	}
      }
    }
  }

  //
  // private function builds set of flats from current selection
  //
  private Vector buildFlatVector()
  {
    int i, rank;
    Vector rankFlats;
    Vector answer = new Vector();
    
    for (rank=2; rank<mRank; rank++) {
      rankFlats = mMatroid.getFlatsOfRank(rank);
      for (i=0; i<mRankFlatCount[rank]; i++) {
	if (mSelFlats[rank].get(i)) {
	  answer.add(rankFlats.get(i));
	}
      }
    }
    
    // modular cut always includes the top flat
    answer.add(mMatroid.getFlatsOfRank(mRank).get(0));
    return answer;
  }
}
