package Oid;
import java.util.*;


/**
 * Generates a list of the modular cuts for a FlatMatroid. Each
 * modular cut is returned as a Vector of Flat references.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 24 Sep 2003
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class ModularCutAlgorithm implements Enumeration {

  private FlatMatroid mMatroid;
  private int mRank;
  private boolean mHasMore;
  private Vector mNextElement;
  private ModCutCandidateEnumeration mCandEnum;

  /**
   * Constructs algorithm instance with a FlatMatroid
   *
   * @param M matroid represented by lattice of flats
   */
  public ModularCutAlgorithm(FlatMatroid M) {
    mMatroid = M;
    mCandEnum = new ModCutCandidateEnumeration(mMatroid);
    gotoNextElement();
  }

  /**
   * Returns a boolean value indicating whether there are any modular cuts
   * left.
   *
   * @return true if there are more modular cuts
   */
  public boolean hasMoreElements() {
    return mHasMore;
  }

  /**
   * Returns the next element in the enumeration, a vector of Flat references
   * with the members of the next modular cut.
   *
   * @return Vector of Flats
   */
  public Object nextElement() {
    Vector answer = mNextElement;
    
    gotoNextElement();

    return mNextElement;
  }

  //
  // goes through candidate enumeration until the end or the next modular
  // cut
  //
  private void gotoNextElement() {
    Vector candidate = (Vector)null;
    for (; mCandEnum.hasMoreElements(); ) {
      candidate = (Vector)mCandEnum.nextElement();
      if (checkCandidate(candidate)) {
	break;
      }
    }
    
    if (mCandEnum.hasMoreElements()) {
      mNextElement = candidate;
      mHasMore = true;
    }
    else {
      mHasMore = false;
    } 
  }

  //
  // checks a candidate against the one condition not guaranteed by the 
  // candidate enumeration: that any flat not in the cut be covered by at most
  // one flat in the cut
  // 
  private boolean checkCandidate(Vector candidate) {
    boolean answer = true;
    int rank, covers;
    Vector allFlats, rankFlats;
    Iterator pFlat, pSuper;
    Flat flat, superFlat;

    allFlats = mMatroid.getFlats();
    for (rank=2; rank<mRank; rank++) {
      rankFlats = (Vector)allFlats.get(rank);
      for (pFlat = rankFlats.iterator(); pFlat.hasNext(); ) {
	flat = (Flat)pFlat.next();
	if (!candidate.contains(flat)) {
	  covers = 0;
	  for (pSuper = flat.getSuperFlats().iterator(); pSuper.hasNext(); ) {
	    superFlat = (Flat)pSuper.next();
	    if (candidate.contains(superFlat)) {
	      ++covers;
	    }
	  }
	  if (covers > 1) {
	    answer = false;
	    break;
	  }
	}
      }
      if (!answer) {
	break;
      }
    }

    return answer;
  }
}
