package Oid;
import java.util.*;

/** 
 * An instance PermutationEnumeration can generate the permutations of
 * an n-set.  It is based on an algorithm of Dijkstra, implemented in C 
 * by Glenn C. Rhoads (http://remus.rutgers.edu/~rhoads/Code/perm_lex.c)
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 23 Feb 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class PermutationEnumeration implements Enumeration
{
  //
  // private data members
  // 
  private int mSize;
  private int[] mPerm;
  private boolean mHasMoreElements;

  /**
   * constructs an enumeration with base set size <code>n</code>
   *
   * @param n size of base set
   */
  public PermutationEnumeration(int n)
    throws IllegalArgumentException
  {
    int i;

    if (n<=0) {
      throw new IllegalArgumentException("Non-positive set size");
    }
    mSize = n;
    mPerm = new int[mSize+1];
    
    for (i=0; i<=mSize; i++) {
      mPerm[i] = i;
    }
    
    mHasMoreElements = true;
  }

  /**
   * tests if this enumeration has more permutations
   *
   * @return <code>true</code> if there are more permutations; 
   * <code>false</code> otherwise 
   */
  public boolean hasMoreElements()
  {
    return mHasMoreElements;
  }

  /**
   * returns the next permutation of the base set
   *
   * @return array of <code>int</code> values with permutation
   * @throws NoSuchElementException if no more permutations exist
   */
  public Object nextElement()
    throws NoSuchElementException
  {
    if (!mHasMoreElements) {
      throw new NoSuchElementException("No more permutations");
    }

    int[] answer = new int[mSize];
    int i, j, r, s, temp, n=mSize;

    for (i=0; i<mSize; i++) {
      answer[i] = mPerm[i+1]-1;
    }

    // generate next permutation
    i = n-1;
    while (mPerm[i] > mPerm[i+1]) {
      i--;
    }
      
    j = n;
    while (mPerm[i] > mPerm[j]) {
      j--;
    }
      
    temp = mPerm[i];
    mPerm[i] = mPerm[j];
    mPerm[j] = temp;
      
    r = n;
    s = i+1;
    while (r > s) {
      temp = mPerm[r];
      mPerm[r] = mPerm[s];
      mPerm[s] = temp;
      r--; s++;
    }

    mHasMoreElements = (i != 0);

    return answer;
  }

  /** 
   * this method resets the enumeration to the first permutation
   */
  public void reset()
  {
    int i;
    for (i=0; i<=mSize; i++) {
      mPerm[i] = i;
    }
    mHasMoreElements = true;
  }
}
