package Oid;
import java.util.*;
import java.io.*;

/**
 * An instance of <code>PrimeFactorization</code> factors integers into
 * primes, returning two-dimensional arrays of primes and exponents.
 * 
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 23 Aug 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class PrimeFactorization
{
  /**
   * Factors n into prime factors and returns an array of the prime 
   * divisors of n and the powers to which they divide n; that is, it
   * returns an array p such that n = p[0][0] ^ p[0][1] * p[1][0] ^ p[1][1] *
   * ... * p[k][0] ^ p[k][1], and each p[i][0] is a distinct prime.
   *
   * @param n number to factor
   * @returns two-dimensional array of primes and exponents
   */
  public int[][] getFactors(int n)
    throws ArithmeticException
  {
    Vector powers = new Vector();
    Vector primes = new Vector();
    int k, p;
    int m = (n > 0) ? n : -n;

    if (0 == m) {
      throw new ArithmeticException("cannot factor 0");
    }

    //
    // check for powers of 2
    //
    p = 2;
    k = 0;
    while (0 == m % p) {
      ++k;
      m /= p;
    }
    if (k > 0) {
      powers.add(new Integer(k));
      primes.add(new Integer(p));
    }

    //
    // loop through odd values; since loop proceeds in
    // ascending order, only primes will divide m
    //
    for (p=3; (p<=m) && (m>1); p+=2) {
      k = 0;
      while (0 == m % p) {
	++k;
	m /= p;
      }
      if (k > 0) {
	powers.add(new Integer(k));
	primes.add(new Integer(p));
      }
    }

    // 
    // now make array
    //
    int[][] ans = new int[powers.size()][2];
    for (p=0; p<powers.size(); p++) {
      ans[p][0] = ((Integer)primes.get(p)).intValue();
      ans[p][1] = ((Integer)powers.get(p)).intValue();
    }
    
    return ans;
  }

  /**
   * main method allows this class to be used stand-alone
   *
   * @param arg first element should be numeric and have integer to factor
   */
  public static void main(String[] arg)
  {
    int i,k,n;
    PrimeFactorization pf = new PrimeFactorization();
    int[][] p;

    if (arg.length > 0) {
      try {
	n = Integer.valueOf(arg[0]).intValue();
	p = pf.getFactors(n);
	k = p.length;
	System.out.print(n + " = ");
	for (i=0; i<k; i++) {
	  System.out.print("(" + p[i][0] + "^" + p[i][1] + ")");
	}
	System.out.println();
      }
      catch (Exception ex) {
	System.out.println(ex.getMessage());
      }
    }
  }
}
