package Oid;
import java.util.*;

/**
 * An instance of <code>SizeRankMatrixCalculator</code> determines,
 * for a <code>Matroid</code> instance, two-dimensional matrix S =
 * {s[i][j]}, where s[i][j] is the number of subsets of the base set
 * of the matroid of size i and rank j.  It also can generate the rank
 * generating, Tutte and chromatic polynomials for the matroid from
 * this matrix, returning them as instances of <code>IntPoly</code> or
 * <code>IntPoly2</code> as appropriate.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 15 June 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class SizeRankMatrixCalculator
{
  //
  // private data members
  // 
  private DefaultIndependentSetAlgorithm mIndAlg;
  private int[][] mSRMat;
  private int[] mConnCoeff; // connectivity polynomial coefficients
  private Matroid mMatroid;
  private IntPoly2 mRankGenPoly;
  private IntPoly2 mTuttePoly;
  private IntPoly mChromaticPoly;
  private IntPoly mConnPoly;

  /**
   * default constructor
   */
  public SizeRankMatrixCalculator()
  {
    mIndAlg = new DefaultIndependentSetAlgorithm();
  }

  /**
   * this method takes the passed matroid and calculates its size-rank matrix
   * @param M matroid
   */
  public void setMatroid(Matroid M)
  {
    mMatroid = M;
    mIndAlg.setInstance(mMatroid);
    SubsetEnumeration pSubset;
    Subset s, sc;
    int n = mMatroid.getSize();
    int r = mMatroid.getRank();
    int i,j,k,rs,rc,cni;
    boolean found;
    Iterator p;

    mSRMat = new int[n+1][r+1];
    for (i=0; i<=n; i++) {
      for (j=0; j<=r; j++) {
	mSRMat[i][j] = 0;
      }
    }

    mConnCoeff = new int[n + 2*r + 1];
    for (i=0; i<=n+2*r; i++) {
      mConnCoeff[i] = 0;
    }

    // make an array of Vectors independent sets and set the bases
    // to be the entry at r
    Vector[] indSets = new Vector[r+1];
    Vector v = mIndAlg.getSubsetListVector();
    for (i=0; i<=r; i++) {
      indSets[i] = (Vector)v.get(i);
    }

    // the whole base set must have rank r and the empty set has rank 0
    mSRMat[n][r] = 1;
    mSRMat[0][0] = 1;

    mConnCoeff[n] = 2;

    int limit = n/2;

    try {
      for (k=1; k<=limit; k++) {
	for (pSubset = new SubsetEnumeration(n,k); 
	     pSubset.hasMoreElements();) {
	  s = (Subset)pSubset.nextElement();
	  sc = s.complement();
	  rs = getRank(s, indSets, n, r);
	  rc = getRank(sc, indSets, n, r);
	  cni = n + 2 * (r - rs - rc);
	  mSRMat[k][rs]++;
	  if (2 * k < n) {
	    // only contribute complement to mSRMat if we aren't at the 
	    // halfway point
	    mConnCoeff[cni] += 2; // contributes for both s and sc
	    mSRMat[n-k][rc]++;
	  }
	  else {
	    mConnCoeff[cni]++; // contributes only for s
	  }
	}
      }

      // now generate the polynomials
      generatePolynomials();
    }
    catch (Exception e) {
      // take care of Subset exceptions
      throw new InternalError(e.toString());
    }
  }

  //
  // private method looks up the rank of a subset given a matrix
  // of vectors of independent sets and the size and rank of the matroid
  //
  private int getRank(Subset s, Vector[] indSets, int n, int r)
  {
    int k = s.numElements();
    int i,j;
    Subset t;
    Iterator pIndSet;
    boolean found;
    int answer = 0;

    if (k > 0) {
      j = (k >= r) ? r : k;
      found = false;
      for (i=j; i>=1; i--) {
	for (pIndSet = indSets[i].iterator(); pIndSet.hasNext();) {
	  if (s.subsetContains((Subset)pIndSet.next())) {
	    found = true;
	    answer = i;
	    break;
	  }
	}
	if (found) {
	  break;
	}
      }
    }

    return answer;
  }

  /**
   * this method returns the size-rank matrix previously calculated
   * @return two-dimensional matrix with numbers of subsets
   */
  public int[][] getSizeRankMatrix()
  {
    return mSRMat;
  }

  /**
   * returns the rank generating polynomial of the matroid
   * @return IntPoly2 reference to polynomial
   */
  public IntPoly2 getRankGeneratingPolynomial()
  {
    IntPoly2 answer = (IntPoly2)null;
    
    if (mMatroid instanceof Matroid) {
      answer = mRankGenPoly;
    }

    return answer;
  }

  /**
   * returns the chromatic polynomial of the matroid
   * @return IntPoly reference to polynomial
   */
  public IntPoly getChromaticPolynomial()
  {
    IntPoly answer = (IntPoly)null;

    if (mMatroid instanceof Matroid) {
      answer = mChromaticPoly;
    }

    return answer;
  }

  /**
   * returns the Tutte polynomial of the matroid
   * @return IntPoly2 reference to polynomial
   */
  public IntPoly2 getTuttePolynomial()
  {
    IntPoly2 answer = (IntPoly2)null;

    if (mMatroid instanceof Matroid) {
      answer = mTuttePoly;
    }

    return answer;
  }

  /** 
   * returns the connectivity polynomial of the matroid
   * @return IntPoly reference to polynomial
   */
  public IntPoly getConnectivityPolynomial()
  {
    IntPoly answer = (IntPoly)null;

    if (mMatroid instanceof Matroid) {
      answer = mConnPoly;
    }

    return answer;
  }

  //
  // private method generates the rank generating, chromatic and Tutte
  // polynomials
  //
  private void generatePolynomials()
  {
    int i,j,k,l,m=mMatroid.getRank(), n=mMatroid.getSize();
    int d,limit;
    int[][] coeff = new int[m+1][n+1]; // coefficients for rank gen poly
    int[] chCoeff = new int[m+1];      // coefficients for chromatic poly
    
    for (i=0; i<=m; i++) {
      chCoeff[i] = 0;
      for (j=0; j<=n; j++) {
	coeff[i][j] = 0;
      }
    }
    
    // get coefficients
    d = 1;
    for (i=0; i<=n; i++) {
      limit = (i > m) ? m : i;
      for (j=0; j<=limit; j++) {
	// coefficient of x^k * y^l in the rank generating polynomial is
	// the number of subsets A with r(M) - r(A) = k, |A| - r(A) = l
	k = m - j;
	l = i - j;
	coeff[k][l] += mSRMat[i][j];
	chCoeff[k] += d * mSRMat[i][j];
      }
      d = -d;
    }

    // get rank generating polynomial and chromatic polynomial
    mRankGenPoly = new IntPoly2(coeff);
    mChromaticPoly = new IntPoly(chCoeff);

    // get Tutte polynomial
    int[] tSubCoeff = new int[] {-1, 1};
    IntPoly tSub = new IntPoly(tSubCoeff); // x-1 or y-1
    IntPoly2 tPoly = mRankGenPoly.substitute1(tSub); // substitute x-1 for x
    mTuttePoly = tPoly.substitute2(tSub); // substitute y-1 for y

    // get connectivity polynomial--use coefficients then substitute x-1 for x
    IntPoly cSub = new IntPoly(mConnCoeff);
    mConnPoly = cSub.substitute(tSub);
  }

}
