package Oid;
import java.util.*;

/**
 * SubSubsetEnumeration, which implements the <em>Enumeration</em>
 * interface, enumerates all of the Subsets which are contained in a
 * given Subset, of all sizes.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 23 Jun 2003
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class SubSubsetEnumeration implements Enumeration
{
  //
  // private data members
  //
  private Subset mU;           // universe--subset we are enumerating subs of
  private Subset mNext;        // next subset
  private boolean mHasMore;    // are there more subsets?
  private boolean mWrapAround; // if true don't throw an exception at end

  /**
   * Creates a new instance with the given subset
   *
   * @param U Subset reference
   * @throws IllegalArgumentException if U has no elements
   */
  public SubSubsetEnumeration(Subset U)
    throws IllegalArgumentException
  {
    mU = U;
    mNext = new Subset(mU.getSize());
    mHasMore = (mU.numElements() > 0);
    mWrapAround = false;
  }

  /**
   * Creates a new instance with the given subset and preference for
   * "wrap-around": if false, no exception is thrown when the caller
   * passed the last subset, but the subset simply passes back to the
   * empty set and starts over.
   *
   * @param U Subset reference
   * @param wrapAround "wrap-around" preference
   * @throws IllegalArgumentException if U is empty
   */
  public SubSubsetEnumeration(Subset U, boolean wrapAround)
    throws IllegalArgumentException
  {

    mU = U;
    mNext = new Subset(mU.getSize());
    mHasMore = (mU.numElements() > 0);
    mWrapAround = wrapAround;
  }

  public boolean hasMoreElements()
  {
    return mHasMore;
  }

  public Object nextElement()
    throws NoSuchElementException
  {
    Subset result;
    int i, index, n = mU.getSize();

    if (!mWrapAround && !mHasMore) {
      throw new NoSuchElementException("No more elements");
    }

    result = new Subset(mNext);
    index = -1;
    // find last element in U not subset, set it and set all followers to 0
    for (i=n-1; i>=0; i--) {
      if (mU.get(i) && !mNext.get(i)) {
	index = i;
	break;
      }
    }

    if (index >= 0) {
      mNext.set(index);
    }
    else {
      mHasMore = false;
    }

    for (i=index+1; i<n; i++) {
      mNext.clear(i);
    }
      
    return result;
  }
}
