package Oid;
import java.util.*;

/** 
 * Subset provides methods to manipulate subsets of a fixed-size base set.
 * 
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 11 Aug 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class Subset implements Cloneable {
  //
  // private data members
  //
  private BitSet mSet;          // holds representation of set
  private int mSize;            // size of base set
  static final int mDEFAULT_SIZE = 63; // default base set size

  //
  // constructors
  //

  /** 
   * creates a new Subset with the specified base set size 
   *
   * @param newSize base set size of new Subset
   * @throws IllegalArgumentException if newSize is not positive
   */
  public Subset(int newSize) 
  throws IllegalArgumentException
  {
    if (newSize > 0) {
      mSize = newSize;
    }
    else {
      throw new IllegalArgumentException();
    }
    mSet = new BitSet(mSize);
  }

  /** 
   * creates a new Subset with the default base set size of 63 
   */
  public Subset() {
    mSize = mDEFAULT_SIZE;
    mSet = new BitSet(mSize);
  }

  /** 
   * creates a new Subset with the indicated base set size, and includes the 
   * specified element in the set (i.e., creates a one-element Subset)
   *
   * @param newSize base set size of new Subset
   * @param setElement index of single element in new Subset
   * @throws IllegalArgumentException if newSize is not positive
   * @throws ArrayIndexOutOfBoundsException if setElement is less than
   * zero or not less than newSize
   */
  public Subset(int newSize, int setElement)
      throws IllegalArgumentException, ArrayIndexOutOfBoundsException
  {
    if (newSize <= 0) {
      throw new IllegalArgumentException();
    }
    mSize = newSize;
    mSet = new BitSet(mSize);
    checkIndex(setElement);
    mSet.set(setElement);
  }

  /** 
   * creates a new Subset as a copy of the passed one
   * 
   * @param s Subset to copy
   */
  public Subset(Subset s) 
  {
    int i;
    mSize = s.getSize();
    mSet = new BitSet(mSize);
    for (i=0; i<mSize; i++) {
      try {
	if (s.get(i)) {
	  mSet.set(i);
	}
      }
      catch (ArrayIndexOutOfBoundsException e) {
	throw new InternalError("Invalid subset index exception in Subset");
      }
    }
  }

  /**
   * constructs a subset by parsing the specified string
   *
   * @param str String containing standard-form subset
   * @param size size of set for subset
   * @param oneUp True if and only if the values in the string start with 1
   * @throws IllegalArgumentException if str does not contain a legitimate
   * subset description, or if it contains and index too large or small
   * @throws NumberFormatException if a non-numeric value is found in the
   * string representation (although most are ignored)
   */
  public Subset(String str, int size, boolean oneUp) 
    throws IllegalArgumentException, NumberFormatException
  {
    if (size <= 0) {
      throw new IllegalArgumentException("set size must be positive");
    }

    mSize = size;
    mSet = new BitSet(mSize);

    //
    // create a new string, filtering out everything except for numbers and
    // spaces, and eliminating duplicate spaces--also change commas to 
    // spaces just in case
    //
    StringBuffer str2 = new StringBuffer("");
    String token;
    int n = str.length();
    int i;
    char b='\0',c;
    for (i=0; i<n; i++) {
      c = str.charAt(i);
      c = (c == ',') ? ' ' : c;
      if (((c == ' ') && (b != ' ')) || ((c >= '0') && (c <= '9'))) {
	str2.append(c);
      }
    }
    
    // now create a tokenizer and parse the simplified string
    for (StringTokenizer t = new StringTokenizer(str2.toString(), " ");
	 t.hasMoreElements(); ) {
      token = (String)t.nextElement();
      i = Integer.parseInt(token);
      i = (oneUp) ? i-1 : i;
      if ((i < 0) || (i >= size)) {
	throw new IllegalArgumentException("index " + token +" out of bounds");
      }
      else {
	mSet.set(i);
      }
    }
  }

  /** 
   * overrides Object.clone() and creates an independent copy of the Subset
   * 
   * @return copy of this Subset
   */
  public Object clone() 
  {
    Subset theAnswer;
    try {
      theAnswer = (Subset)super.clone();
      // must also clone this subset's BitSet
      theAnswer.mSet = (BitSet)mSet.clone();
    }
    catch (CloneNotSupportedException c) {
      // this "can't happen"
      throw new InternalError("All subclasses of Subset must be Cloneable");
    }

    return (Object)theAnswer;
  }

  //
  // set/gets
  //

  /** 
   * returns the size of the base set 
   *
   * @return size of base set
   */
  public int getSize() 
  {
    return mSize;
  }

  /** 
   * returns a reference to the underlying BitSet for this Subset 
   *
   * @return BitSet reference
   */
  public BitSet getBitSet() 
  {
    return mSet;
  }

  /** 
   * indicates that the element with the passed index is in the set 
   *
   * @param theIndex index of element to set
   */
  public void set(int theIndex) 
  throws ArrayIndexOutOfBoundsException
  {
    checkIndex(theIndex);
    mSet.set(theIndex);
  }

  /** 
   * indicates that the element with the passed index is not in the set 
   *
   * @param theIndex index of element to clear
   */
  public void clear(int theIndex) 
  throws ArrayIndexOutOfBoundsException
  {
    checkIndex(theIndex);
    mSet.clear(theIndex);
  }
  
  /** 
   * empties the Subset 
   */
  public void clearAll()
  {
    int i;
    for (i=0; i<mSize; i++) {
      mSet.clear(i);
    }
  }

  /** 
   * returns the membership status of the element with the passed index 
   *
   * @param theIndex index of element to check
   * @return boolean indicating membership status of element
   */
  public boolean get(int theIndex) 
  throws ArrayIndexOutOfBoundsException
  {
    checkIndex(theIndex);
    return mSet.get(theIndex);
  }

  /** 
   * returns the number of elements in the subset 
   *
   * @return integer with number of elements
   */
  public int numElements() 
  {
    int i, n=0;
    for (i=0; i<mSize; i++) {
      if (mSet.get(i)) {
      	n++;
      }
    }
    return n;
  }

  /** 
   * returns the index of the first element in the subset, or -1 if the Subset
   * is empty
   *
   * @return index of first set element
   */
  public int firstElement() 
  {
    int i;
    for (i=0; (i<mSize) && !mSet.get(i); i++) { }
    
    if (i == mSize) {
      i = -1;
    }
    return i;
  }
  
  /** 
   * returns the length of the underlying BitSet 
   *
   * @return BitSet length
   */
  public int length() 
  {
    return mSet.length();
  }

  /** 
   * converts the underlying BitSet to a string 
   *
   * @return String representation of underlying BitSet (0-based)
   */
  public String toString()
  {
    return mSet.toString();
  }

  /** 
  * converts the underlying BitSet to a string, but does
  * it 1-up
  *
  * @param oneUp if true, returns 1-based string
  * @return string representation of subset
  */
  public String toString(boolean oneUp)
  {
    String answer;
    int i, j;
    boolean bNonEmpty = false;
    StringBuffer bufAns;

    if (oneUp) {
      bufAns = new StringBuffer("{");
      for (i=0; i<mSize; i++) {
	if (mSet.get(i)) {
	  j = i+1;
	  bufAns.append(j + ", ");
	  bNonEmpty = true;
	}
      }
      if (bNonEmpty) {
	answer = bufAns.substring(0, bufAns.length()-2) + "}";
      }
      else {
	answer = new String("{}");
      }
    }
    else {
      answer = mSet.toString();
    }

    return answer;
  }

  /**
   * converts the underlying BitSet to a string, 1-up without formatting
   *
   * @return string representation of subset
   */
  public String toStringNoFmt()
  {
    int i;
    StringBuffer sb = new StringBuffer();

    for (i=0; i<mSize; i++) {
      if (mSet.get(i)) {
	sb.append((i+1));
	sb.append(" ");
      }
    }

    return sb.toString();
  }


  /** 
   * converts the Subset to a String using the passed array of element labels
   *
   * @param elementLabel array of strings with element labels
   * @return String representation of Subset using labels
   * @throws IllegalArgumentException if array size is not correct
   */
  public String toString(String[] elementLabel)
    throws IllegalArgumentException
  {
    if (elementLabel.length != mSize) {
      throw new IllegalArgumentException("Invalid element label array");
    }

    String answer;
    int i;
    boolean bNonEmpty = false;
    StringBuffer bufAns;

    bufAns = new StringBuffer("{");
    for (i=0; i<mSize; i++) {
      if (mSet.get(i)) {
	bufAns.append(elementLabel[i] + ", ");
	bNonEmpty = true;
      }
    }
    if (bNonEmpty) {
      answer = bufAns.substring(0, bufAns.length()-2) + "}";
    }
    else {
      answer = new String("{}");
    }
    
    return answer;
  }

  /**
   * converts the Subset ot a String using the passed array of element numbers
   *
   * @param elementNumber array of integers with element numbers
   * @return String representation of Subset using element numbers
   * @throws IllegalArgumentException if array size is not correct
   */
  public String toString(int[] elementNumber)
    throws IllegalArgumentException
  {
    if (elementNumber.length != mSize) {
      throw new IllegalArgumentException("Invalid element number array");
    }

    String answer;
    int i;
    boolean bNonEmpty = false;
    StringBuffer bufAns;

    bufAns = new StringBuffer("{");
    for (i=0; i<mSize; i++) {
      if (mSet.get(i)) {
	bufAns.append(String.valueOf(elementNumber[i]) + ", ");
	bNonEmpty = true;
      }
    }
    if (bNonEmpty) {
      answer = bufAns.substring(0, bufAns.length()-2) + "}";
    }
    else {
      answer = new String("{}");
    }
    
    return answer;
  }

  //
  // equality check and subset operations 
  //

  /** 
   * overrides Object.equals for Subsets 
   *
   * @param o object to compare
   * @return boolean value (true if and only if o is a subset, has the
   * same base set size, and has the same elements)
   */
  public boolean equals(Object o) {

    int i,n;
    
    if(!(o instanceof Subset)) {
      return false;
    }
    
    Subset s = (Subset)o;
    n = s.getSize();
    if (n != mSize) {
      return false;
    }
    
    boolean ans = true;
    BitSet theSet = s.getBitSet();
    try {
      for (i=0; i<n && ans; i++) {
	ans = ans && (mSet.get(i) == s.get(i));
      }
    }
    catch (ArrayIndexOutOfBoundsException e) {
      // this "can't happen"
      throw new InternalError("Invalid index in Subset.equals()");
    }
    
    return ans;
  }

  /** 
   * checks for subset equality to another subset 
   *
   * @param theSubset the other subset to check
   * @return boolean value (true if and only if theSubset has the same base
   * set size and elements as this)
   */
  public boolean subsetEquals(Subset theSubset) 
  {
    int i, n;
    n = theSubset.getSize();
    if (n != mSize) {
      return false;
    }
    
    boolean ans = true;
    BitSet theSet = theSubset.getBitSet();
    for (i=0; i<n && ans; i++) {
      ans = ans && (mSet.get(i) == theSet.get(i));
    }
    return ans;
  }

  /** 
   * determines whether this Subset contains the passed Subset 
   *
   * @param theSubset subset to compare
   * @return boolen value (true if and only if theSubset has the same base
   * set size and every element of theSubset is an element of this)
   */
  public boolean subsetContains(Subset theSubset)
  {
    int i,n;
    n = theSubset.getSize();
    if (n != mSize) {
      return false;
    }

    boolean ans = true;
    BitSet theSet = theSubset.getBitSet();
    for (i=0; i<n && ans; i++) {
      ans = ans && (!theSet.get(i) || mSet.get(i));
    }

    return ans;
  }

  /** 
   * intersects this Subset with another 
   * 
   * @param theSubset subset to intersect (must have same base set size)
   * @throws IllegalArgumentException if theSubset does not have the same
   * base set size as this
   */
  public void and(Subset theSubset) 
  throws IllegalArgumentException
  {
    checkSize(theSubset);
    BitSet theSet = theSubset.getBitSet();
    mSet.and(theSet);
  }

  /** 
   * sets this subset to be the union of itself with another 
   * 
   * @param theSubset subset to calculate union (must have same base set size)
   * @throws IllegalArgumentException if theSubset does not have the same
   * base set size as this
   */
  public void or(Subset theSubset) 
  throws IllegalArgumentException
  {
    checkSize(theSubset);
    BitSet theSet = theSubset.getBitSet();
    mSet.or(theSet);
  }

  /** 
   * sets this Subset to the set difference of this with another Subset 
   *
   * @param theSubset the other subset (must have the same base set size)
   * @throws IllegalArgumentException if theSubset does not have the same
   * base set size as this
   */
  public void andNot(Subset theSubset) 
  throws IllegalArgumentException
  {
    checkSize(theSubset);
    BitSet theSet = theSubset.getBitSet();
    mSet.andNot(theSet);
  }

  /** 
   * sets this Subset to exclusive-or of this with another Subset 
   *
   * @param theSubset subset to compare (must have the same base set size)
   * @throws IllegalArgumentException if theSubset does not have the same
   * base set size as this
   */
  public void xor(Subset theSubset) 
  throws IllegalArgumentException
  {
    checkSize(theSubset);
    BitSet theSet = theSubset.getBitSet();
    mSet.xor(theSet);
  }

  /** 
   * returns the complement of this Subset as a new Subset 
   *
   * @return Subset with same base set size containing complement of this
   */
  public Subset complement()
  {
    Subset answer;
    int i;
    try {
      answer = new Subset(mSize);
      for (i=0; i<mSize; i++) {
	if (!(this.get(i))) {
	  answer.set(i);
	}
      }
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError("Internal error in Subset.complement");
    }
    return answer;
  }

  /** 
   * returns a new Subset of this Subset, whose elements are selected
   * from the elements of this Subset according to the elements of the 
   * passed indexSubset, whose size should be the same as this.numElements().
   * For example, if the current subset is the subset {3, 4, 5} of the 
   * ground set {0, 1, 2, 3, 4, 5, 6, 7} and indexSubset is the subset
   * {0, 2} of the ground set {0, 1, 2}, this method would return
   * the subset {3, 5} of the ground set {0, 1, 2, 3, 4, 5, 6, 7}.
   * 
   * @param indexSubset subset of indices to select from this
   * @throws IllegalArgumentException if indexSubset does not have 
   * this.numElements() as its base set size
   */
  public Subset getSubSubset(Subset indexSubset)
  throws IllegalArgumentException
  {
    if (indexSubset.getSize() != this.numElements()) {
      throw new IllegalArgumentException("index subset has incorrect " +
					 "set size");
    }

    int i=0, j=0;
    Subset answer;

    try {
      answer = new Subset(mSize);
      for (i=0; i<mSize; i++) {
	if (this.get(i)) {
	  if (indexSubset.get(j)) {
	    answer.set(i);
	  }
	  j++;
	}
      }
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError("internal error in getSubSubset");
    }
    return answer;
  }
    
  //
  // private utility functions
  //
  private void checkIndex(int theIndex)
  throws ArrayIndexOutOfBoundsException
  {
    if ((theIndex < 0) || (theIndex >= mSize)) {
      throw new ArrayIndexOutOfBoundsException();
    }
  }

  private void checkSize(Subset theSubset) 
  throws IllegalArgumentException
  {
    if (mSize != theSubset.getSize()) {
      throw new IllegalArgumentException();
    }
  }  
}
