package Oid;
import java.io.*;
import java.util.*;

/**
 * This Enumeration implementation enumerates the k-subsets of an n-set in
 * lexicographic order. It is loosely based on the algorithm described in: 
 * Reingold, Nievergelt and Deo (1977) <i>Combinatorial algorithms : 
 * theory and practice</i>. Englewood Cliffs, N.J. : Prentice-Hall.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 21 Sep 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class SubsetEnumeration implements Enumeration {
  private Subset mSubset;
  private int mSubsetSize;
  private int mSetSize;
  private boolean mHasMore;
  private boolean mAllSubsets;
  static final int mDEFAULT_SubsetSize = 1;
  static final int mDEFAULT_SetSize = 10;
  
  // constructors
  /** 
   * the default constructor uses the default base set size of 10 and
   * enumerates all subsets
   */
  public SubsetEnumeration() 
  {
    mSubset = new Subset(mDEFAULT_SetSize);
    mAllSubsets = true;
    initSet();
  }

  /**
   * this constructor uses the indicated base set size and enumerates all
   * subsets
   *
   * @param n base set size
   * @throws IllegalArgumentException if <code>n</code> is invalid
   */
  public SubsetEnumeration(int n)
    throws IllegalArgumentException
  {
    mAllSubsets = true;
    mSetSize = n;
    initSet();
  }

  /** 
   * this constructor allows you to specify the base set size <i>n</i> and
   * the subset size <i>k</i>.
   *
   * @param n base set size
   * @param k subset size
   * @throws IllegalArgumentException if <code>n</code> or <code>k</code> is
   * invalid
   */
  public SubsetEnumeration(int n, int k)
    throws IllegalArgumentException
  {
    mSubset = new Subset(n);
    mSetSize = n;
    mSubsetSize = k;
    mAllSubsets = false;
    initSet();
  }

  /** 
   * returns the base set size
   *
   * @return base set size
   */
  public int getBaseSetSize()
  {
    return mSetSize;
  }

  /**
   * returns the subset size being generated, or -1 if all subsets are
   * being enumerated
   *
   * @return subset size
   */
  public int getSubsetSize()
  {
    int answer = -1;
    if (!mAllSubsets) {
      answer = mSubsetSize;
    }
    return answer;
  }

  /** 
   * implements Enumeration hasMoreElements method; determines whether there
   * are more subsets left to enumerate
   *
   * @return <code>true</code> if and only the enumeration has at least one
   * more subset; <code>false</code> otherwise
   */
  public boolean hasMoreElements() 
  {
    return mHasMore;
  }

  /** 
   * returns the next subset in the list, if there is one
   *
   * @return the next subset in the list
   * @throws NoSuchElementException if there are no more subsets
   */
  public Object nextElement() 
    throws NoSuchElementException
  {
    Object theAnswer;
    if (mHasMore) {
      theAnswer = mSubset.clone();

      nextSet();
    }
    else {
      throw new NoSuchElementException();
    }
    return theAnswer;
  }

  // generates the next subset in the list
  private void nextSet()
  {
    int n = mSetSize;
    int k = mSubsetSize;
    int j = n-1;
    int m=0;
    int i;

    if (mAllSubsets) {
      //
      // just find last element that is not set, set it, and clear 
      // everything after
      //
      i = -1;
      for (j=n-1; j>=0; j--) {
	if (!mSubset.get(j)) {
	  i = j;
	  break;
	}
      }
      if (i < 0) {
	mHasMore = false;
      }
      else {
	mSubset.set(i);
	for (j=i+1; j<n; j++) {
	  mSubset.clear(j);
	}
      }
    }
    else {
      if ((k == n) || (k == 0)) {
	mHasMore = false;
      }
      else {
	try {
	  // determine the last element that is not in its final place
	  while(mSubset.get(j) && j >= n-k) {
	    m++;
	    j--;
	  }

	  if (j >= n-k) {
	    // this is not the last subset; find a one and move it up
	    while(j >= 0 && !mSubset.get(j)) {
	      j--;
	    }
	    if (j >= 0) {
	      mSubset.clear(j++);
	      mSubset.set(j++);
	      // now set the next m elements
	      for (;m > 0;m--) {
		mSubset.set(j++);
	      }
	      for (;j < n;) {
		mSubset.clear(j++);
	      }
	    }
	  }
	  else {
	    // we've gone past the last subset
	    mHasMore = false;
	  }
	}
	catch (Exception exc) {
	  // this "can't happen"
	  throw new InternalError("Internal error in subset enumeration ");
	}
      }
    }
  }

   private void initSet() 
    throws ArrayIndexOutOfBoundsException
  {
    int i;

    if (mAllSubsets) {
      for (i=0; i<mSetSize; i++) {
	mSubset.clear(i);
      }
    }
    else {
      for (i=0; i<mSubsetSize; i++) {
	mSubset.set(i);
      }
    }
    mHasMore = true;
  }

}
