package Oid;
import java.util.*;

/** 
 * A <code>TutteMatrixChecker</code> instance is able to accept a
 * {0,1}-matrix in standard form and determine whether it is the GF(2)
 * representation of a graphic matroid.  It does this recursively;
 * that is, at some point it creates more instances of itself to check
 * smaller matrices based on the one it was given.  See the following
 * reference for details:
 *
 * Tutte, W.T. (1959) "An algorithm for determining whether a given binary
 * matroid is graphic." <i>ToDo : find out journal name</i> <b>ToDo : 
 * volume</b> pp. 905-917.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 20 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class TutteMatrixChecker
{
  // 
  // private data members
  // 
  private String  LF;
  private String  mName;
  private int[][] mMatrix;
  private int[]   mRowNum;
  private int[]   mColNum;
  private int     mNextRowNum;        // next "artificial" row number to use
  private int     mRows;
  private int     mCols;
  private int[]   mColCount;
  private int     mY;
  private Subset  mYSub;              // subset of columns with 1 in row Y
  private int[][] mContraction;
  private int[]   mContractionRowNum;
  private int[]   mContractionColNum;
  private int     mCRows;             // number of rows in contraction
  private int     mCCols;             // number of columns in contraction
  private Vector  mBridges;           // bridges as Subsets of cols of contrct
  private Vector  mRBridges;          // bridges as Subsets of cols of matrix
  private int     mBridgeCount;
  private Vector  mYComponents;       // Vector of int[][]
  private Vector  mYComponentRowNums; // Vector of int[]
  private Vector  mPartitions;        // Vector of Vectors of Subsets
  private Subset  mBridgeClass1;      // 1st class of bridges from even calc.
  private Subset  mBridgeClass2;      // 2nd class of bridges from even calc.
  private int[]   mBridgeClassMap;    // which class each bridge is in
  private boolean mIsGraphic;
  private Vector  mComponentMatrices; // Vector of int[][] of inc. matrices
  private Vector  mIncidenceRowNums;  // Vector of int[] of inc. mat. row nums
  private int[][] mClass1IncidenceMatrix;
  private int[]   mClass1IncidenceRowNum;
  private int[][] mClass2IncidenceMatrix;
  private int[]   mClass2IncidenceRowNum;
  private int[][] mIncidenceMatrix;   // graph incidence matrix if mIsGraphic
  private int[]   mIncidenceRowNum;   // row nums of incidence matrix
  private String  mClassName1;        // cat names of class 1 matrices
  private String  mClassName2;        // cat names of class 2 matrices
  private IncidenceMatrixCombiner mCombiner;
  private String  mExplanation;
  private int mLen;

  public TutteMatrixChecker() 
  {
    mNextRowNum = -1;
    LF = System.getProperty("line.separator");
  }

  /**
   * provides the object instance with a {0,1} - matrix to check, along
   * with the number assignments for each of its rows and columns
   *
   * @param R {0,1}-matrix to check
   * @param colNum array of column numbers for R
   * @param rowNum array of row numbers for R
   * @throws IllegalArgumentException thrown if arrays are not consistent or
   *         matrix is not in standard form
   */
  public void setMatrix(String name, int[][]R, int[]colNum, int[]rowNum)
    throws IllegalArgumentException
  {
    // perform "common-sense" checks on the matrices
    if (R.length != rowNum.length) {
      throw new IllegalArgumentException("Row numbers do not match matrix");
    }

    if (R[0].length != colNum.length) {
      throw new IllegalArgumentException("Column numbers do not match matrix");
    }

    // make sure matrix is in standard form
    if (!inStandardForm(R)) {
      throw new IllegalArgumentException("R is not in standard form");
    }

    // make local copies of the matrix and execute the algorithm
    copyMatrices(R, colNum, rowNum);

    // if the next new row number hasn't been set, set it to one plus
    // the largest row number
    int i;
    if (mNextRowNum < 0) {
      for (i=0; i<mRows; i++) {
	if (mRowNum[i] > mNextRowNum) {
	  mNextRowNum = mRowNum[i];
	}
      }
      mNextRowNum++;
    }

    mName = new String(name);
    checkMatrix();
  }

  /**
   * provides the object instance with a {0,1} - matrix to check, along
   * with the number assignments for each of its rows and columns
   *
   * @param R {0,1}-matrix to check
   * @param colNum array of column numbers for R
   * @param rowNum array of row numbers for R
   * @param nextNewRowNum next "artificial" row number to use
   * @throws IllegalArgumentException thrown if arrays are not consistent or
   *         matrix is not in standard form
   */
  public void setMatrix(String name, int[][]R, int[]colNum, int[]rowNum,
			int nextNewRowNum)
    throws IllegalArgumentException
  {
    int i;
    int maxRowNum=0;
    for (i=0; i<rowNum.length; i++) {
      if (rowNum[i] > maxRowNum) {
	maxRowNum = rowNum[i];
      }
    }

    if (nextNewRowNum <= maxRowNum) {
      nextNewRowNum = maxRowNum+1;
    }
    mNextRowNum = nextNewRowNum;
    setMatrix(name, R, colNum, rowNum);
  }

  /**
   * if a valid matrix has been supplied to the object, this method returns
   * the deterimination whether the matrix represents a graphic matroid
   *
   * @return boolean value indicating whether matrix is graphic
   * @throws IllegalStateException thrown if no matrix has been set
   */
  public boolean isGraphic()
  throws IllegalStateException
  {
    if (!(mMatrix instanceof int[][])) {
      throw new IllegalStateException("No matrix provided to "
				      + "TutteMatrixChecker");
    }
    return mIsGraphic;
  }

  /** 
   * returns a String containing a textual explanation of the steps performed
   * by the object to determine its answer
   *
   * @return String with explanation
   * @throws IllegalStateException thrown if no matrix has been set
   */
  public String getExplanation()
  {
    if (!(mMatrix instanceof int[][])) {
      throw new IllegalStateException("No matrix provided to "
				      + "TutteMatrixChecker");
    }
    return mExplanation;
  }

  /**
   * returns the incidence matrix calculated for the matrix, if it is graphic
   */
  public int[][] getIncidenceMatrix() {
    if (!(mMatrix instanceof int[][])) {
      throw new IllegalStateException("No matrix provided to "
				      + "TutteMatrixChecker");
    }
    return mIncidenceMatrix;
  }
    
  /**
   * returns the incidence matrix row numbers if the matrix was graphic
   */
  public int[] getIncidenceRowNum() {
    if (!(mMatrix instanceof int[][])) {
      throw new IllegalStateException("No matrix provided to "
				      + "TutteMatrixChecker");
    }
    return mIncidenceRowNum;
  }

  /** 
   * returns the next available row number
   */
  public int getNextRowNum()
  {
    return mNextRowNum;
  }

  //
  // private method determines whether passed matrix is in standard form
  //
  private boolean inStandardForm(int[][] R)
  {
    boolean answer = true;
    int r = R.length;
    int n = R[0].length;
    int i, j;
    
    if (n < r) {
      answer = false;
    }

    if (answer) {
      for (i=0; i<r && answer; i++) {
	for (j=0; j<r; j++) {
	  if (((i == j) && (R[i][j] != 1)) || ((i != j) && (R[i][j] != 0))) {
	    answer = false;
	    break;
	  }
	}
      }
    }
    return answer;
  }

  //
  // private method makes a local copy of the passed matrix, row and column
  // numberings
  //
  private void copyMatrices(int[][] R, int[] colNum, int[] rowNum)
  {
    mRows = R.length;
    mCols = R[0].length;
    mMatrix = new int[mRows][mCols];
    mRowNum = new int[mRows];
    mColNum = new int[mCols];
    int i,j;
    
    for (i=0; i<mCols; i++) {
      mColNum[i] = colNum[i];
    }

    for (i=0; i<mRows; i++) {
      mRowNum[i] = rowNum[i];
      for (j=0; j<mCols; j++) {
	mMatrix[i][j] = R[i][j] % 2; // reduce mod 2 just in case
      }
    }

    // set up integer formatter -- how many places are needed for the
    // biggest row or column number?
    i = 0;
    for (j=0; j<mRows; j++) {
      if (mRowNum[j] > i) {
	i = mRowNum[j];
      }
    }

    for (j=0; j<mCols; j++) {
      if (mColNum[j] > i) {
	i = mColNum[j];
      }
    }

    j = 0;
    do {
      j++;
      i /= 10;
    } while (i > 0);

    mLen = j+1;
  } 

  //
  // private method executes Tutte's Algorithm to check the matrix
  //
  private void checkMatrix()
  {
    boolean finished = false;
    mExplanation = new String();
    int i,j,k,m,n, col;
    Subset b, b2;
    boolean found;
    
    mExplanation += "----------------------------------------" + LF;
    mExplanation += mName + LF;
    addMatrixToExplanation(mMatrix, mColNum, mRowNum);

    if (isTriviallyGraphic()) {
      mIsGraphic = true;
      finished = true;
    }

    if (!finished) {
      col = identifyColumn(); // find column with smallest # of 1's >= 3
      if (col < 0) {
	throw new InternalError("Internal error in checkMatrix");
      }

      mY = nextNonZeroRow(col, 0);
      while (mY >= 0) {
	determineSubsetY();
	mExplanation += "Row #" + String.valueOf(mRowNum[mY]) +
	    ", Y = " + mYSub.toString(mColNum) + LF;
	computeContraction();
	mExplanation += "M/Y:" + LF;
	addMatrixToExplanation(mContraction, mContractionColNum, 
			       mContractionRowNum);
	mBridges = computeBridges(mContraction);
	mBridgeCount = mBridges.size();

	try {
	  // now we need the bridges as subsets of the columns of the
	  // original matrix
	  mRBridges = new Vector();
	  for (Iterator pb = mBridges.iterator(); pb.hasNext(); ) {
	    b = (Subset)pb.next();
	    b2 = new Subset(mCols);
	    j = 0;
	    for (i=0; i<mCols; i++) {
	      if (mMatrix[mY][i] == 0) {
		if (b.get(j)) {
		  b2.set(i);
		}
		j++;
	      }
	    }
	    mRBridges.add(b2);
	  }
	  
	  // add bridge list to explanation
	  mExplanation += "Bridges:" + LF;
	  i=1;
	  for (Iterator pBridge = mBridges.iterator(); 
	       pBridge.hasNext(); i++) {
	    mExplanation += "B" + String.valueOf(i) + ": " + 
	      ((Subset)pBridge.next()).toString(mContractionColNum) + LF;
	  }
	}
	catch (Exception e) {
	  // this "can't happen"
	  throw new InternalError("Internal subset error in checkMatrix");
	}

	if (mBridgeCount > 1) {
	  break;
	}
	mY = nextNonZeroRow(col, mY+1);
      }
      if (mY < 0) {
	mIsGraphic = false;
	finished = true;
	mExplanation += "NOT GRAPHIC: " +
	  "All choices of Y resulted in just one bridge" + LF;
      }
    }

    if (!finished) {
      computeYComponents();

      // do all bridges partition Y?
      if (!bridgesPartition()) {
	mExplanation += "NOT GRAPHIC: The bridges do not all partition Y" + LF;
	mIsGraphic = false;
	finished = true;
      }
    }

    if (!finished) {
      if (!isYEven()) {
	mExplanation += "NOT GRAPHIC: Y is not even" + LF;
	mIsGraphic = false;
	finished = true;
      }
    }

    mClass1IncidenceMatrix = (int[][])null;
    mClass1IncidenceRowNum = (int[])null;
    if (!finished) {
      mIsGraphic = true;
      mCombiner = new IncidenceMatrixCombiner();
      // first check components in class 1
      mExplanation += "Checking class 1 components..." + LF;
      for (i=0; i<mBridgeCount; i++) {
	if (mBridgeClassMap[i] == 1) {
	  if (!checkComponent(i, 1)) {
	    finished = true;
	    break;
	  }
	}
      }
    }

    mClass2IncidenceMatrix = (int[][])null;
    mClass2IncidenceRowNum = (int[])null;
    if (!finished) {
      // then check components in class 2
      mExplanation += "Checking class 2 components..." + LF;
      for (i=0; i<mBridgeCount; i++) {
	if (mBridgeClassMap[i] == 2) {
	  if (!checkComponent(i, 2)) {
	    finished = true;
	    break;
	  }
	}
      }
    }

    if (!finished) {
      mExplanation += "Combining bridge class incidence matrices" + LF;
      mExplanation += "(" + mClassName1 + ")" + LF;
      addMatrixToExplanation(mClass1IncidenceMatrix, mColNum, 
			     mClass1IncidenceRowNum);
      mExplanation += "(" + mClassName2 + ")" + LF;
      addMatrixToExplanation(mClass2IncidenceMatrix, mColNum,
			     mClass2IncidenceRowNum);
      mExplanation += "=======================================" + LF;
      mCombiner.setIncidenceMatrices(mClass1IncidenceMatrix,
				     mClass1IncidenceRowNum,
				     mClass2IncidenceMatrix,
				     mClass2IncidenceRowNum);
      if (mCombiner.canCombineMatrices()) {
	mIncidenceMatrix = mCombiner.getCombinedMatrix();
	mIncidenceRowNum = mCombiner.getCombinedRowNumbers(); 
	mExplanation += mCombiner.getExplanation();
      }
      else {
	mExplanation += mCombiner.getExplanation();
	mExplanation += "Unable to combine matrices." + LF;
	mIsGraphic = false;
	finished = true;
      }
      mExplanation += "=======================================" + LF;
    }

    if (mIsGraphic) {
      mExplanation += "Incidence matrix for " + mName + ":" + LF;
      addMatrixToExplanation(mIncidenceMatrix, mColNum, mIncidenceRowNum);
      mExplanation += mName + " is graphic" + LF;
      mExplanation += "----------------------------------------" + LF;
    }
  }

  //
  // private method adds current matrix to the explanation string
  //
  private void addMatrixToExplanation(int[][] matrix, int[] colNum,
				      int[] rowNum)
  {
    int i,j,m,n;

    m = matrix.length;
    n = matrix[0].length;
    
    // first do column labels, leave enough space for row labels
    for (i=0; i<mLen; i++) {
      mExplanation += ' ';
    }
    for (i=0; i<n; i++) {
      mExplanation += 
	TextAlignment.rightAlign(String.valueOf(colNum[i]), mLen);
    }
    mExplanation += LF;
    
    // now do rows of matrix, preceded by row labels
    for (i=0; i<m; i++) {
      mExplanation += 
	TextAlignment.rightAlign(String.valueOf(rowNum[i]), mLen);
      for (j=0; j<n; j++) {
	mExplanation += 
	  TextAlignment.rightAlign(String.valueOf(matrix[i][j]), mLen);
      }
      mExplanation += LF;
    }
  }

  // 
  // private method checks to see if one component is graphic, and combines
  // its incidence matrix with the existing incidence matrix if it is
  //
  private boolean checkComponent(int compNum, int which)
  {
    boolean answer=true;
    TutteMatrixChecker checker = new TutteMatrixChecker();
    int[][] component;
    int[] compRowNum;
    int[][] matrix;
    int[] colNum;
    int[] colRef;
    int[][] incidenceMatrix;
    boolean found;
    int nonZeroColumns;
    String subName;
    Iterator pComponent = mYComponents.iterator();
    Iterator pRowNum = mYComponentRowNums.iterator();
    int i,j,k,m,n;
    int[][] accumIncidenceMatrix = (which == 1) ? mClass1IncidenceMatrix :
      mClass2IncidenceMatrix;
    int[] accumIncidenceRowNum = (which == 1) ? mClass1IncidenceRowNum :
      mClass2IncidenceRowNum;
    component = (int[][])mYComponents.get(compNum);
    compRowNum = (int[])mYComponentRowNums.get(compNum);
    m = component.length;
    //
    // copy nonzero columns of component to matrix
    //
    // first count nonzero columns
    nonZeroColumns = 0;
    for (j=0; j<mCols; j++) {
      found = false;
      for (k=0; k<m; k++) {
	if (component[k][j] != 0) {
	  found = true;
	  break;
	}
      }
      if (found) {
	nonZeroColumns++;
      }
    }
    // copy nonzero columns over 
    matrix = new int[m][nonZeroColumns];
    colNum = new int[nonZeroColumns];
    colRef = new int[nonZeroColumns];
    n = 0;
    for (j=0; j<mCols && n<nonZeroColumns; j++) {
      found = false;
      for (k=0; k<m; k++) {
	matrix[k][n] = component[k][j];
	if (0 != component[k][j]) {
	  found = true;
	}
      }
      // only move to the next column if there was a nonzero entry
      // in this one
      if (found) {
	colNum[n] = mColNum[j];
	colRef[n] = j;
	n++;
      }
    }
    
    subName = new String(mName);
    subName += "," + String.valueOf(compNum+1);

    checker.setMatrix(subName, matrix, colNum, compRowNum, mNextRowNum);
    mExplanation += indent(checker.getExplanation());
    if (!checker.isGraphic()) {
      mExplanation += "NOT GRAPHIC: Component " + subName + 
	" is not graphic" + LF;
      mIsGraphic = false;
      answer = false;
    }
    else {
      matrix = checker.getIncidenceMatrix();
      compRowNum = checker.getIncidenceRowNum();
      mNextRowNum = checker.getNextRowNum();
      //
      // restore incidence matrix with zero columns
      //
      incidenceMatrix = new int[matrix.length][mCols];
      for (i=0; i<matrix.length; i++) {
	for (j=0; j<mCols; j++) {
	  incidenceMatrix[i][j] = 0;
	}
      }
      for (i=0; i<matrix.length; i++) {
	for (j=0; j<matrix[0].length; j++) {
	  incidenceMatrix[i][colRef[j]] = matrix[i][j];
	}
      }
      //
      // either combine incidence matrix with earlier matrices
      // or copy it into accumIncidenceMatrix
      //
      if (!(accumIncidenceMatrix instanceof int[][])) {
	mExplanation += "Copying " + subName + " incidence matrix to " +
	  "bridge class matrix" + LF;
	accumIncidenceMatrix = new int [incidenceMatrix.length][mCols];
	accumIncidenceRowNum = new int[incidenceMatrix.length];
	for (i=0; i<incidenceMatrix.length; i++) {
	  accumIncidenceRowNum[i] = compRowNum[i];
	  for (j=0; j<mCols; j++) {
	    accumIncidenceMatrix[i][j] = incidenceMatrix[i][j];
	  }
	}
	if (1 == which) {
	  mClass1IncidenceMatrix = accumIncidenceMatrix;
	  mClass1IncidenceRowNum = accumIncidenceRowNum;
	  mClassName1 = new String(subName);
	}
	else {
	  mClass2IncidenceMatrix = accumIncidenceMatrix;
	  mClass2IncidenceRowNum = accumIncidenceRowNum;
	  mClassName2 = new String(subName);
	}
      }
      else {
	mExplanation += "Combining incidence matrices:" + LF;
	mExplanation += (which == 1) ? mClassName1 : mClassName2;
	mExplanation += ":" + LF;
	addMatrixToExplanation(accumIncidenceMatrix, mColNum, 
			       accumIncidenceRowNum);
	mExplanation += "Incidence matrix for " + subName + LF;
	addMatrixToExplanation(incidenceMatrix, mColNum,
			       compRowNum);
	mExplanation += "=======================================" + LF;
	mCombiner.setIncidenceMatrices(accumIncidenceMatrix, 
				       accumIncidenceRowNum,
				       incidenceMatrix, compRowNum);
	if (mCombiner.canCombineMatrices()) {
	  accumIncidenceMatrix = mCombiner.getCombinedMatrix();
	  accumIncidenceRowNum = mCombiner.getCombinedRowNumbers(); 
	  if (1 == which) {
	    mClass1IncidenceMatrix = accumIncidenceMatrix;
	    mClass1IncidenceRowNum = accumIncidenceRowNum;
	  }
	  else {
	    mClass2IncidenceMatrix = accumIncidenceMatrix;
	    mClass2IncidenceRowNum = accumIncidenceRowNum;
	  }
	  mExplanation += mCombiner.getExplanation();
	}
	else {
	  mExplanation += mCombiner.getExplanation();
	  mExplanation += "Unable to combine matrices." + LF;
	  mIsGraphic = false;
	  answer = false;
	}
	mExplanation += "=======================================" + LF;
	if (1 == which) {
	  mClassName1 += " " + subName;
	}
	else {
	  mClassName2 += " " + subName;
	}
      }
    }
    return answer;
  }

  // 
  // private method checks whether matrix is trivially graphic (can be
  // extended by one row to get an incidence matrix); adds explanation to 
  // mExplanation 
  //
  private boolean isTriviallyGraphic()
  {
    mColCount = new int[mCols];
    int i, j;
    boolean answer = true;

    for (i=0; i<mCols; i++) {
      mColCount[i] = 0;
      for (j=0; j<mRows; j++) {
	if (mMatrix[j][i] != 0) {
	  mColCount[i]++;
	}
      }
    }

    for (i=0; i<mCols; i++) {
      if (mColCount[i] > 2) {
	answer = false;
	break;
      }
    }

    // set explanation
    if (answer) {
      mExplanation += mName + 
	" with an additional row, is an incidence matrix:" + LF;
      mIncidenceMatrix = new int[mRows+1][mCols];
      mIncidenceRowNum = new int[mRows+1];
      for (j=0; j<mCols; j++) {
	mIncidenceMatrix[mRows][j] = (mColCount[j] % 2);
      }
      mIncidenceRowNum[mRows] = mNextRowNum;
      mNextRowNum++;

      for (i=0; i<mRows; i++) {
	for (j=0; j<mCols; j++) {
	  mIncidenceMatrix[i][j] = mMatrix[i][j];
	}
	mIncidenceRowNum[i] = mRowNum[i];
      }
      
    }
    else {
      mExplanation += mName + " is not the incidence matrix of a graph" + LF;
    }

    return answer;
  }

  //
  // private method identifies the column with the least number of ones
  // that is at least 3
  //
  private int identifyColumn()
  {
    int answer = -1;
    int count = mRows;
    int i;

    for (i=0; i<mCols; i++) {
      if ((mColCount[i] >= 3) && ((answer < 0) || (mColCount[i] < count))) {
	answer = i;
	count = mColCount[i];
      }
    }
    
    mExplanation += "Column #" + mColNum[answer] + LF;

    return answer;
  }

  //
  // private method returns the index of the first row of the matrix
  // with a nonzero entry in column col that is greater than or equal to 
  // the passed value startRow, or -1 if no nonzero entries exist in that
  // range
  //
  private int nextNonZeroRow(int col, int startRow) 
  {
    int answer = -1;
    int i;

    for (i=startRow; i<mRows; i++) {
      if (mMatrix[i][col] != 0) {
	answer = i;
	break;
      }
    }
    return answer;
  }

  //
  // private method indents every line of the passed string 2 spaces
  //
  private String indent(String s) {
    StringTokenizer st = new StringTokenizer(s, LF);
    String answer = new String("");
    for (; st.hasMoreElements(); ) {
      answer += "  ";
      answer += (String)st.nextElement();
      answer += LF;
    }
    return answer;
  }

  //
  // private subset determines subset of columns with nonzero entries in
  // row mY
  //
  private void determineSubsetY()
  {
    int i;

    try {
      mYSub = new Subset(mCols);
      for (i=0; i<mCols; i++) {
	if (mMatrix[mY][i] != 0) {
	  mYSub.set(i);
	}
      }
    }
    catch (Exception e) {
      throw new InternalError("Internal error in determineSubsetY");
    }
  }

  // 
  // private method contracts row mY; i.e., removes the row
  // and all columns with nonzero entries in that row, placing the results
  // in mContraction, mContractionRowNum and mContractionColNum
  // (requires mY to be set)
  //
  private void computeContraction()
  {
    // determine size of contraction and allocate arrays
    mCRows = mRows - 1;
    mCCols = mCols - numNonZeroRowEntries(mY);
    mContraction = new int[mCRows][mCCols];
    mContractionRowNum = new int[mCRows];
    mContractionColNum = new int[mCCols];
    int i,j,r=0,c;

    for (i=0; i<mRows; i++) {
      if (i != mY) {
	c = 0;
	for (j=0; j<mCols; j++) {
	  if (mMatrix[mY][j] == 0) {
	    mContraction[r][c] = mMatrix[i][j];
	    c++;
	  }
	}
	mContractionRowNum[r] = mRowNum[i];
	r++;
      }
    }

    c = 0;
    for (j=0; j<mCols; j++) {
      if (mMatrix[mY][j] == 0) {
	mContractionColNum[c] = mColNum[j];
	c++;
      }
    }
  }

  //
  // private method counts the nonzero entries in the indicated row of mMatrix
  //
  private int numNonZeroRowEntries(int row)
  {
    int answer = 0, i;
    for (i=0; i<mCols; i++) {
      answer += (mMatrix[row][i] != 0) ? 1 : 0;
    }
    return answer;
  }


  //
  // private method computes the bridges of mContraction (requires mY to be
  // set and the contraction to be computed)
  //
  private Vector computeBridges(int[][] matrix)
  {
    int curBridge=0;
    Vector answer = new Vector();
    int rows = matrix.length, cols = matrix[0].length;
    int[] bridge = new int[cols];
    Vector toBeAssigned = new Vector();
    Iterator p;
    int numAssigned = 0;
    boolean found;
    int i,j,k,m,n;

    for (i=0; i<cols; i++) {
      bridge[i] = -1;
    }

    while (numAssigned < cols) {
      // find an unassigned column
      found = false;
      for (i=0; i<cols; i++) {
	if (bridge[i] < 0) {
	  found = true;
	  break;
	}
      }
      if (!found) {
	throw new InternalError("Internal error in computeBridges");
      }
      toBeAssigned = new Vector();
      toBeAssigned.add(new Integer(i));
      while (toBeAssigned.size() > 0) {
	i = ((Integer)toBeAssigned.iterator().next()).intValue();
	for (j=0; j<rows; j++) {
	  if (matrix[j][i] != 0) {
	    for (k=0; k<cols; k++) {
	      if ((matrix[j][k] != 0) && 
		  (bridge[k] < 0) && 
		  (k != i) &&
		  (!toBeAssigned.contains(new Integer(k)))) {
		toBeAssigned.add(new Integer(k));
	      }
	    }
	  }
	}
	p = toBeAssigned.iterator();
	bridge[((Integer)p.next()).intValue()] = curBridge;
	numAssigned++;
	p.remove();
      }
      curBridge++;
    }

    // determine bridges as subsets
    Subset b, b2;
    try {
      for (i=0; i<curBridge; i++) {
	b = new Subset(cols);
	answer.add(b);
      }
      // the i-th column is in the bridge[i]-th bridge
      for (i=0; i<mCCols; i++) {
	((Subset)answer.get(bridge[i])).set(i);
      }
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError("Internal error in bridge calculation");
    }
    return answer;
  }

  //
  // private method computes the Y components of the matrix based on the
  // bridges (requires mY to be set and the bridges to be computed)
  //
  private void computeYComponents()
  {
    int[][] YComp;
    int[] YCompRowNum;
    int i, j, k, m, n = mCols, count=1;

    mYComponents = new Vector();
    mYComponentRowNums = new Vector();
    Subset basis, rows, bridge;

    try {
      basis = new Subset(mCols);
      for (i=0; i<mRows; i++) {
	basis.set(i);
      }
    }
    catch (Exception e) {
      throw new InternalError("Subset error in computeYComponents");
    }

    // loop through bridges
    for (Iterator p = mRBridges.iterator(); p.hasNext(); count++) {
      bridge = (Subset)p.next();
      try {
	rows = new Subset(basis);
	rows.and(bridge);
      }
      catch (Exception e) {
	throw new InternalError("Subset error in computeYComponents");
      }
      m = rows.numElements()+1;
      // make YComp, YCompColNum, YCompRowNum
      YComp = new int[m][n];
      YCompRowNum = new int[m];
      // fill up components & set row & col nums
      j = 0;
      for (i=0; i<mRows; i++) {
	try {
	  if ((i == mY) || rows.get(i)) {
	    YCompRowNum[j] = mRowNum[i];
	    for (k=0; k<mCols; k++) {
	      YComp[j][k] = mMatrix[i][k];
	    }
	    j++;
	  }
	}
	catch (Exception e) {
	  throw new InternalError("Subset error in computeYComponents");
	}
      }

      // add matrix to explanation
      mExplanation += "M X (B" + String.valueOf(count) + " U Y):" + LF;
      addMatrixToExplanation(YComp, mColNum, YCompRowNum);
      
      // add this component to the vectors
      mYComponents.add(YComp);
      mYComponentRowNums.add(YCompRowNum);
    }
  }

  //
  // private method determines whether the passed bridge partitions Y
  // (requires mY to be set and the indicated bridge and bridge component
  // to be determined)
  //
  private boolean bridgesPartition()
  {
    boolean answer = true;
    int[][] YComp;
    int[] YRowNums;
    int[][] matrix;
    int[] colRef;
    int[] colNum;
    Iterator pYComp = mYComponents.iterator();
    Iterator pYCompLbl = mYComponentRowNums.iterator();
    int i, j, k, m, n, count=1;

    mPartitions = new Vector();

    n = mYSub.numElements();
    colRef = new int[n];
    colNum = new int[n];

    j = 0;
    for (i=0; i<mCols; i++) {
      if (mYSub.get(i)) {
	colRef[j] = i;
	colNum[j] = mColNum[i];
	j++;
      }
    }

    for (; pYComp.hasNext(); ) {
      // make a copy of this Y component 
      YComp = (int[][])pYComp.next();
      YRowNums = (int[])pYCompLbl.next();
      // make another copy with only the columns in Y
      m = YComp.length;
      matrix = new int[m][n];
      for (i=0; i<m; i++) {
	k = 0;
	for (j=0; j<mCols; j++) {
	  if (mYSub.get(j)) {
	    matrix[i][k] = YComp[i][j];
	    k++;
	  }
	}
      }
	
      //  	// add matrix to explanation
      //  	mExplanation += "(M | B" + String.valueOf(count) + " U Y) . Y:" + LF;
      //  	addMatrixToExplanation(matrix, colNum, YRowNums);

      // row-reduce the copy
      rowReduce(matrix);

      mExplanation += LF + "(M | B" + String.valueOf(count) + 
	" U Y) . Y After row reduction:" + LF;
      addMatrixToExplanation(matrix, colNum, YRowNums);

      // check for a single 1 in each column
      if (!isAPartition(matrix)) {
	// add explanation
	mExplanation += "This is not a partition" + LF;
	answer = false;
	break;
      }
      else {
	// add explanation
	mExplanation += "This is a partition" + LF;
      }

      // determine the partition for this Y component
      mExplanation += "B" + String.valueOf(count) + ":";
      mPartitions.add(getYComponentPartitions(matrix,colRef));
      count++;
    }

    return answer;
  }

  //
  // private method row-reduces a {0,1}-matrix
  //
  private void rowReduce(int[][] matrix)
  {
    int i,j,r,c,m,n,nUsed = 0,limit;
    boolean found;
    boolean[] rowUsed;

    m = matrix.length;
    n = matrix[0].length;
    rowUsed = new boolean[m];
    for (i=0; i<m; i++) {
      rowUsed[i] = false;
    }
    
    limit = (m < n) ? m : n;
    c = 0;
    nUsed = 0;
    while ((c<n) && (nUsed<limit)) {
      // look for a nonzero entry in an unused row in column c
      found = false;
      r = -1;
      for (i=0; i<m; i++) {
	if (!rowUsed[i] && (0 != matrix[i][c])) {
	  found = true;
	  r = i;
	  break;
	}
      }
      if (found) {
	// pivot on this element
	for (i=0; i<m; i++) {
	  if ((i != r) && (0 != matrix[i][c])) {
	    for (j=0; j<n; j++) {
	      matrix[i][j] = (matrix[i][j] + matrix[r][j]) % 2;
	    }
	  }
	}
	rowUsed[r] = true;
	nUsed++;
      }
      c++;
    }
  }

  //
  // private method checks a {0,1)-matrix to make sure there is exactly
  // one nonzero entry in each column
  //
  private boolean isAPartition(int[][] matrix)
  {
    boolean answer = true;
    int i,j,m,n,c;
    m = matrix.length;
    n = matrix[0].length;

    for (i=0; i<n; i++) {
      c = 0;
      for (j=0; j<m; j++) {
	if (0 != matrix[j][i]) {
	  c++;
	}
      }
      if (1 != c) {
	answer = false;
	break;
      }
    }

    return answer;
  }

  //
  // private method returns the Vector of partitions from the passed
  // row-reduced Y component and array of column references (positions of
  // each column in the original array)
  //
  private Vector getYComponentPartitions(int[][] matrix, int[] colRef)
  {
    Vector answer = new Vector();
    int i,j,c,m =matrix.length,n=matrix[0].length;
    Subset s;

    // loop through rows of matrix
    for (i=0; i<m; i++) {
      s = new Subset(mCols);
      for (j=0; j<n; j++) {
	if (0 != matrix[i][j]) {
	  s.set(colRef[j]);
	}
      }
      if (s.numElements() > 0) {
	answer.add(s);
      }
    }

    // add partitions to explanation
    mExplanation += "{ ";
    for (Iterator p = answer.iterator(); p.hasNext(); ) {
      mExplanation += ((Subset)p.next()).toString(mColNum) + " ";
    }
    mExplanation += " }" + LF;
    return answer;
  }

  // 
  // private method determines whether Y is even (requires mY to be set
  // and the bridges and partitions from Y to be identified)
  //
  private boolean isYEven()
  {
    boolean answer = false;
    int k;
    // first portion of partition is always smaller, to avoid duplication
    int limit = mBridgeCount / 2;
    SubsetEnumeration se;

    try {
      for (k=1; (k<=limit) && !answer; k++) {
	for (se = new SubsetEnumeration(mBridgeCount, k); 
	     se.hasMoreElements();) {
	  mBridgeClass1 = (Subset)se.nextElement();
	  mBridgeClass2 = mBridgeClass1.complement();
	  if (isCurrentBridgePartitionNonOverlapping()) {
	    mExplanation += "Bridge classes: { " + 
	      mBridgeClass1.toString(true) + ", " + 
	      mBridgeClass2.toString(true) + " } are non-overlapping" + LF;
	    answer = true;
	    break;
	  }
	  else {
	    mExplanation += "Bridge classes: { " + 
	      mBridgeClass1.toString(true) + ", " + 
	      mBridgeClass2.toString(true) + " } are NOT non-overlapping" + LF;
	  }
	}
      }
    }
    catch (Exception e) {
      // this "can't happen"
      throw new InternalError("Internal error in evenness calculation");
    }

    // add to explanation
    if (answer) {
      mExplanation += "Y is even" + LF;
      mBridgeClassMap = new int[mBridgeCount];
      for (k=0; k<mBridgeCount; k++) {
	try {
	  if (mBridgeClass1.get(k)) {
	    mBridgeClassMap[k] = 1;
	  }
	  else {
	    mBridgeClassMap[k] = 2;
	  }
	}
	catch (Exception e) {
	  throw new InternalError("Internal error in evenness calculation");
	}
      }
    }


    return answer;
  }

  // 
  // private method determines whether, for the current choice of
  // mBridgeClass1 and mBridgeClass2, "every pair of bridges in the same
  // class are non-overlapping"
  //
  private boolean isCurrentBridgePartitionNonOverlapping()
  {
    boolean answer = true;
    int i, j;

    // check every pair of bridges in the first partition
    if (mBridgeClass1.numElements() > 1) {
      for (i=0; i<mBridgeCount && answer; i++) {
	if (mBridgeClass1.get(i)) {
	  for (j=0; j<mBridgeCount && answer; j++) {
	    if ((j != i) && mBridgeClass1.get(j)) {
	      answer = answer && areNonOverlapping(i,j);
	    }
	  }
	}
      }
    }

    if (answer && mBridgeClass2.numElements() > 1) {
      for (i=0; i<mBridgeCount && answer; i++) {
	if (mBridgeClass2.get(i)) {
	  for (j=0; j<mBridgeCount && answer; j++) {
	    if ((j != i) && mBridgeClass2.get(j)) {
	      answer = answer && areNonOverlapping(i,j);
	    }
	  }
	}
      }
    }

    return answer;
  }

  //
  // private method determines if bridges b1 and b2 are non-overlapping
  //
  private boolean areNonOverlapping(int b1, int b2) 
  {
    boolean answer = false;
    Iterator p1, p2;
    Subset s1, s2, su;
    
    for (p1 = ((Vector)mPartitions.get(b1)).iterator(); 
	 p1.hasNext() && !answer; ) {
      s1 = (Subset)p1.next();
      for (p2 = ((Vector)mPartitions.get(b2)).iterator(); p2.hasNext(); ) {
	s2 = (Subset)p2.next();
	su = new Subset(s1);
	su.or(s2);
	if (su.equals(mYSub)) {
	  answer = true;
	  break;
	}
      }
    }

    return answer;
  }
}
