import Oid.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

/**
 * <code>SingleFrameGUI</code> implements the GUI interface with a
 * simple <code>JFrame</code> -based user interface.  The interface
 * includes a menu bar and a <code>JSplitPane</code>, which in turn
 * contains a <code>JList</code> on the left for displaying currently
 * open objects, and a <code>JDesktopPane</code> on the right for
 * displaying internal frames.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 14 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class SingleFrameGUI 
  extends JFrame 
  implements ActionListener, 
	     MouseListener,
	     InternalFrameListener,
	     ComponentListener,
	     GUI
{
  //
  // private data members
  //
  private JMenuBar mMenuBar;
  private JMenu mFileMenu;
  private JMenu mEditMenu;
  private JMenu mTasksMenu;
  private JMenu mViewMenu;
  private JMenu mHelpMenu;
  private JList mOpenObjectList;
  private JDesktopPane mDesktop;
  private JTextArea mMessageArea;
  private Vector mInternalFrames;
  private Vector mPanels;
  private Vector mPanelParents;
  private Vector mActionListeners;
  private Vector mCreatableClasses;
  private Vector mCreatableClassNames;
  private Vector mOpenObjects;
  private static final int INIT_X = 800;
  private static final int INIT_Y = 600;
  private static final String DEFAULT_CAPTION = "Display ";
  private static final int X_INTFRAME_INC = 30;
  private static final int Y_INTFRAME_INC = 30;
  private static final int X_INTFRAME_MIN = 100;
  private static final int Y_INTFRAME_MIN = 40;

  public SingleFrameGUI()
  {
    constructFrame();
    mOpenObjects = new Vector();
  }

  public SingleFrameGUI(String title)
  {
    super(title);
    constructFrame();
    mOpenObjects = new Vector();
  }

  /**
   * Responds to ListSelectionEvents fired by the JList on the left side
   * of the frame's split pane.
   *
   * @param e The ListSelectionEvent to respond to
   */
  public void mouseClicked(MouseEvent e)
  {
    if (e.getSource() instanceof JList) {
      Object selected = mOpenObjectList.getSelectedValue();
      if (selected != null) {
	int i = mPanelParents.indexOf(selected);
	if (i >= 0) {
	  JInternalFrame f = (JInternalFrame)mInternalFrames.get(i);
	  f.show();
	}
      }
    }
  }
  
  // ignore other mouse events
  public void mouseEntered(MouseEvent e) { }
  public void mouseExited(MouseEvent e) { }
  public void mousePressed(MouseEvent e) { }
  public void mouseReleased(MouseEvent e) { }

  // ComponentListener events
  public void componentHidden(ComponentEvent e) { }
  /**
   * responds to movement of an internal frame by forcing a repaint
   * of the frame; otherwise there's a bad effect when the internal frame
   * is moved across the desktop boundary
   *
   * @param e ComponentEvent to respond to
   */
  public void componentMoved(ComponentEvent e) 
  {
    if (e.getSource() instanceof JInternalFrame) {
      JInternalFrame internalFrame = (JInternalFrame)e.getSource();
      internalFrame.repaint();
    }
  }
  public void componentResized(ComponentEvent e) { }
  public void componentShown(ComponentEvent e) { }
 
  /**
   * Responds to ActionEvents fired by menus 
   *
   * @param e The ActionEvent to respond to
   */
  public void actionPerformed(ActionEvent e)
  {
    if (e.getSource() instanceof JMenuItem) {
      JMenuItem menu = (JMenuItem)e.getSource();
      handleDefaultMenuClick(menu.getText());
    }
  }

  //
  // private routine to handle mouse clicks on default menus
  //
  private void handleDefaultMenuClick(String menuText)
  {
    JFileChooser fc;
    int fcResult;

    if (menuText.equals("Exit")) {
      System.exit(0);
    }
    else if (menuText.equals("Open")) {
      fc = new JFileChooser();
      fc.setCurrentDirectory(new File("c:\\Sandra\\Examples"));
      fcResult = fc.showOpenDialog(this);
      if (fcResult == JFileChooser.APPROVE_OPTION) {
	generateActionEvent("open", fc.getSelectedFile().getPath());
      }
    }
    else if (menuText.equals("Close")) {
      generateActionEvent("close", selectedObjectName());
    }
    else if (menuText.equals("Save")) {
      fc = new JFileChooser();
      fc.setCurrentDirectory(new File("c:\\Sandra\\Examples"));
      fcResult = fc.showSaveDialog(this);
      if ((fcResult == JFileChooser.APPROVE_OPTION) &&
	  checkFile(fc.getSelectedFile())){
  	generateActionEvent("save", fc.getSelectedFile().getPath());
      }
    }
    else if (menuText.equals("New")) {
      generateCreateEvent();
    }
  }

  //
  // private routine to double-check whether user wants to save over 
  // an existing file
  //
  private boolean checkFile(File filePath)
  {
    boolean ans = true;
    Object[] options = {"Yes", "No"};

    if (filePath.exists()) {
      ans = (0 == JOptionPane.showOptionDialog(this,
					       "Replace existing file " +
					       filePath.getName() + "?",
					       this.getTitle(),
					       JOptionPane.YES_NO_OPTION,
					       JOptionPane.QUESTION_MESSAGE,
					       null,
					       options,
					       options[1]));
    }

    return ans;
  }

  //
  // private routine to display a dialog, let the user choose an
  // object class to create, and fire an event for the client to catch
  //
  private void generateCreateEvent()
  {
    if (mCreatableClasses instanceof Vector) {
      int n = mCreatableClasses.size();
      String[] possibleValue = new String[n];
      Class c;
      int i;
      String selection;

      // build list of class names
      if (mCreatableClassNames instanceof Vector) {
	for (i=0; i<n; i++) {
	  possibleValue[i] = (String)mCreatableClassNames.get(i);
	}
      }
      else {
	for (i=0; i<n; i++) {
	  c = (Class)mCreatableClasses.get(i);
	  possibleValue[i] = c.getName();
	}
      }

      // display list dialog with class names to choose from
      ListDialog.initialize(mMenuBar, 
			    possibleValue, 
			    getTitle(),
			    "Select object to create: ");
      selection = ListDialog.showDialog(mMenuBar, "");
      if ((selection instanceof String) && !(selection.equals(""))) {
	if (mCreatableClassNames instanceof Vector) {
	  i = mCreatableClassNames.indexOf(selection);
	  if (i >= 0) {
	    generateActionEvent("create", 
				((Class)mCreatableClasses.get(i)).getName());
	  }
	}
	else {
	  generateActionEvent("create", selection);
	}
      }
    }
  }

  /** 
   * Invoked when an internal frame is activated--GUI responds by making
   * the appropriate object the default
   *
   * @param e event thrown when frame is activated
   */
  public void internalFrameActivated(InternalFrameEvent e) 
  {
    int i;
    Object parent;
    i = mInternalFrames.indexOf(e.getInternalFrame());
    if (i >= 0) {
      parent = mPanelParents.get(i);
      if (parent != (Object)null) {
	mOpenObjectList.setSelectedValue(parent,true);
      }
    }
  }

  /**
   * Invoked when an internal frame has been closed; then listeners
   * are notified
   *
   * @param e event thrown when frame is closed
   */
  public void internalFrameClosed(InternalFrameEvent e)
  {
    int i;
    Object parent;
    i = mInternalFrames.indexOf(e.getInternalFrame());
    if (i >= 0) {
      parent = mPanelParents.get(i);
      mInternalFrames.remove(i);
      mPanels.remove(i);
      mPanelParents.remove(i);
      if (!mPanelParents.contains(parent)) {
	mOpenObjects.remove(parent);
	mOpenObjectList.setListData(mOpenObjects);
	// send message to action listeners
	parent = (parent == null) ? this : parent;
	ActionEvent ae = new ActionEvent(parent, 
					 ActionEvent.ACTION_FIRST,
					 "close");
	for (Iterator pListener = mActionListeners.iterator();
	     pListener.hasNext(); ) {
	  ((ActionListener)pListener.next()).actionPerformed(ae);
	}
      }
    }
    i = mInternalFrames.indexOf(mDesktop.getSelectedFrame());
    if (i >= 0) {
      parent = mPanelParents.get(i);
      if (parent != null) {
	mOpenObjectList.setSelectedValue(parent, true);
      }
    }
  } 

  //
  // other internal frame listener events are ignored
  //
  public void internalFrameClosing(InternalFrameEvent e) { }
  public void internalFrameDeactivated(InternalFrameEvent e) { } 
  public void internalFrameDeiconified(InternalFrameEvent e) { } 
  public void internalFrameIconified(InternalFrameEvent e) { }
  public void internalFrameOpened(InternalFrameEvent e) { } 

  /**
   * Allows the client to programatically select an open object from the
   * list.
   *
   * @param openObject object to select
   */
  public void selectOpenObject(Object openObject)
  {
    mOpenObjectList.setSelectedValue(openObject, true);
  }

  /**
   * Returns a reference to the currently selected object
   * 
   * @return reference to selected object
   */
  public Object getSelectedOpenObject()
  {
    return mOpenObjectList.getSelectedValue();
  }

  /**
   * Provides the UI with a list of classes which can currently be created.
   *
   * @param creatableClasses Vector of creatable classes
   */
  public void setCreatableClassList(Vector creatableClasses)
  {
    mCreatableClasses = creatableClasses;
  }

  /**
   * Provides the UI with a list of the names associated with the classes
   * which can currently be created
   *
   * @param creatableClassNames Vector of Strings with names'
   * @throws IllegalArgumentException if invalid items are in vector
   */
  public void setCreatableClassNames(Vector creatableClassNames)
  throws IllegalArgumentException
  {
    if (!(mCreatableClasses instanceof Vector) ||
	mCreatableClasses.size() != creatableClassNames.size()) {
      throw new IllegalArgumentException();
    }
    mCreatableClassNames = creatableClassNames;
  }

  /**
   * Provides the UI with the contents of its Task menu.
   *
   * @param taskMenu JMenu containing Task menu for UI
   */
  public void setTaskMenu(JMenu taskMenu)
  {
    mTasksMenu = taskMenu;
    mTasksMenu.setText("Tasks");
    mTasksMenu.setMnemonic('t');
    setMenus();
  }

  /**
   * Provides the UI with the contents of its View menu.
   *
   * @param viewMenu JMenu containing View menu for UI
   */
  public void setViewMenu(JMenu viewMenu)
  {
    mViewMenu = viewMenu;
    mViewMenu.setText("View");
    mViewMenu.setMnemonic('v');
    setMenus();
  }

  /**
   * Adds an action listener to the UI.
   *
   * @param a The ActionListener to be added.
   */
  public void addActionListener(ActionListener a)
  {
    if (!(mActionListeners instanceof Vector)) {
      mActionListeners = new Vector();
    }

    mActionListeners.add(a);
  }

  /**
   * Displays the passed JPanel, indicates the parent object whose name should
   * be displayed on the list of open objects, and returns a handle for 
   * reference to the displayed panel.
   *
   * @param newPanel The panel to be displayed
   * @param caption Caption for the new panel's window
   * @param parent Parent object (name must show on open objects list)
   *
   * @return handle
   */
  public int displayPanel(JPanel newPanel, String caption, Object parent)
  {
    StringBuffer sCaption;
    int i;

    if (!(mInternalFrames instanceof Vector)) {
      mInternalFrames = new Vector();
      mPanels = new Vector();
      mPanelParents = new Vector();
    }
    i = mInternalFrames.size();

    if (!(caption instanceof String)) {
      sCaption = new StringBuffer(DEFAULT_CAPTION);
      sCaption.append(String.valueOf(1 + i));
    }
    else {
      sCaption = new StringBuffer(caption);
    }
    
    JInternalFrame newFrame = new JInternalFrame(sCaption.toString(),
                                                 true,
                                                 true);
    newFrame.getContentPane().setLayout(new BorderLayout());
    newFrame.getContentPane().add(newPanel, BorderLayout.CENTER);
    newFrame.addInternalFrameListener(this);
    newFrame.addComponentListener(this);
    mDesktop.add(newFrame);
    newFrame.setLocation(X_INTFRAME_INC * i, Y_INTFRAME_INC * i);
    newFrame.pack();
    Dimension newFrameSize = newFrame.getSize();
    int X = (int)newFrameSize.getWidth();
    int Y = (int)newFrameSize.getHeight();
    if (X < X_INTFRAME_MIN) {
      X = X_INTFRAME_MIN;
    }
    if (Y < Y_INTFRAME_MIN) {
      Y = Y_INTFRAME_MIN;
    }
    newFrameSize.setSize(X, Y);
    newFrame.setSize(newFrameSize);
    newFrame.setVisible(true);

    mInternalFrames.add(i, newFrame);
    mPanels.add(i, newPanel);
    mPanelParents.add(i, parent);

    if (!(parent == (Object)null)) {
      if (!(mOpenObjects.contains(parent))) {
	mOpenObjects.add(parent);
	mOpenObjectList.setListData(mOpenObjects);
      }
      mOpenObjectList.setSelectedValue(parent, true);
    }
    
    return i;
  }

  /** 
   * Removes the displayed panel with the specified handle from the display
   *
   * @param panelHandle The handle of the panel to remove
   */
  public void removePanel(JPanel thePanel)
  {
    int i;
    JInternalFrame theFrame;

    if (mPanels instanceof Vector) {
      i = mPanels.indexOf(thePanel);
      if ( i >= 0 ) {
        mPanels.remove(thePanel);
        theFrame = (JInternalFrame)mInternalFrames.get(i);
        mInternalFrames.remove(i);
	theFrame.setVisible(false);
        mDesktop.remove(theFrame);
      }
    }
  }

  /** 
   * Determines whether the panel with the specified handle should be
   * visible.
   *
   * @param handle The handle of the panel to query
   * @param isVisible Indicates whether the panel should be visible
   */
  public void setPanelVisible(JPanel thePanel, boolean isVisible)
  {
    int j;
    JInternalFrame theFrame;

    if (mPanels instanceof Vector) {
      j = mPanels.indexOf(thePanel);
      if (j >= 0) {
        theFrame = (JInternalFrame)mInternalFrames.get(j);
        theFrame.setVisible(isVisible);
      }
    }
  }

  /**
   * Indicates whether the panel with the specified handle is currently
   * visible.
   *
   * @param thePanel panel to check
   * @return boolean indication
   */
  public boolean getPanelVisible(JPanel thePanel)
  {
    int j;
    JInternalFrame theFrame;
    boolean bAnswer=false;

    if (mPanels instanceof Vector) {
      j = mPanels.indexOf(thePanel);
      if (j >= 0) {
        theFrame = (JInternalFrame)mInternalFrames.get(j);
        bAnswer = theFrame.isVisible();
      }
    }
    return bAnswer;
  }

  /**
   * Displays a dialog box with the passed message
   *
   * @param message message to display
   */
  public void showMessageDialog(Object message)
  {
    JOptionPane optionPane = new JOptionPane();
    optionPane.showMessageDialog((Component)this, message, getTitle(),
				 JOptionPane.INFORMATION_MESSAGE);
  }

  /**
   * Displays a warning dialog box with the passed message
   *
   * @param message message to display
   */
  public void showWarningDialog(Object message)
  {
    JOptionPane optionPane = new JOptionPane();
    optionPane.showMessageDialog((Component)this, message, getTitle(), 
				 JOptionPane.WARNING_MESSAGE);
  }

  /**
   * Displays an error dialog box with the passed message
   *
   * @param message message to display
   */
  public void showErrorDialog(Object message)
  {
    JOptionPane optionPane = new JOptionPane();
    optionPane.showMessageDialog((Component)this, message, getTitle(), 
				 JOptionPane.ERROR_MESSAGE);
  }

  /**
   * Displays an input dialog box with the passed message and returns
   * the string the user enters.
   *
   * @param message message to display
   */
  public String showInputDialog(Object message)
  {
    JOptionPane optionPane = new JOptionPane();
    return optionPane.showInputDialog(message);
  }

  /**
   * Adds a text message to the message display area of the screen
   *
   * @param message message to display
   */
  public void addMessage(String message)
  {
    String contents = mMessageArea.getText();
    if (contents.length() > 0) {
      contents += '\n';
    }
    contents += message;
    mMessageArea.setText(contents);
  }

  //
  // private method to construct frame
  //
  private void constructFrame()
  {
    constructMenus();
    setMenus();
    constructSplitPane();
    setSize(new Dimension(INIT_X, INIT_Y));
  }

  //
  // private method to construct menus
  //
  private void constructMenus()
  {
    //
    // file menu
    //
    mFileMenu = new JMenu("File");
    mFileMenu.setMnemonic('f');
    addMenuItem(mFileMenu, "New", 'n');
    addMenuItem(mFileMenu, "Open", 'o');
    addMenuItem(mFileMenu, "Save", 's');
    addMenuItem(mFileMenu, "Close", 'c');
    addMenuItem(mFileMenu, "Exit", 'x');
    //
    // edit menu
    //
    mEditMenu = new JMenu("Edit");
    mEditMenu.setMnemonic('e');
    addMenuItem(mEditMenu, "Cut", 't');
    addMenuItem(mEditMenu, "Copy", 'c');
    addMenuItem(mEditMenu, "Paste", 'p');
    //
    // tasks menu (empty until set by client)
    //
    mTasksMenu = new JMenu("Tasks");
    mTasksMenu.setMnemonic('t');
    //
    // view menu (empty until set by client)
    //
    mViewMenu = new JMenu("View");
    mViewMenu.setMnemonic('v');
    //
    // help menu
    //
    mHelpMenu = new JMenu("Help");
    mHelpMenu.setMnemonic('h');
    addMenuItem(mHelpMenu, "About matroids", 'a');
  }

  //
  // private routine to attach menus to frame
  //
  private void setMenus()
  {
    mMenuBar = new JMenuBar();
    mMenuBar.add(mFileMenu);
    mMenuBar.add(mEditMenu);
    mMenuBar.add(mTasksMenu);
    mMenuBar.add(mViewMenu);
    mMenuBar.add(mHelpMenu);
    setJMenuBar(mMenuBar);
  }

  //
  // quickie private routine to add a menu item
  //
  private void addMenuItem(JMenu menu, String name, char mnemonic)
  {
    JMenuItem menuItem = new JMenuItem(name);
    menuItem.setMnemonic(mnemonic);
    menu.add(menuItem);
    menuItem.addActionListener(this);
  }

  //
  // private method to construct split pane
  // 
  private void constructSplitPane()
  {
    JScrollPane scrollPaneLeft, scrollPaneRight, scrollPaneMessage;
    
    mOpenObjectList = new JList();
    mOpenObjectList.addMouseListener(this);
    mOpenObjectList.setPreferredSize(new Dimension(700, 200));
    mDesktop = new JDesktopPane();
    mDesktop.setPreferredSize(new Dimension(700,500));
    mMessageArea = new JTextArea(3, 80);
    mMessageArea.setEditable(false);
    Font f = mMessageArea.getFont();
    mMessageArea.setFont(new Font("Monospaced", Font.PLAIN, f.getSize()));


    scrollPaneLeft = 
      new JScrollPane(mOpenObjectList,
		      JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
		      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneRight = 
      new JScrollPane(mDesktop,
		      JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
		      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneMessage = 
      new JScrollPane(mMessageArea,
		      JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
		      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
					  scrollPaneLeft,
					  scrollPaneRight);
    JSplitPane outerSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
					       splitPane,
					       scrollPaneMessage);
    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(outerSplitPane, BorderLayout.CENTER);
    splitPane.setDividerLocation(200);
    outerSplitPane.setDividerLocation(400);
  }

  //
  // private method to return the name of the selected object on the JList
  // at the left of the screen
  //
  private String selectedObjectName()
  {
    String sAnswer;
    if (!mOpenObjectList.isSelectionEmpty()) {
      sAnswer = mOpenObjectList.getSelectedValue().toString();
    }
    else {
      sAnswer = new String("");
    }
    return sAnswer;
  }

  //
  // private method to generate an ActionEvent instance and pass it to
  // all ActionListeners associated with this object
  //
  private void generateActionEvent(String sVerb, String sObject)
  {
    ActionListener al;

    if (mActionListeners instanceof Vector) {
      StringBuffer sCommand = new StringBuffer(sVerb);
      sCommand.append(" ");
      sCommand.append(sObject);
  
      ActionEvent e = new ActionEvent(this, 
                                      ActionEvent.ACTION_PERFORMED,
                                      sCommand.toString());
      
      for (Iterator pListener = mActionListeners.iterator(); 
	   pListener.hasNext();) {
        al = (ActionListener)pListener.next();
        al.actionPerformed(e);
      }
    }
  }
}





