import Oid.*;
import java.util.*;

/**
 * A <code>SmartMatroid</code> performs matroid operations using a set
 * of generic matroid algorithms and a set of different
 * representations of the same matroid
 * 
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 13 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
public class SmartMatroid implements Matroid
{
  // 
  // private data members
  //
  private Vector       mReps;    // matroid represenations
  private String       mName;    // name of this matroid
  private String[]     mLabel;   // labels for matroid elements
  private int          mRank;    // rank of this matroid
  private int          mSize;    // size of this matroid
  // tasks and algorithm for this matroid
  private TaskAlgorithmContainer mTaskAlgorithmContainer; 
  private SmartMatroid mDual;    // dual of this matroid  
  // store reference to at least one algorithm (algorithm) for the matroid's 
  // built-in tasks and instances of their algorithms
  private Vector       mCircuitAlgorithms;
  private Vector       mFlatAlgorithms;
  private Vector       mBasisAlgorithms;
  private Vector       mDualAlgorithms;
  private Vector       mDeletionAlgorithms;
  private Vector       mContractionAlgorithms;
  private Vector       mIndependentSetAlgorithms;
  private Vector       mHyperplaneAlgorithms;
  private Vector       mCircuitAlgInstances;
  private Vector       mFlatAlgInstances;
  private Vector       mBasisAlgInstances;
  private Vector       mDualAlgInstances;
  private Vector       mDeletionAlgInstances;
  private Vector       mContractionAlgInstances;
  private Vector       mIndependentSetAlgInstances;
  private Vector       mHyperplaneAlgInstances;
  // currently selected algorithm and representative
  // (makes this non-thread safe)
  private Object       mCurAlgorithm;
  private Matroid      mCurRep;

  //
  // constructors
  //
  /** 
   * If <code>m</code> is an instance of SmartMatroid, this constructor
   * copies it.  Otherwise, it constructs itself with m as its single
   * representation.
   *
   * @param m <code>Matroid</code> to copy
   */
  public SmartMatroid(Matroid m)
  {
    setupSmartMatroid(m);
  }

  //
  // private method sets up instance using a matroid
  //
  private void setupSmartMatroid(Matroid m)
  {
    int i;
    mRank = m.getRank();
    mSize = m.getSize();
    mName = new String(m.getName());
    if (m instanceof SmartMatroid) {
      mReps = new Vector(((SmartMatroid)m).getRepresentations());
    } else {
      mReps = new Vector();
      mReps.add(m);
    }
    mLabel = new String[mSize];
    for (i=0; i<mSize; i++) {
      mLabel[i] = new String(m.getLabel(i));
    }
  }

  /** 
   * returns the vector containing the matroid representations 
   * for this SmartMatroid
   *
   * @return matroid representations
   */
  protected Vector getRepresentations()
  {
    return mReps;
  }

  /**
   * adds a representation to this SmartMatroid
   *
   * @param m matroid representation
   * @throws IllegalArgumentException if <code>m</code> is (obviously) not
   * the same as the other matroid representations
   */
  protected void addRepresentation(Matroid m)
  throws IllegalArgumentException
  {
    if ((m.getRank() != mRank) || (m.getSize() != mSize)) {
      throw new IllegalArgumentException("non-identical matroid in "
					 + "SmartMatroid");
    }
    m.setName(mName);
    for (int i = 0; i<mSize; i++) {
      m.setLabel(i, mLabel[i]);
    }

    mReps.add(m);
  }

  /** 
   * sets a reference to the dual of this matroid 
   *
   * @param myDual dual of matroid
   */
  protected void setDual(SmartMatroid myDual)
  {
    mDual = myDual;
  }

  /** 
   * returns the default representation for this matroid
   *
   * @return matroid representation
   */
  public Matroid getDefaultRepresentation() 
  {
    Matroid answer = null;
    if (mReps.size() > 0) {
      answer = (Matroid)mReps.iterator().next();
    }
    return answer;
  }

  /** 
   * returns (if possible) a representation of this matroid
   * matching the passed Class; otherwise, returns null
   *
   * @param c class to match
   * @return matroid representation as instance of <code>c</code> or 
   * <code>null</code>
   */
  public Matroid getRepresentation(Class c)
  {
    Matroid answer = (Matroid)null;
    boolean found = false;
    for (Iterator pRep = mReps.iterator(); pRep.hasNext();) {
      answer = (Matroid)pRep.next();
      if (c.isAssignableFrom(answer.getClass())) {
	found = true;
	break;
      }
    }
    if (!found) {
      answer = (Matroid)null;
    }
    return answer;
  }

  /** 
   * returns the rank of this matroid
   *
   * @return rank of matroid
   */
  public int getRank() 
  {
    return mRank;
  }

  /**
   * returns the size of this matroid
   *
   * @return size of matroid
   */
  public int getSize()
  {
    return mSize;
  }

  /**
   * sets the label for the passed element of this matroid 
   *
   * @param element index of element (0-based)
   * @param label label for element
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is out
   * of range
   */
  public void setLabel(int element, String label)
  throws ArrayIndexOutOfBoundsException
  {
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("Invalid matroid element " +
                                               element);
    }
    mLabel[element] = new String(label);
    Matroid m;
    for (Iterator pMatroid = mReps.iterator(); pMatroid.hasNext();) {
      m = (Matroid)pMatroid.next();
      m.setLabel(element, label);
    }
  }

  /** 
   * returns the label of the passed element of this matroid
   *
   * @param element index of element (0-based)
   * @return label for element
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is out
   */
  public String getLabel(int element)
  throws ArrayIndexOutOfBoundsException
  {
    if ((element < 0) || (element >= mSize)) {
      throw new ArrayIndexOutOfBoundsException("Invalid matroid element " +
                                               element);
    }
    return new String(mLabel[element]);
  }

  /** 
   * sets the name of this matroid 
   * 
   * @param newName new name for matroid
   */
  public void setName(String newName)
  {
    mName = new String(newName);
    Matroid m;
    for (Iterator pMatroid = mReps.iterator(); pMatroid.hasNext();) {
      m = (Matroid)pMatroid.next();
      m.setName(newName);
    }
  }

  /** 
   * returns the name of this matroid 
   *
   * @return name of matroid
   */
  public String getName()
  {
    return new String(mName);
  }

  /** 
   * returns the name of this matroid 
   *
   * @return name of matroid
   */
  public String toString()
  {
    return new String(mName);
  }

  /** 
   * returns last algorithm used 
   *
   * @return last algorithm reference
   */
  public Object lastAlgorithm()
  {
    return mCurAlgorithm;
  }

  /** 
   * returns last representation used 
   *
   * @return last representation reference
   */
  public Matroid lastRepresentation()
  {
    return mCurRep;
  }

  /** 
   * sets the reference to the <code>TaskAlgorithmContainer</code>
   * with information on algorithms for this matroid
   *
   * @param newTaskAlgorithmContainer reference to container
   * @throws IllegalArgumentException if container is invalid
   */
  public void setTaskAlgorithmContainer(TaskAlgorithmContainer 
					newTaskAlgorithmContainer)
  throws IllegalArgumentException
  {
    mTaskAlgorithmContainer = newTaskAlgorithmContainer;
    getDefaultAlgorithms();
  }

  /**
   * returns the reference to the TaskAlgorithmContainer for this matroid
   *
   * @return reference to container for this object
   */
  public TaskAlgorithmContainer getTaskAlgorithmContainer()
  {
    return mTaskAlgorithmContainer;
  }

  /**
   * returns a representation of this matroid's state in the passed format
   *
   * @param format format for return string (i.e., "text")
   * @throws IllegalArgumentException if <code>format</code> is invalid
   */
  public String getState(String format)
  throws IllegalArgumentException
  {
    // for now, just return the state from the first matroid on the list
    return((Matroid)mReps.get(0)).getState(format);
  }

  /**
   * sets the state of this matroid using the passed string, in the 
   * passed format
   * 
   * @param state string describing state of matroid
   * @param format format of string (i.e., "text")
   * @throws IllegalArgumentException if <code>state</code> or 
   * <code>format</code> is invalid
   */
  public void setState(String state, String format)
  throws IllegalArgumentException
  {

    if (format.equals("text")) {
      StringTokenizer st = new StringTokenizer(state);

      if (!(st.hasMoreElements())) {
        throw new IllegalArgumentException("Empty state string");
      }
      String typeOfMatroid = (String)st.nextElement();
      if (typeOfMatroid.toLowerCase().equals("finitefieldmatroid")) {
        FiniteFieldMatroid ffm = new FiniteFieldMatroid();
        ffm.setState(state, format);
        setupSmartMatroid(ffm);
      } else {
        throw new IllegalArgumentException("Unrecognized matroid type: " +
                                           typeOfMatroid);
      }
    } else {
      throw new IllegalArgumentException("Unrecognized format: " + format);
    }
  }

  /**
   * determines whether an element is in the closure of a set
   *
   * @param s <code>Subset</code> of matroid ground set
   * @param e element index (0-based)
   * @return <code>true</code> if and only if <code>e</code> is in the closure
   * of <code>s</code>
   */
  public boolean isInClosure(Subset s, int e) 
  {
    if ((e < 0) || (e >= mSize) || (s.getSize() != mSize)) {
      return false;
    }
    Matroid m = (Matroid)mReps.get(0);
    return m.isInClosure(s,e);
  }

  /** 
   * returns the closure of the passed Subset in this matroid
   * 
   * @param s <code>Subset</code> of matroid ground set
   * @return closure of <code>s</code>
   */
  public Subset closure(Subset s)
  {
    // ToDo : make this method "smart"
    Matroid m = (Matroid)mReps.get(0);
    return m.closure(s);
  }

  /**
   * determines whether the passed subset is a basis of this matroid
   *
   * @param s <code>Subset</code> of matroid ground set
   * @return <code>true</code> if and only if <code>s</code> is a basis
   */
  public boolean isBasis(Subset s)
  {
    // ToDo : make this method "smart"
    Matroid m = (Matroid)mReps.get(0);
    return m.isBasis(s);
  }

  /**
   * determines whether the passed subset is a circuit of this matroid
   *
   * @param s <code>Subset</code> of matroid ground set
   * @return <code>true</code> if and only if <code>s</code> is a circuit
   */
  public boolean isCircuit(Subset s)
  {
    // ToDo : make this method "smart"
    Matroid m = (Matroid)mReps.get(0);
    return m.isCircuit(s);
  }

  /**
   * determines whether the passed subset is a flat of this matroid
   *
   * @param s <code>Subset</code> of matroid ground set
   * @return <code>true</code> if and only if <code>s</code> is a flat
   */
  public boolean isFlat(Subset s)
  {
    // ToDo : make this method "smart"
    Matroid m = (Matroid)mReps.get(0);
    return m.isFlat(s);
  }

  /**
   * determines whether the passed subset is a hyperplane of this matroid
   *
   * @param s <code>Subset</code> of matroid ground set
   * @return <code>true</code> if and only if <code>s</code> is a hyperplane
   */
  public boolean isHyperplane(Subset s)
  {
    // ToDo : make this method "smart"
    Matroid m = (Matroid)mReps.get(0);
    return m.isHyperplane(s);
  }

  /** 
   * determines whether the passed subset of this matroid is independent
   *
   * @param s <code>Subset</code> of matroid ground set
   * @return <code>true</code> if and only if <code>s</code> is independent
   */
  public boolean isIndependent(Subset s)
  {
    // ToDo : make this method "smart"
    Matroid m = (Matroid)mReps.get(0);
    return m.isIndependent(s);
  }

  /** 
   * determines the rank of the passed subset of this matroid
   *
   * @param s <code>Subset</code> of matroid ground set
   * @return rank of <code>s</code> in this matroid
   */
  public int subsetRank(Subset s)
  {
    // ToDo : make this method "smart"
    Matroid m = (Matroid)mReps.get(0);
    return m.subsetRank(s);
  }

  /**
   * returns a collection of this matroid's sets using the passed algorithm
   *
   * @param algorithm how to get the sets
   * @return collection of sets
   * @throws IllegalArgumentException if <code>algorithm</code> won't work
   */
  public Collection applySubsetLister(SubsetLister algorithm)
  throws IllegalArgumentException
  {
    Collection answer;

    // ToDo : "smart" way to decide which representative to use
    algorithm.setInstance((Matroid)mReps.get(0));
    try {
      answer = algorithm.getSubsetListVector();
    } catch (NullPointerException e) {
      // this "can't happen"
      throw new InternalError("Null pointer in SmartMatroid.getSubsetList");
    }
    return answer;
  }

  /** 
   * calculates a property using the passed algorithm
   *
   * @param algorithm how to get the property
   * @return calculated property
   * @throws IllegalArgumentException if <code>algorithm</code> won't work
   */
  public int applyPropertyCalculator(PropertyCalculator algorithm)
  throws IllegalArgumentException
  {
    int answer;

    // ToDo : "smart" way to pick representation
    if (algorithm.requiresParameter()) {
      throw new IllegalArgumentException("Parameter required");
    }
    algorithm.setInstance((Matroid)mReps.get(0));
    answer = algorithm.getProperty();
    return answer;
  }

  /** 
   * calculates a property using the passed algorithm and parameter
   *
   * @param algorithm how to get the property
   * @param parameter parameter for algorithm
   * @return calculated property
   * @throws IllegalArgumentException if <code>algorithm</code> won't work
   */
  public int applyPropertyCalculator(PropertyCalculator algorithm, 
				     int parameter)
  throws IllegalArgumentException
  {
    int answer;

    // ToDo : "smart" way to pick representation
    algorithm.setInstance((Matroid)mReps.get(0));
    algorithm.setParameter(parameter);
    answer = algorithm.getProperty();
    return answer;
  }

  /**
   * constructs an object using the passed algorithm
   *
   * @param algorithm how to get the object
   * @return constructed object
   * @throws IllegalArgumentException if <code>algorithm</code> won't work
   */
  public Object applySingleConstructor(SingleConstructor algorithm)
  throws IllegalArgumentException
  {
    Object answer;

    // ToDo : "smart" way to pick representation
    if (algorithm.requiresParameter()) {
      throw new IllegalArgumentException("Parameter required");
    }
    algorithm.setInstance((Matroid)mReps.get(0));
    answer = algorithm.getConstruction();
    return answer;
  }

  /** 
   * constructs an object using the passed algorithm and parameter
   *
   * @param algorithm how to get the object
   * @param params String with parameters for algorithm
   * @return constructed object
   * @throws IllegalArgumentException if <code>algorithm</code> won't work
   */
  public Object applySingleConstructor(SingleConstructor algorithm, 
				       String params)
  throws IllegalArgumentException
  {
    Object answer;

    // ToDo : "smart" way to pick representation
    algorithm.setInstance((Matroid)mReps.get(0));
    algorithm.setParameters(params);
    answer = algorithm.getConstruction();
    return answer;
  }

  /**
   * constructs a collection of objects using the passed algorithm
   *
   * @param algorithm how to get the collection
   * @return constructed collection
   * @throws IllegalArgumentException if <code>algorithm</code> won't work
   */
  public Collection applyCollectionConstructor(CollectionConstructor algorithm)
  throws IllegalArgumentException
  {
    Collection answer;

    // ToDo : "smart" way to pick representation
    algorithm.setInstance((Matroid)mReps.get(0));
    answer = algorithm.getConstructions();
    return answer;
  }

  /** 
   * constructs a collection of objects using the passed algorithm and 
   * parameter
   *
   * @param algorithm how to get the property
   * @param parameter parameter for algorithm
   * @return constructed object
   * @throws IllegalArgumentException if <code>algorithm</code> won't work
   */
  public Collection applyCollectionConstructor(CollectionConstructor algorithm,
					       int parameter)
  throws IllegalArgumentException
  {
    Collection answer;

    // ToDo : "smart" way to pick representation
    algorithm.setInstance((Matroid)mReps.get(0));
    algorithm.setParameter(parameter);
    answer = algorithm.getConstructions();
    return answer;
  }

  /** 
   * given the passed Task, selects a SubsetLister algorithm, runs it, and 
   * returns the algorithm (which contains the result)
   *
   * @param t task to perform
   * @return <code>SubsetLister</code> implementation with results
   * @throws IllegalArgumentException if no algorithm is available
   * @throws IllegalStateException if no algorithm works
   */
  public SubsetLister performSubsetListerTask(Task t)
    throws IllegalArgumentException, IllegalStateException
  {
    SubsetLister answer = null;
    Collection algorithms;
    Algorithm algorithm;
    Vector algInstances;
    Object algInstance;
    Iterator pAlgorithms;
    long bestEstimate = -1;

    // build vector of algorithms that perform this task
    algorithms = mTaskAlgorithmContainer.getAlgorithms(t);
    if (algorithms.size() == 0) {
      throw new IllegalArgumentException("no algorithm available for " +
					 t.toString());
    }

    algInstances = new Vector();
    try {
      for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext(); ) {
	algorithm = (Algorithm)pAlgorithms.next();
	// doesn't hurt to double-check
	algInstance = algorithm.getInstance();
	if (algInstance instanceof SubsetLister) {
	  algInstances.add(algInstance);
	}
      }
    }
    catch (Exception e) {
      throw new IllegalStateException(e.toString());
    }
    bestEstimate = selectAlgRep("getSubsetList", algInstances);
    if (bestEstimate >= 0) {
      answer = (SubsetLister)mCurAlgorithm;
      answer.setInstance(mCurRep);
    }
    else {
      answer = null;
    }
    return answer;
  }

  /** 
   * given the passed Task, selects a PropertyCalculator algorithm, runs it
   * and returns the algorithm (which contains the result)
   *
   * @param t task to perform
   * @return <code>PropertyCalculator</code> implementation with result
   * @throws IllegalArgumentException if no algorithm is available
   * @throws IllegalStateException if no algorithm works
   */
  public PropertyCalculator performPropertyCalculatorTask(Task t)
    throws IllegalArgumentException, IllegalStateException
  {
    PropertyCalculator answer = null;
    Collection algorithms;
    Algorithm algorithm;
    Vector algInstances;
    Object algInstance;
    Iterator pAlgorithms;
    long bestEstimate = -1;

    // build vector of algorithms that perform this task
    algorithms = mTaskAlgorithmContainer.getAlgorithms(t);
    if (algorithms.size() == 0) {
      throw new IllegalArgumentException("no algorithm available for " +
					 t.toString());
    }

    algInstances = new Vector();
    try {
      for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext(); ) {
	algorithm = (Algorithm)pAlgorithms.next();
	// doesn't hurt to double-check
	algInstance = algorithm.getInstance();
	if (algInstance instanceof PropertyCalculator) {
	  algInstances.add(algInstance);
	}
      }
    }
    catch (Exception e) {
      throw new IllegalStateException(e.toString());
    }
    bestEstimate = selectAlgRep("getProperty", algInstances);
    if (bestEstimate >= 0) {
      answer = (PropertyCalculator)mCurAlgorithm;
      answer.setInstance(mCurRep);
      if (answer.requiresParameter()) {
	answer.setParameter(0);
      }
    }
    else {
      answer = null;
    }
    return answer;
  }
  
  /** 
   * given the passed Task and parameter, selects a PropertyCalculator 
   * algorithm, runs it and returns the algorithm (which contains the result)
   *
   * @param t task to perform
   * @param parameter parameter for algorithm
   * @return <code>PropertyCalculator</code> implementation with result
   * @throws IllegalArgumentException if no algorithm is available
   * @throws IllegalStateException if no algorithm works
   */
  public PropertyCalculator performPropertyCalculatorTask(Task t,
							  int parameter)
    throws IllegalArgumentException, IllegalStateException
  {
    PropertyCalculator answer = null;
    Collection algorithms;
    Algorithm algorithm;
    Vector algInstances;
    Object algInstance;
    Iterator pAlgorithms;
    long bestEstimate = -1;

    // build vector of algorithms that perform this task
    algorithms = mTaskAlgorithmContainer.getAlgorithms(t);
    if (algorithms.size() == 0) {
      throw new IllegalArgumentException("no algorithm available for " +
					 t.toString());
    }

    algInstances = new Vector();
    try {
      for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext(); ) {
	algorithm = (Algorithm)pAlgorithms.next();
	// doesn't hurt to double-check
	algInstance = algorithm.getInstance();
	if (algInstance instanceof PropertyCalculator) {
	  algInstances.add(algInstance);
	}
      }
    }
    catch (Exception e) {
      throw new IllegalStateException(e.toString());
    }
    bestEstimate = selectAlgRep("getProperty", algInstances);
    if (bestEstimate >= 0) {
      answer = (PropertyCalculator)mCurAlgorithm;
      answer.setInstance(mCurRep);
      if (answer.requiresParameter()) {
	answer.setParameter(parameter);
      }
    }
    else {
      answer = null;
    }
    return answer;
  }
  
  /** 
   * given the passed Task, selects a SingleConstructor algorithm, runs it
   * and returns the algorithm (which contains the result)
   *
   * @param t task to perform
   * @return <code>SingleConstructor</code> implementation with result
   * @throws IllegalArgumentException if no algorithm is available
   * @throws IllegalStateException if no algorithm works
   */
  public SingleConstructor performSingleConstructorTask(Task t)
    throws IllegalArgumentException, IllegalStateException
  {
    SingleConstructor answer = null;
    Collection algorithms;
    Algorithm algorithm;
    Vector algInstances;
    Object algInstance;
    Iterator pAlgorithms;
    long bestEstimate = -1;

    // build vector of algorithms that perform this task
    algorithms = mTaskAlgorithmContainer.getAlgorithms(t);
    if (algorithms.size() == 0) {
      throw new IllegalArgumentException("no algorithm available for " +
					 t.toString());
    }

    algInstances = new Vector();
    try {
      for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext(); ) {
	algorithm = (Algorithm)pAlgorithms.next();
	// doesn't hurt to double-check
	algInstance = algorithm.getInstance();
	if (algInstance instanceof SingleConstructor) {
	  algInstances.add(algInstance);
	}
      }
    }
    catch (Exception e) {
      throw new IllegalStateException(e.toString());
    }
    bestEstimate = selectAlgRep("getConstruction", algInstances);
    if (bestEstimate >= 0) {
      answer = (SingleConstructor)mCurAlgorithm;
      answer.setInstance(mCurRep);
    }
    else {
      answer = null;
    }
    return answer;
  }

  /** 
   * given the passed Task and parameter, selects a SingleConstructor 
   * algorithm, runs it and returns the algorithm (which contains the result)
   *
   * @param t task to perform
   * @param parameter parameter for algorithm
   * @return <code>SingleConstructor</code> implementation with result
   * @throws IllegalArgumentException if no algorithm is available
   * @throws IllegalStateException if no algorithm works
   */
  public SingleConstructor performSingleConstructorTask(Task t,
							String params)
    throws IllegalArgumentException, IllegalStateException
  {
    SingleConstructor answer = null;
    Collection algorithms;
    Algorithm algorithm;
    Vector algInstances;
    Object algInstance;
    Iterator pAlgorithms;
    long bestEstimate = -1;

    // build vector of algorithms that perform this task
    algorithms = mTaskAlgorithmContainer.getAlgorithms(t);
    if (algorithms.size() == 0) {
      throw new IllegalArgumentException("no algorithm available for " +
					 t.toString());
    }

    algInstances = new Vector();
    try {
      for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext(); ) {
	algorithm = (Algorithm)pAlgorithms.next();
	// doesn't hurt to double-check
	algInstance = algorithm.getInstance();
	if (algInstance instanceof SingleConstructor) {
	  algInstances.add(algInstance);
	}
      }
    }
    catch (Exception e) {
      throw new IllegalStateException(e.toString());
    }
    bestEstimate = selectAlgRep("getConstruction", algInstances);
    if (bestEstimate >= 0) {
      answer = (SingleConstructor)mCurAlgorithm;
      answer.setInstance(mCurRep);
      if (answer.requiresParameter()) {
	answer.setParameters(params);
      }
    }
    else {
      answer = null;
    }
    return answer;
  }

  /**
   * given the passed Task, selects a CollectionConstructor algorithm,
   * runs it and returns the algorithm (which contains the result)
   *
   * @param t task to perform
   * @return <code>CollectionConstructor</code> implementation with result
   * @throws IllegalArgumentException if no algorithm is available
   * @throws IllegalStateException if no algorithm works
   */
  public CollectionConstructor performCollectionConstructorTask(Task t) 
    throws IllegalArgumentException, IllegalStateException
  {
    CollectionConstructor answer = null;
    Collection algorithms;
    Algorithm algorithm;
    Vector algInstances;
    Object algInstance;
    Iterator pAlgorithms;
    long bestEstimate = -1;

    // build vector of algorithms that perform this task
    algorithms = mTaskAlgorithmContainer.getAlgorithms(t);
    if (algorithms.size() == 0) {
      throw new IllegalArgumentException("no algorithm available for " +
					 t.toString());
    }

    algInstances = new Vector();
    try {
      for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext(); ) {
	algorithm = (Algorithm)pAlgorithms.next();
	// doesn't hurt to double-check
	algInstance = algorithm.getInstance();
	if (algInstance instanceof CollectionConstructor) {
	  algInstances.add(algInstance);
	}
      }
    }
    catch (Exception e) {
      throw new IllegalStateException(e.toString());
    }
    bestEstimate = selectAlgRep("getConstructions", algInstances);
    if (bestEstimate >= 0) {
      answer = (CollectionConstructor)mCurAlgorithm;
      answer.setInstance(mCurRep);
      if (answer.requiresParameter()) {
	answer.setParameter(0);
      }
    }
    else {
      answer = null;
    }
    return answer;
  }

  /**
   * given the passed Task and parameter, selects a CollectionConstructor 
   * algorithm, runs it and returns the algorithm (which contains the result)
   *
   * @param t task to perform
   * @param parameter parameter for algorithm
   * @return <code>CollectionConstructor</code> implementation with result
   * @throws IllegalArgumentException if no algorithm is available
   * @throws IllegalStateException if no algorithm works
   */
  public CollectionConstructor performCollectionConstructorTask(Task t,
								int parameter) 
    throws IllegalArgumentException, IllegalStateException
  {
    CollectionConstructor answer = null;
    Collection algorithms;
    Algorithm algorithm;
    Vector algInstances;
    Object algInstance;
    Iterator pAlgorithms;
    long bestEstimate = -1;

    // build vector of algorithms that perform this task
    algorithms = mTaskAlgorithmContainer.getAlgorithms(t);
    if (algorithms.size() == 0) {
      throw new IllegalArgumentException("no algorithm available for " +
					 t.toString());
    }

    algInstances = new Vector();
    try {
      for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext(); ) {
	algorithm = (Algorithm)pAlgorithms.next();
	// doesn't hurt to double-check
	algInstance = algorithm.getInstance();
	if (algInstance instanceof CollectionConstructor) {
	  algInstances.add(algInstance);
	}
      }
    }
    catch (Exception e) {
      throw new IllegalStateException(e.toString());
    }
    bestEstimate = selectAlgRep("getConstructions", algInstances);
    if (bestEstimate >= 0) {
      answer = (CollectionConstructor)mCurAlgorithm;
      answer.setInstance(mCurRep);
      if (answer.requiresParameter()) {
	answer.setParameter(parameter);
      }
    }
    else {
      answer = null;
    }
    return answer;
  }

  /** 
   * returns a collection of this matroid's bases using the default algorithm
   *
   * @return collection of bases
   * @throws IllegalStateException if no basis algorithm is available
   */
  public Collection getBases()
    throws IllegalStateException
  {
    Collection answer;
    SubsetLister algInstance;
    Matroid rep;
    long bestEstimate = selectAlgRep("getSubsetList", mBasisAlgInstances);
    if (bestEstimate < 0) {
      throw new IllegalStateException("no basis algorithm available");
    }
    else {
      algInstance = (SubsetLister)mCurAlgorithm;
      rep = mCurRep;
      try {
	algInstance.setInstance(rep);
	answer = algInstance.getSubsetList();
      }
      catch (Exception e) {
	throw new InternalError(e.toString());
      }
    }

    boolean found = false;
    if (answer instanceof Collection) {
      for (Iterator pRep = mReps.iterator(); pRep.hasNext(); ) {
	if (pRep.next() instanceof BasisMatroid) {
	  found = true;
	  break;
	}
      }
      if (!found) {
	mReps.add(new BasisMatroid(answer, mName, mLabel));
      }
    }

    return answer;
  }

  

  /**
   * returns a collection of this matroid's circuits using the default 
   * algorithm
   *
   * @return collection of circuits
   * @throws IllegalStateException if no algorithm/representation available
   */
  public Collection getCircuits()
    throws IllegalStateException
  {
    Collection answer;
    SubsetLister algInstance;
    Matroid rep;
    long bestEstimate = selectAlgRep("getSubsetList", mCircuitAlgInstances);
    if (bestEstimate < 0) {
      throw new IllegalStateException("no circuit algorithm available");
    }
    else {
      algInstance = (SubsetLister)mCurAlgorithm;
      rep = mCurRep;
      try {
	algInstance.setInstance(rep);
	answer = algInstance.getSubsetList();
      }
      catch (Exception e) {
	throw new InternalError(e.toString());
      }
    }
    return(Collection)answer;
  }

  /** 
   * returns a collection of this matroid's flats using the default algorithm
   *
   * @return collection of flats
   * @throws IllegalStateException if no algorithm/representation available
   */
  public Collection getFlats()
    throws IllegalStateException
  {
    Collection answer;
    SubsetLister algInstance;
    Matroid rep;
    long bestEstimate = selectAlgRep("getSubsetList", mFlatAlgInstances);
    if (bestEstimate < 0) {
      throw new IllegalStateException("no flat algorithm available");
    }
    else {
      algInstance = (SubsetLister)mCurAlgorithm;
      rep = mCurRep;
      try {
	algInstance.setInstance(rep);
	answer = algInstance.getSubsetList();
      }
      catch (Exception e) {
	throw new InternalError(e.toString());
      }
    }
    return(Collection)answer;
  }

  /** 
   * returns a collection of this matroid's hyperplanes using the default 
   * algorithm
   *
   * @return collection of hyperplanes
   * @throws IllegalStateException if no algorithm/representation available
   */
  public Collection getHyperplanes()
    throws IllegalStateException
  {
    Collection answer;
    SubsetLister algInstance;
    Matroid rep;
    long bestEstimate = selectAlgRep("getSubsetList", mHyperplaneAlgInstances);
    if (bestEstimate < 0) {
      throw new IllegalStateException("no hyperplane algorithm available");
    }
    else {
      algInstance = (SubsetLister)mCurAlgorithm;
      rep = mCurRep;
      try {
	algInstance.setInstance(rep);
	answer = algInstance.getSubsetList();
      }
      catch (Exception e) {
	throw new InternalError(e.toString());
      }
    }
    return(Collection)answer;
  }

  /** 
   * returns a collection of this matroid's independent sets using the default 
   * algorithm
   *
   * @return collection of independent sets
   * @throws IllegalStateException if no algorithm/representation available
   */
  public Collection getIndependentSets()
    throws IllegalStateException
  {
    Collection answer;
    SubsetLister algInstance;
    Matroid rep;
    long bestEstimate = selectAlgRep("getSubsetList", 
				     mIndependentSetAlgInstances);
    if (bestEstimate < 0) {
      throw new IllegalStateException("no independent set " +
				      "algorithm available");
    }
    else {
      algInstance = (SubsetLister)mCurAlgorithm;
      rep = mCurRep;
      try {
	algInstance.setInstance(rep);
	answer = algInstance.getSubsetList();
      }
      catch (Exception e) {
	throw new InternalError(e.toString());
      }
    }
    return(Collection)answer;
  }

  /**
   * generates a matroid by contracting the selected element of this
   * matroid, and returns it as a SmartMatroid instance
   *
   * @param element index of element to contract
   * @return matroid with element contracted
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is out
   * of range
   * @throws IllegalStateException if no algorithm is available
   */
  public SmartMatroid getContraction(int element)
    throws ArrayIndexOutOfBoundsException, IllegalStateException
  {
    Matroid answer;
    SingleConstructor algInstance;
    Matroid rep;
    long bestEstimate = selectAlgRep("getConstruction", 
				     mContractionAlgInstances);
    if (bestEstimate < 0) {
      throw new IllegalStateException("no contraction algorithm available");
    }
    else {
      algInstance = (SingleConstructor)mCurAlgorithm;
      rep = mCurRep;
      try {
	algInstance.setInstance(rep);
	algInstance.setParameters(String.valueOf(element));
	answer = (Matroid)algInstance.getConstruction();
      }
      catch (Exception e) {
	throw new InternalError(e.toString());
      }
    }
    return new SmartMatroid(answer);
  }

  /**
   * generates a matroid by deleting the selected element of this 
   * matroid, and returns it as a SmartMatroid instance
   *
   * @param element index of element to delete
   * @return matroid with element deleted
   * @throws ArrayIndexOutOfBoundsException if <code>element</code> is out
   * of range
   * @throws IllegalStateException if no algorithm is available
   */
  public SmartMatroid getDeletion(int element)
    throws ArrayIndexOutOfBoundsException, IllegalStateException
  {
    Matroid answer;
    SingleConstructor algInstance;
    Matroid rep;
    long bestEstimate = selectAlgRep("getConstruction", 
				     mDeletionAlgInstances);
    if (bestEstimate < 0) {
      throw new IllegalStateException("no deletion algorithm available");
    }
    else {
      algInstance = (SingleConstructor)mCurAlgorithm;
      rep = mCurRep;
      try {
	algInstance.setInstance(rep);
	algInstance.setParameters(String.valueOf(element));
	answer = (Matroid)algInstance.getConstruction();
      }
      catch (Exception e) {
	throw new InternalError(e.toString());
      }
    }
    return new SmartMatroid(answer);
  }

  /** 
   * generates the dual of this matroid and returns it as a SmartMatroid
   * instance
   *
   * @return dual of matroid
   */
  public SmartMatroid getDual()
  {
    SmartMatroid answer = null;
    Object rep;
    Matroid dual;
    for (Iterator pRep = mReps.iterator(); pRep.hasNext(); ) {
      rep = pRep.next();
      if (rep instanceof FiniteFieldMatroid) {
	dual = ((FiniteFieldMatroid)rep).getDual();
	if (!(answer instanceof SmartMatroid)) {
	  answer = new SmartMatroid(dual);
	}
	else {
	  answer.addRepresentation(dual);
	}
      }
      else if (rep instanceof BasisMatroid) {
	dual = ((BasisMatroid)rep).getDual();
	if (!(answer instanceof SmartMatroid)) {
	  answer = new SmartMatroid(dual);
	}
	else {
	  answer.addRepresentation(dual);
	}
      }
    }
    return answer;
  }

  //
  // private method looks for a Algorithm for the SmartMatroid's built-in tasks
  //
  private void getDefaultAlgorithms()
  {
    Collection tasks = mTaskAlgorithmContainer.getTasks();
    Collection algorithms;
    Task task;
    Algorithm algorithm;
    String taskName;
    Iterator pAlgorithms;
    Object algInstance;
    boolean madeInstance;

    mCircuitAlgorithms = null;
    mFlatAlgorithms = null;
    mBasisAlgorithms = null;
    mDualAlgorithms = null;
    mDeletionAlgorithms = null;
    mContractionAlgorithms = null;
    mIndependentSetAlgorithms = null;
    mHyperplaneAlgorithms = null;

    for (Iterator pTask = tasks.iterator(); pTask.hasNext();) {
      task = (Task)pTask.next();
      taskName = task.getName();
      if (taskName.equals("getCircuits") && 
	  (mTaskAlgorithmContainer.getNumAlgorithms(task) > 0)) {
	// 
	// go through circuit algorithms, try to create instances of the 
	// classes; if successful, add them to the algorithm list
        mCircuitAlgorithms = new Vector();
	mCircuitAlgInstances = new Vector();
	algorithms = mTaskAlgorithmContainer.getAlgorithms(task);
	for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext();) {
	  algorithm = (Algorithm)pAlgorithms.next();
	  try {
	    algInstance = algorithm.getInstance();
	    mCircuitAlgorithms.add(algorithm);
	    mCircuitAlgInstances.add(algInstance);
	  }
	  catch (Exception e) {
	    // don't do anything, just don't add the algorithm and the algorithm
	  }
	}
      } else if (taskName.equals("getFlats") && 
		 (mTaskAlgorithmContainer.getNumAlgorithms(task) > 0)) {
	// 
	// go through flat algorithms, try to create instances of the 
	// classes; if successful, add them to the algorithm list
        mFlatAlgorithms = new Vector();
	mFlatAlgInstances = new Vector();
	algorithms = mTaskAlgorithmContainer.getAlgorithms(task);
	for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext();) {
	  algorithm = (Algorithm)pAlgorithms.next();
	  try {
	    algInstance = algorithm.getInstance();
	    mFlatAlgorithms.add(algorithm);
	    mFlatAlgInstances.add(algInstance);
	  }
	  catch (Exception e) {
	    // don't do anything, just don't add the algorithm and the algorithm
	  }
	}
      } else if (taskName.equals("getBases") && 
		 (mTaskAlgorithmContainer.getNumAlgorithms(task) > 0)) {
	// 
	// go through basis algorithms, try to create instances of the 
	// classes; if successful, add them to the algorithm list
        mBasisAlgorithms = new Vector();
	mBasisAlgInstances = new Vector();
	algorithms = mTaskAlgorithmContainer.getAlgorithms(task);
	for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext();) {
	  algorithm = (Algorithm)pAlgorithms.next();
	  try {
	    algInstance = algorithm.getInstance();
	    mBasisAlgorithms.add(algorithm);
	    mBasisAlgInstances.add(algInstance);
	  }
	  catch (Exception e) {
	    // don't do anything, just don't add anything
	  }
	}
      } else if (taskName.equals("getDual") && 
		 (mTaskAlgorithmContainer.getNumAlgorithms(task) > 0)) {
	// 
	// go through dual algorithms, try to create instances of the 
	// classes; if successful, add them to the algorithm list
        mDualAlgorithms = new Vector();
	mDualAlgInstances = new Vector();
	algorithms = mTaskAlgorithmContainer.getAlgorithms(task);
	for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext();) {
	  algorithm = (Algorithm)pAlgorithms.next();
	  try {
	    algInstance = algorithm.getInstance();
	    mDualAlgorithms.add(algorithm);
	    mDualAlgInstances.add(algInstance);
	  }
	  catch (Exception e) {
	    // don't do anything, just don't add the algorithm and the algorithm
	  }
	}
      } else if (taskName.equals("getDeletion") && 
		 (mTaskAlgorithmContainer.getNumAlgorithms(task) > 0)) {
	// 
	// go through dual algorithms, try to create instances of the 
	// classes; if successful, add them to the algorithm list
        mDeletionAlgorithms = new Vector();
	mDeletionAlgInstances = new Vector();
	algorithms = mTaskAlgorithmContainer.getAlgorithms(task);
	for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext();) {
	  algorithm = (Algorithm)pAlgorithms.next();
	  try {
	    algInstance = algorithm.getInstance();
	    mDeletionAlgorithms.add(algorithm);
	    mDeletionAlgInstances.add(algInstance);
	  }
	  catch (Exception e) {
	    // don't do anything, just don't add the algorithm and the algorithm
	  }
	}
      } else if (taskName.equals("getContraction") && 
		 (mTaskAlgorithmContainer.getNumAlgorithms(task) > 0)) {
	// 
	// go through dual algorithms, try to create instances of the 
	// classes; if successful, add them to the algorithm list
        mContractionAlgorithms = new Vector();
	mContractionAlgInstances = new Vector();
	algorithms = mTaskAlgorithmContainer.getAlgorithms(task);
	for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext();) {
	  algorithm = (Algorithm)pAlgorithms.next();
	  try {
	    algInstance = algorithm.getInstance();
	    mContractionAlgorithms.add(algorithm);
	    mContractionAlgInstances.add(algInstance);
	  }
	  catch (Exception e) {
	    // don't do anything, just don't add the algorithm and the algorithm
	  }
	}
      } else if (taskName.equals("getIndependentSets") && 
		 (mTaskAlgorithmContainer.getNumAlgorithms(task) > 0)) {
	// 
	// go through dual algorithms, try to create instances of the 
	// classes; if successful, add them to the algorithm list
        mIndependentSetAlgorithms = new Vector();
	mIndependentSetAlgInstances = new Vector();
	algorithms = mTaskAlgorithmContainer.getAlgorithms(task);
	for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext();) {
	  algorithm = (Algorithm)pAlgorithms.next();
	  try {
	    algInstance = algorithm.getInstance();
	    mIndependentSetAlgorithms.add(algorithm);
	    mIndependentSetAlgInstances.add(algInstance);
	  }
	  catch (Exception e) {
	    // don't do anything, just don't add the algorithm and the algorithm
	  }
	}
      } else if (taskName.equals("getHyperplanes") && 
		 (mTaskAlgorithmContainer.getNumAlgorithms(task) > 0)) {
	// 
	// go through dual algorithms, try to create instances of the 
	// classes; if successful, add them to the algorithm list
        mHyperplaneAlgorithms = new Vector();
	mHyperplaneAlgInstances = new Vector();
	algorithms = mTaskAlgorithmContainer.getAlgorithms(task);
	for (pAlgorithms = algorithms.iterator(); pAlgorithms.hasNext();) {
	  algorithm = (Algorithm)pAlgorithms.next();
	  try {
	    algInstance = algorithm.getInstance();
	    mHyperplaneAlgorithms.add(algorithm);
	    mHyperplaneAlgInstances.add(algInstance);
	  }
	  catch (Exception e) {
	    // don't do anything, just don't add the algorithm and the algorithm
	  }
	}
      }
    }
  }

  //
  // private method selects the best algorithm & representation from
  // the passed vector and the mReps collection, sets mCurAlgorithm
  // and mCurRep, and returns the complexity estimate of the best 
  // combination
  // 
  private long selectAlgRep(String method, Vector algorithms)
  {
    ComplexityProvider bestAlgorithm = null;
    Matroid bestRep = (Matroid)null;
    Object rep;
    ComplexityProvider algorithm;
    long bestEstimate = -1;
    long estimate;
    Iterator pAlg;
    Iterator pRep;

    if ((mReps.size() == 0) || (algorithms.size() == 0)) {
      bestAlgorithm = null;
      bestRep = null;
    }
    else if ((algorithms.size() == 1) && (mReps.size() == 1)) {
      // only one combination
      bestAlgorithm = (ComplexityProvider)algorithms.iterator().next();
      bestRep = (Matroid)mReps.iterator().next();
      bestEstimate = bestAlgorithm.getComplexityEstimate(method, bestRep);
    }
    else if (algorithms.size() == 1) {
      // only one algorithm, but several representations
      bestAlgorithm = (ComplexityProvider)algorithms.iterator().next();
      for (pRep = mReps.iterator(); pRep.hasNext();) {
	rep = pRep.next();
	estimate = bestAlgorithm.getComplexityEstimate(method, rep);
	if (((bestEstimate == -1) && (estimate >= 0)) ||
	    ((estimate >= 0) && (estimate < bestEstimate))) {
	  bestEstimate = estimate;
	  bestRep = (Matroid)rep;
	}
      }
    }
    else if (mReps.size() == 1) {
      // only one representation, but several algorithms
      bestRep = (Matroid)mReps.iterator().next();
      for (pAlg = algorithms.iterator(); pAlg.hasNext();) {
	algorithm = (ComplexityProvider)pAlg.next();
	estimate = algorithm.getComplexityEstimate(method, bestRep);
	if (((bestEstimate == -1) && (estimate >= 0)) ||
	    ((estimate >= 0) && (estimate < bestEstimate))) {
	  bestEstimate = estimate;
	  bestAlgorithm = algorithm;
	}
      }
    }
    else {
      // more than one representation and more than one algorithm
      for (pAlg = algorithms.iterator(); pAlg.hasNext();) {
	algorithm = (ComplexityProvider)pAlg.next();
	for (pRep = mReps.iterator(); pRep.hasNext();) {
	  rep = pRep.next();
	  estimate = algorithm.getComplexityEstimate(method, rep);
	  if (((bestEstimate == -1) && (estimate >= 0)) ||
	      ((estimate >= 0) && (estimate < bestEstimate))) {
	    bestEstimate = estimate;
	    bestAlgorithm = algorithm;
	    bestRep = (Matroid)rep;
	  }
	}
      }
    }

    if (bestEstimate >= 0) {
      mCurRep = bestRep;
      mCurAlgorithm = bestAlgorithm;
    }
    else {
      mCurRep = null;
      mCurAlgorithm = null;
    }

    return bestEstimate;
  }
}


