import Oid.*;
import java.util.*;

/** 
 * A Task instance describes one type of task.  It includes the task's name,
 * description, and a Class member which is the interface implemented by
 * algorithms which perform this task.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date   2 Oct 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class Task
{
  // private data members
  private String  mName;
  private String  mDescription;
  private Class   mCategory;
  private boolean mPrecedeWithSeparator;

  /** 
   * constructs a Task instance from a name, description, and the 
   * class name of the task's category
   *
   * @param name name of task
   * @param categoryClass name of task's category class (<code>Combiner</code>,
   * etc.)
   * @param description description of task
   */
  public Task(String name, String categoryClassName, String description)
    throws ClassNotFoundException
  {
    mCategory = Class.forName(categoryClassName);
    if (!mCategory.isInterface()) {
      throw new ClassNotFoundException("Task category must be an interface");
    }
    mName = new String(name);
    mDescription = new String(description);
  }

  /**
   * constructs a Task instance from a name, description and a reference
   * to the class name of the task's category
   * 
   * @param name name of task
   * @param categoryClass name of task's category class (<code>Combiner</code>,
   * etc.)
   * @param description description of task
   */
  public Task(String name, Class categoryClass, String description)
    throws ClassNotFoundException
  {
    mCategory = categoryClass;
    if (!mCategory.isInterface()) {
      throw new ClassNotFoundException("Task category must be an interface");
    }
    mName = new String(name);
    mDescription = new String(description);
  }

  /** 
   * indicates to the Task whether it should be preceded by a separator when
   * displayed on a menu
   *
   * @param b boolean value
   */
  public void setPrecedeWithSeparator(boolean b)
  {
    mPrecedeWithSeparator = b;
  }

  /**
   * returns a value indicating whether this Task should be preceded by a 
   * separator when displayed on a menu
   *
   * @return boolean value
   */
  public boolean getPrecedeWithSeparator()
  {
    return mPrecedeWithSeparator;
  }

  /** 
   * returns the task's name 
   * 
   * @return name of task
   */
  public String getName()
  {
    return mName;
  }

  /** 
   * overloaded method returns task's name 
   *
   * @param name of task
   */
  public String toString()
  {
    return mName;
  }

  /** 
   * overloaded equals method determines if another Task has the same
   * attributes
   *
   * @param obj another object to compare
   * @return <code>true</code> if and only if <code>obj</code> is an
   * instance of <code>Task</code> with the same attributes as the current
   * instance
   */
  public boolean equals(Object obj)
  {
    boolean answer=false;
    Task t;
    if (obj instanceof Task) {
      t = (Task)obj;
      answer = (mName.equals(t.getName()) && 
		mCategory.getName().equals(t.getCategoryClass().getName()));
    }
    return answer;
  }

  /** 
   * returns the task's description 
   * 
   * @return description of task
   */
  public String getDescription()
  {
    return mDescription;
  }

  /** 
   * returns the task's category class 
   *
   * @return this task's category class (<code>SubsetLister</code>, 
   * <code>Combiner</code>, etc.)
   */
  public Class getCategoryClass()
  {
    return mCategory;
  }
}
