import Oid.*;
import java.util.*;
import java.io.*;

/**
 * An instance of <code>TaskAlgorithmContainer</code> maintains a
 * collection of classes (<code>Algorithm</code>s) that perform
 * <code>Tasks</code>, that in turn implement interfaces known as task
 * categories. For example, for <code>Matroids</code>, one algorithm
 * class is <code>DefaultCircuitAlgorithm</code>, which performs the
 * task of generating circuits, and implements the
 * <code>SubsetLister</code> interface to do so.
 *
 * Distinguishing <code>Tasks</code> and <code>Algorithms</code>
 * allows the user interface to provide the user with a list of things
 * the program can do (<code>Tasks</code>), each of which is linked to
 * a list of ways in which the program can do it
 * (<code>Algorithms</code>).
 * 
 * Maintaining a separation (through task category interfaces) between
 * <code>Algorithm</code> classes and the rest of the program allows
 * new algorithms to be added without changing the rest of the
 * program.
 *
 * A <code>TaskAlgorithmContainer</code> may be constructed, or
 * augmented, by the entries in an "algorithmFile". An algorithmFile
 * should contain lines in two formats:
 * 
 * task <taskname> <category> <description>
 *               - and -
 * algorithm <classname> <taskname> <for-classname> [<description> | default]
 * 
 * where:
 *   <taskname>      is a single-word description of a task (e.g. getFlats)
 *   <category>      is a known task category (specifies an interface, e.g.
 *                   SubsetLister)
 *   <classname>     is the name of one class that performs a task (e.g.
 *                   DefaultFlatCalculator)
 *   <for-classname> (if present) is the implementation that this class
 *                   requires in order to work (e.g. Matroid, GFpMatroid)
 *   <description>   is a brief description of a task or algorithm; if the 
 *                   description of a algorithm is one word, "default" then the
 *                   algorithm is taken to be the default algorithm for the 
 *                   task.
 *
 * If the <code>TaskAlgorithmContainer</code> instance is unable to
 * load and parse the algorithmFile successfully, an
 * <code>IOException</code> is thrown.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 5 Oct 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class TaskAlgorithmContainer
{
  // private data members
  private Vector  mTasks;
  private Vector  mAlgorithms;
  private boolean mSeparatorFlag;

  /** 
   * constructs a TaskAlgorithmContainer instance with no 
   * <code>Tasks</code> or <code>Algorithms</code>
   */
  public TaskAlgorithmContainer()
  {
    mTasks = new Vector();
    mAlgorithms = new Vector();
  }

  /** 
   * constructs a TaskAlgorithmContainer instance by reading a 
   * "algorithmFile" with the specified name.
   * 
   * @param algorithmFileName name of file to read
   * @throws IOException if file cannot be read
   * @throws ClassNotFoundException if class name in file is invalid
   */
  public TaskAlgorithmContainer(String algorithmFileName) 
    throws IOException, ClassNotFoundException
  {
    TextFileParser parser = new TextFileParser(algorithmFileName);
    Vector algorithmFileLine;
    mTasks = new Vector();
    mAlgorithms = new Vector();
    mSeparatorFlag = false;
    
    for (Iterator p = parser.getContents().iterator(); p.hasNext();) {
      algorithmFileLine = (Vector)p.next();
      if (algorithmFileLine.size() > 0) {
	processAlgorithmFileLine(algorithmFileLine);
      }
    }
  }

  /**
   * constructs a TaskAlgorithmContainer instance using the passed 
   * collections of algorithms and tasks
   *
   * @param tasks tasks for container
   * @param algorithms algorithms for container
   * @throws IllegalArgumentExeption thrown if a member of tasks or 
   *   algorithms is not of the appropriate class
   */
  public TaskAlgorithmContainer(Collection tasks, Collection algorithms)
    throws IllegalArgumentException
  {
    Object obj;
    Iterator p;

    mAlgorithms = new Vector();
    mTasks = new Vector();
    for (p=tasks.iterator(); p.hasNext();) {
      obj = p.next();
      if (!(obj instanceof Task)) {
	throw new IllegalArgumentException("non-task object passed");
      }
      addTask((Task)obj);
    }

    for (p=algorithms.iterator(); p.hasNext();) {
      obj = p.next();
      if (!(obj instanceof Algorithm)) {
	throw new IllegalArgumentException("non-algorithm object passed");
      }
      addAlgorithm((Algorithm)obj);
    }
  }

  /**
   * constructs a TaskAlgorithmContainer by copying another 
   * TaskAlgorithmContainer
   *
   * @param t another instance
   */
  public TaskAlgorithmContainer(TaskAlgorithmContainer t)
  {
    Iterator p;

    mTasks = new Vector(t.getTasks());
    mAlgorithms = new Vector();
    

    for (p=mTasks.iterator(); p.hasNext();) {
      mAlgorithms.add(new Vector(t.getAlgorithms((Task)p.next())));
    }
  }

  /**
   * returns a TaskAlgorithmContainer containing references to all Tasks in 
   * this TaskAlgorithmContainer, but only those algorithms that apply to 
   * the passed object class
   *
   * @param objClass class to filter by
   * @return reference to new <code>TaskAlgorithmContainer</code> instance
   */
  public TaskAlgorithmContainer appliesTo(Class objClass)
  {
    TaskAlgorithmContainer tb = new TaskAlgorithmContainer();
    Vector algorithmVector;
    Algorithm algorithm;

    tb.addAllTasks(mTasks);
    
    Iterator pAlgorithmVec, pAlgorithm;
    for (pAlgorithmVec=mAlgorithms.iterator(); pAlgorithmVec.hasNext();) {
      algorithmVector = (Vector)pAlgorithmVec.next();
      for (pAlgorithm = algorithmVector.iterator(); pAlgorithm.hasNext();) {
	algorithm = (Algorithm)pAlgorithm.next();
	if (algorithm.doesAlgorithmApply(objClass)) {
	  tb.addAlgorithm(algorithm);
	}
      }
    }

    return tb;
  }

  /**
   * returns a TaskAlgorithmContainer containing references to all Tasks in 
   * this TaskAlgorithmContainer, but only those algorithms that apply to 
   * one of the classes in the passed Collection
   *
   * @param classes collection of classes to filter by
   */
  public TaskAlgorithmContainer appliesTo(Collection classes)
  {
    Class objClass;
    TaskAlgorithmContainer tb = new TaskAlgorithmContainer();
    Vector algorithmVector;
    Algorithm algorithm;
    boolean flag;

    tb.addAllTasks(mTasks);
    
    Iterator pAlgorithmVec, pAlgorithm, pClass;
    for (pAlgorithmVec=mAlgorithms.iterator(); pAlgorithmVec.hasNext();) {
      algorithmVector = (Vector)pAlgorithmVec.next();
      for (pAlgorithm = algorithmVector.iterator(); pAlgorithm.hasNext();) {
	algorithm = (Algorithm)pAlgorithm.next();
	flag = false;
	for (pClass = classes.iterator(); pClass.hasNext() && !flag;) {
	  objClass = (Class)pClass.next();
	  if (algorithm.doesAlgorithmApply(objClass)) {
	    tb.addAlgorithm(algorithm);
	    flag=true;
	  }
	}
      }
    }
    return tb;
  }

  /**
   * loads the specified algorithmFile and adds its algorithms and tasks to 
   * this TaskAlgorithmContainer
   * 
   * @param algorithmFileName name of file to read
   * @throws IOException if file cannot be read
   * @throws ClassNotFoundException if file contains invalid class name
   */
  public void loadAlgorithmFile(String algorithmFileName)
    throws IOException, ClassNotFoundException
  {
    TextFileParser parser = new TextFileParser(algorithmFileName);
    Vector algorithmFileLine;
    
    for (Iterator p = parser.getContents().iterator(); p.hasNext();) {
      algorithmFileLine = (Vector)p.next();
      if (algorithmFileLine.size() > 0) {
	processAlgorithmFileLine(algorithmFileLine);
      }
    }
  }

  /**
   * saves this TaskAlgorithmContainer's algorithms and tasks into a valid 
   * algorithmFile with the specified name
   *
   * @param algorithmFileName name of file to write
   * @throws IOException if file cannot be written to
   */
  public void saveAlgorithmFile(String algorithmFileName)
    throws IOException
  {
    FileWriter writer = new FileWriter(algorithmFileName);
    Task task;
    Vector algorithms;
    Algorithm algorithm;
    int i, j;

    for(i=0; i<mTasks.size(); i++) {
      writer.write(new String("task ").toCharArray());
      task = (Task)mTasks.get(i);
      writer.write(task.getName().toCharArray());
      writer.write(' ');
      writer.write(task.getCategoryClass().toString().toCharArray());
      writer.write(' ');
      writer.write(task.getDescription().toCharArray());
      writer.write(Character.LINE_SEPARATOR);
      algorithms = (Vector)mAlgorithms.get(i);
      for (j=0; j<algorithms.size(); j++) {
	writer.write(new String("algorithm ").toCharArray());
	algorithm = (Algorithm)algorithms.get(j);
	writer.write(algorithm.getClassName().toCharArray());
	writer.write(' ');
	writer.write(task.getName().toCharArray());
	writer.write(' ');
	writer.write(algorithm.getAppliesToName().toCharArray());
	writer.write(' ');
	writer.write(algorithm.getDescription().toCharArray());
	writer.write(Character.LINE_SEPARATOR);
      }
    }
    writer.flush();
    writer.close();
  }

  /** 
   * adds a task to the TaskAlgorithmContainer's collection at the specified 
   * index
   * 
   * @param index location for task
   * @param newTask <code>Task</code> to add
   */
  public void addTask(int index, Task newTask)
    throws ArrayIndexOutOfBoundsException
  {
    if (!mTasks.contains(newTask)) {
      mTasks.add(index, newTask);
      mAlgorithms.add(index, new Vector());
    }
  }

  /** 
   * adds a task to the end of the TaskAlgorithmContainer's collection
   * 
   * @param newTask <code>Task</code> to add
   */
  public void addTask(Task newTask)
  {
    if (!mTasks.contains(newTask)) {
      mTasks.add(newTask);
      mAlgorithms.add(new Vector());
    }
  }

  /** 
   * adds a Collection of Tasks to the end of the TaskAlgorithmContainer's 
   * collection
   *
   * @param newTasks <code>Tasks</code> to add
   */
  public void addAllTasks(Collection newTasks)
  {
    Vector localNewTasks = new Vector(newTasks);
    Task newTask;

    for (Iterator p = localNewTasks.iterator(); 
	 p.hasNext();) {
      newTask = (Task)p.next();
      if (!mTasks.contains(newTask)) {
	mTasks.add(newTask);
	mAlgorithms.add(new Vector());
      }
    }
  }

  /**
   * inserts a Collection of Tasks at the specified position within the
   * TaskAlgorithmContainer's collection
   * 
   * @param index starting location for new entries
   * @param newTasks <code>Tasks</code> to add
   */
  public void addAllTasks(int index, Collection newTasks)
    throws ArrayIndexOutOfBoundsException
  {
    mTasks.addAll(index, newTasks);
    for (int i=0; i<newTasks.size(); i++) {
      mAlgorithms.add(index, new Vector());
    }
  }

  /**
   * clears all Tasks from this TaskAlgorithmContainer
   *
   * @throws IllegalStateException thrown if any Tasks have Algorithms in 
   *   TaskAlgorithmContainer
   */
  public void clearTasks()
    throws IllegalStateException
  {
    boolean bHasAlgorithms = false;
    for (Iterator pAlgorithms = mAlgorithms.iterator(); 
	 pAlgorithms.hasNext() && !bHasAlgorithms;) {
      bHasAlgorithms = (((Vector)pAlgorithms.next()).size() > 0);
    }

    if (bHasAlgorithms) {
      throw new IllegalStateException("TaskAlgorithmContainer has algorithms");
    }
   
    mAlgorithms.clear();
    mTasks.clear();
  }

  /**
   * indicates whether this TaskAlgorithmContainer's Task collection contains 
   * the passed Task
   *
   * @param t task to search for
   * @param boolean result
   */
  public boolean containsTask(Task t)
  {
    return mTasks.contains(t);
  }

  /**
   * returns an array containing all of the Tasks for this 
   * TaskAlgorithmContainer
   * 
   * @return array of <code>Task</code> references
   */
  public Task[] getTaskArray()
  {
    return (Task[])mTasks.toArray();
  }

  /**
   * returns this TaskAlgorithmContainer's collection of tasks
   *
   * @return <code>Collection</code> of <code>Tasks</code>
   */
  public Collection getTasks()
  {
    return mTasks;
  }

  /**
   * returns the task from this TaskAlgorithmContainer's collection at the 
   * specified index
   *
   * @param index location of task
   * @return <code>Task</code> reference
   * @throws ArrayIndexOutOfBoundsException if <code>index</code> is invalid
   */
  public Task getTask(int index)
  throws ArrayIndexOutOfBoundsException
  {
    return (Task)mTasks.get(index);
  }

  /**
   * returns the index of the specified Task in this TaskAlgorithmContainer's 
   * collection, or -1 if it is not contained in the collection
   *
   * @param t task to search for
   * @return index of <code>t</code> 
   */
  public int indexOfTask(Task t)
  {
    return mTasks.indexOf(t);
  }

  /**
   * removes the specified task from this collection, if it occurs
   *
   * @param t task to remove
   */
  public void removeTask(Task t)
  {
    int i = mTasks.indexOf(t);
    if (i >= 0) {
      try {
	mAlgorithms.remove(i);
	mTasks.remove(i);
      }
      catch (ArrayIndexOutOfBoundsException e) {
	throw new InternalError("internal error in TaskAlgorithmContainer");
      }
    }
  }

  /** 
   * removes the Task at the specified index from this collection
   *
   * @param index location of task to remove
   */
  public void removeTask(int index)
    throws ArrayIndexOutOfBoundsException
  {
    mAlgorithms.remove(index);
    mTasks.remove(index);
  }

  /**
   * returns the number of Tasks in this TaskAlgorithmContainer's collection
   *
   * @return number of tasks
   */
  public int getNumTasks()
  {
    return mTasks.size();
  }

  /**
   * adds a Algorithm to this TaskAlgorithmContainer; if the Algorithm's 
   * specified Task is not already in the TaskAlgorithmContainer's 
   * collection, it is appended to the list
   *
   * @param t algorithm to add
   */
  public void addAlgorithm(Algorithm t)
  {
    Task algorithmTask = t.getTask();
    int i = mTasks.indexOf(algorithmTask);

    if (i < 0) {
      addTask(algorithmTask);
      i = mTasks.indexOf(algorithmTask);
    }

    try {
      ((Vector)mAlgorithms.get(i)).add(t);
    }
    catch (ArrayIndexOutOfBoundsException e) {
      throw new InternalError("internal error in TaskAlgorithmContainer");
    }
  }

  /**
   * adds all of the Algorithms in the passed Collection to this 
   * TaskAlgorithmContainer; if any of their specified Tasks are not already 
   * in the TaskAlgorithmContainer's collection, they are appended to the list
   *
   * @param algorithms <code>Collection</code> of algorithms to add
   */
  public void addAllAlgorithms(Collection algorithms)
  {
    for (Iterator p = algorithms.iterator(); p.hasNext();) {
      addAlgorithm((Algorithm)p.next());
    }
  }

  /** 
   * removes all Algorithms from this TaskAlgorithmContainer
   */
  public void clearAlgorithms()
  {
    for (Iterator p = mAlgorithms.iterator(); p.hasNext();) {
      ((Vector)p.next()).clear();
    }
  }

  /**
   * removes all Algorithms from this TaskAlgorithmContainer which are 
   * associated with the specified Task
   * 
   * @param t task whose algorithms to remove
   * @throws ArrayIndexOutOfBoundsException thrown if the Task is not found
   */
  public void clearAlgorithmsFromTask(Task t)
    throws ArrayIndexOutOfBoundsException
  {
    int i = mTasks.indexOf(t);
    ((Vector)mAlgorithms.get(i)).clear();
  }

  /** 
   * removes all Algorithms from this TaskAlgorithmContainer which are 
   * associated with the Task at the specified index
   *
   * @param index task index
   * @throws ArrayIndexOutOfBoundsException
   */
  public void clearAlgorithmsFromTask(int index)
    throws ArrayIndexOutOfBoundsException
  {
    ((Vector)mAlgorithms.get(index)).clear();
  }

  /**
   * indicates whether this TaskAlgorithmContainer contains the specified 
   * Algorithm
   *
   * @param t algorithm to look for
   * @return <code>true</code> if and only if the container has the algorithm
   */
  public boolean containsAlgorithm(Algorithm t)
  {
    boolean answer=false;
    Task task = t.getTask();
    int i = mTasks.indexOf(task);

    if (i>=0) {
      answer = ((Vector)mAlgorithms.get(i)).contains(t);
    }
    return answer;
  }

  /**
   * returns an array containing all the algorithms associated with the 
   * specified task
   *
   * @param t task to return algorithms for
   * @return array of algorithms
   */
  public Algorithm[] getAlgorithmArray(Task t)
    throws ArrayIndexOutOfBoundsException
  {
    int i = mTasks.indexOf(t);
    if (i<0) {
      throw new ArrayIndexOutOfBoundsException();
    }
    return (Algorithm[])((Vector)mAlgorithms.get(i)).toArray();
  }

  /**
   * returns an array containing all the algorithms associated with the task at
   * the specified index
   *
   * @param index task index for algorithms to return
   * @return array of algorithms
   */
  public Algorithm[] getAlgorithmArray(int index)
    throws ArrayIndexOutOfBoundsException
  {
    if ((index<0) || (index>=mTasks.size())) {
      throw new ArrayIndexOutOfBoundsException();
    }
    return (Algorithm[])((Vector)mAlgorithms.get(index)).toArray();
  }

  /**
   * returns a Collection of all the algorithms associated with the specified
   * task
   *
   * @param t task to get algorithms for
   * @return <code>Collection</code> of algorithms for <code>t</code>
   */
  public Collection getAlgorithms(Task t)
    throws ArrayIndexOutOfBoundsException
  {
    int i = mTasks.indexOf(t);
    if (i<0) {
      throw new ArrayIndexOutOfBoundsException();
    }
    return (Collection)mAlgorithms.get(i);
  }

  /**
   * returns an array containing all the algorithms associated with the task at
   * the specified index
   *
   * @param index task index for algorithms to return
   * @return <code>Collection</code> of algorithms for task 
   * at <code>index</code>
   */
  public Collection getAlgorithms(int index)
    throws ArrayIndexOutOfBoundsException
  {
    if ((index < 0) || (index >= mTasks.size())) {
      throw new ArrayIndexOutOfBoundsException();
    }
    return (Collection)mAlgorithms.get(index);
  }

  /** 
   * removes the specified Algorithm from this TaskAlgorithmContainer 
   * (if it exists)
   *
   * @param t algorithm to remove
   */
  public void removeAlgorithm(Algorithm t)
  {
    Task task = t.getTask();
    int i = mTasks.indexOf(task);
    int j;

    if (i >= 0) {
      try {
	((Vector)mAlgorithms.get(i)).remove(t);
      }
      catch (ArrayIndexOutOfBoundsException e) {
	throw new InternalError("internal error in TaskAlgorithmContainer");
      }
    }
  }

  /**
   * returns the number of Algorithms associated with the specified Task in 
   * this TaskAlgorithmContainer
   *
   * @param t task for algorithms to count
   * @return number of algorithms
   * @throws ArrayIndexOutOfBoundsException
   */
  public int getNumAlgorithms(Task t)
    throws ArrayIndexOutOfBoundsException
  {
    int i = mTasks.indexOf(t);

    if (i<0) {
      throw new ArrayIndexOutOfBoundsException();
    }

    return ((Vector)mAlgorithms.get(i)).size();
  }

  /**
   * returns the number of Algorithms associated with the Task at the specified
   * index in this TaskAlgorithmContainer
   *
   * @param index task index
   * @return number of algorithms for task
   * @throws ArrayIndexOutOfBoundsException
   */
  public int getNumAlgorithms(int index)
    throws ArrayIndexOutOfBoundsException
  {
    if ((index<0) || (index >= mTasks.size())) {
      throw new ArrayIndexOutOfBoundsException();
    }

    return ((Vector)mAlgorithms.get(index)).size();
  }

  /**
   * returns the total number of Algorithms in this TaskAlgorithmContainer
   *
   * @return number of algorithms
   */
  public int getNumAlgorithms()
  {
    int answer=0;
    for (int i = 0; i<mAlgorithms.size(); i++) {
      answer += ((Vector)mAlgorithms.get(i)).size();
    }
    return answer;
  }

  // processes a vector containing the strings for one line of a algorithmFile
  private void processAlgorithmFileLine(Vector tfLine)
    throws IllegalArgumentException, ClassNotFoundException
  {
    String lineType = tfLine.get(0).toString();
    Task lineTask;
    Algorithm lineAlgorithm;
    int i;

    if (lineType.equals("separator")) {
      mSeparatorFlag = true;
    }
    else if (lineType.equals("task")) {
      if (tfLine.size() < 4) {
	throw new IllegalArgumentException("too few items in task line");
      }
      lineTask = new Task(tfLine.get(1).toString(),
			  tfLine.get(2).toString(),
			  cat(tfLine, 3));
      lineTask.setPrecedeWithSeparator(mSeparatorFlag);
      addTask(lineTask);
      mSeparatorFlag = false;
    }
    else if (lineType.equals("algorithm")) {
      i = findTaskName(tfLine.get(2).toString());
      if (i >= 0) {
	lineAlgorithm = new Algorithm(tfLine.get(1).toString(),
			 (Task)mTasks.get(i),
			 tfLine.get(3).toString(),
			 cat(tfLine, 4));
	addAlgorithm(lineAlgorithm);
      }
      else {
	throw new ClassNotFoundException("line refers to unknown task: " +
					 tfLine.get(2).toString());
      }
      mSeparatorFlag = false;
    }
    else {
      throw new IllegalArgumentException("invalid line: " + lineType);
    }
  }

  // concatenates strings to make a description
  private String cat(Vector v, int offset)
  {
    if ((offset < 0) || (offset >= v.size())) {
      return new String("");
    }
    
    StringBuffer catBuf = new StringBuffer();
    for (int i=offset; i<v.size(); i++) {
      catBuf.append(v.get(i).toString());
      catBuf.append(' ');
    }
    return catBuf.toString();
  }

  // searches for a task with the specified name in the collection
  private int findTaskName(String taskName)
  {
    boolean found=false;
    int i;
    int j = -1;

    for (i = 0; !found && i<mTasks.size(); i++) {
      found = taskName.equals(mTasks.get(i).toString());
      if (found) {
	j = i;
      }
    }
    
    return j;
  }
}
