import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;

/** 
 * Determines the matrix of entries b_ij = #{bases of M containing
 * elements i and j}, noting any pairs with b * b_ij > b_ii * b_jj,
 * where b is the number of bases of M (violations of the negative
 * correlation condition).
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 2004-01-27
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class VisBasisSizeMatrix implements Visualization
{
  private static String mLF;

  private static String LF() 
  {
    if (!(mLF instanceof String)) {
      mLF = "\n";
    }
    return mLF;
  }

  /** 
   * generates a visualization of the passed object and returns it as
   * a JPanel
   */
  public JPanel getVisualization(Object obj)
    throws IllegalArgumentException {

    if (!(obj instanceof Matroid)) {
      throw new IllegalArgumentException("Requires a matroid");
    }

    Matroid M = (Matroid)obj, Mcef;
    int n=M.getSize(), i, j, e;
    
    int MBases, basesC2;
    int[][] B = new int[n][n];
    boolean[][] nc = new boolean[n][n];
    boolean negCorr = true;
    StringBuffer txt = new StringBuffer();
    DefaultBasisAlgorithm basAlg = new DefaultBasisAlgorithm();
    DecimalFormat fmt4 = new DecimalFormat("0000");
    Vector bases;
    Subset basis;

    // 
    // get basis count for M
    //
    basAlg.setInstance(M);
    bases = (Vector)basAlg.getSubsetList();
    MBases = bases.size();

    //
    // initialize matrices
    // 
    for (i=0; i<n; i++) {
      for (j=0; j<n; j++) {
	B[i][j] = 0;
	nc[i][j] = true;
      }
    }

    // count up B
    for (Iterator pBasis = bases.iterator(); pBasis.hasNext(); ) {
      basis = (Subset)pBasis.next();
      for (i=0; i<n; i++) {
	if (basis.get(i)) {
	  ++B[i][i];
	  for (j=i+1; j<n; j++) {
	    if (basis.get(j)) {
	      ++B[i][j];
	      ++B[j][i];
	    }
	  }
	}
      }
    }

    // check neg. corr.
    negCorr = true;
    for (i=0; i<n; i++) {
      for (j=i+1; j<n; j++) {
	nc[i][j] = (MBases * B[i][j] <= B[i][i] * B[j][j]);
	nc[j][i] = nc[i][j];
	negCorr = negCorr && nc[i][j];
      }
    }

    // creat output
    txt.append("B(M) = " + String.valueOf(MBases) + "\n");

    txt.append("     ");
    for (i=0; i<n; i++) {
      txt.append(spaceFmt(fmt4.format(i+1)));
      txt.append("  ");
    }
    txt.append("\n");
    for (i=0; i<n; i++) {
      txt.append(spaceFmt(fmt4.format(i+1)));
      txt.append(" ");
      for (j=0; j<n; j++) {
	txt.append(spaceFmt(fmt4.format(B[i][j])));
	if (!nc[i][j]) {
	  txt.append("*");
	}
	else {
	  txt.append(" ");
	}
	txt.append(" ");
      }
      txt.append("\n");
    }

    txt.append("\n");

    if (negCorr) {
      txt.append("\nThe matroid IS negatively correlated\n");
    }
    else {
      txt.append("\nThe matroid IS NOT negatively correlated\n");
    }

    

    ReadOnlyTextPanel panel = new ReadOnlyTextPanel();
    panel.setText(txt.toString());
    return panel;
  }

  private int matroidLabelIndex(Matroid M, String label) {
    int i, n=M.getSize(), answer=-1;
    for (i=0; i<n; i++) {
      if (M.getLabel(i).equals(label)) {
	answer = i;
	break;
      }
    }
    return answer;
  }

  private String spaceFmt(String s) {
    StringBuffer sb = new StringBuffer(s);
    int n = sb.length();
    for (int i=0; i<n-1; i++) {
      if (sb.charAt(i) == '0') {
	sb.setCharAt(i, ' ');
      }
      else if (sb.charAt(i) != ' ') {
	break;
      }
    }
    return sb.toString();
  }

  private String padLeft(String s, int len) {
    StringBuffer answer = new StringBuffer(s);
    while (answer.length() < len) {
      answer.insert(0, ' ');
    }
    return answer.toString();
  }
  
  /**
   * returns the visualization's name 
   */
  public String getName()
  {
    return new String("Negative correlation checker");
  }

  /**
   * returns a string describing the visualization
   */
  public String getDescription()
  {
    StringBuffer answer = new StringBuffer();
    answer.append("Checks whether a matroid is negatively correlated; ");
    answer.append("that is, B(M) <= B(M/e) * B(M/f) / B(M/e,f) for all ");
    answer.append("e,f in E(M), where B(M) is the number of bases in M.");
    return answer.toString();
  }
}
