import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * Takes an instance of <code>Matroid</code>, calculates its dual,
 * uses the matroid and its dual to generate its circuits and
 * cocircuits, and then generates all of the circuit-cocircuit
 * intersections. It returns a <code>ReadOnlyTextPanel</code> instance
 * with the sets listed in order by size.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 13 March 2003
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class VisCIC implements Visualization
{
  //
  // private data members
  //
  private Matroid mMatroid;
  private Matroid mDual;

  /** returns a ReadOnlyTextPanel with the matroid's id matrix */
  public JPanel getVisualization(Object obj)
    throws IllegalArgumentException
  {
    if (!(obj instanceof Matroid)) {
      throw new IllegalArgumentException("Requires a matroid");
    }
    if (((ComplexityProvider)obj).getComplexityEstimate("getDual") < 0) {
      throw new IllegalArgumentException("Cannot get dual");
    }
    mMatroid = (Matroid)obj;
    if (mMatroid instanceof FiniteFieldMatroid) {
      mDual = ((FiniteFieldMatroid)mMatroid).getDual();
    }
    else if (mMatroid instanceof BasisMatroid) {
      mDual = ((BasisMatroid)mMatroid).getDual();
    }


    // set up algorithm
    DefaultCircuitAlgorithm cctAlg = new DefaultCircuitAlgorithm();
    // set up string for results
    StringBuffer results = new StringBuffer();
    
    Vector ccts, coccts;

    // get circuits, cocircuits
    cctAlg.setInstance(mMatroid);
    ccts = (Vector)cctAlg.getSubsetList();

    cctAlg.setInstance(mDual);
    coccts = (Vector)cctAlg.getSubsetList();

    // generate intersections
    Subset cct, cocct, cint;
    Iterator pcct, pcocct, pint;
    int size, maxSize, n, r;
    int i;

    n = mMatroid.getSize();
    r = mMatroid.getRank();
    maxSize = (r+1 < n-r+1) ? r+1 : n-r+1;
    Vector[] ints = new Vector[maxSize+1];
    for (i=0; i<=maxSize; i++) {
      ints[i] = new Vector();
    }

    for (pcct = ccts.iterator(); pcct.hasNext(); ) {
      cct = (Subset) pcct.next();
      for (pcocct = coccts.iterator(); pcocct.hasNext(); ) {
	cocct = (Subset) pcocct.next();
	cint = new Subset(cct);
	cint.and(cocct);
	
	size = cint.numElements();
	if (!(ints[size].contains(cint))) {
	  ints[size].add(cint);
	}
      }
    }
    
    // put intersections in a string
    for (i=0; i<=maxSize; i++) {
      if (ints[i].size() > 0) {
	results.append("Size: " + String.valueOf(i) + "\n");
	for (pint = ints[i].iterator(); pint.hasNext(); ) {
	  results.append(((Subset)pint.next()).toString(true) + "\n");
	}
	results.append("\n");
      }
    }

    // output counts for each size
    results.append("Size Count\n");
    for (i=0; i<=maxSize; i++) {
      results.append(String.valueOf(i) + "   " + 
		     String.valueOf(ints[i].size()) + "\n");
    }


    ReadOnlyTextPanel answer = new ReadOnlyTextPanel();
    answer.setText(results.toString());
    return answer;
  }
  
  /** returns the name of this visualization */
  public String getName()
  { 
    return new String("Circuit-cocircuit intersections");
  }
  
  public String getDescription()
  { 
    return new String("Takes an instance of Matroid, calculates its dual, uses the matroid and its dual to generate its circuits and cocircuits, and then generates all of the circuit-cocircuit intersections. It returns a ReadOnlyTextPanel instance with the sets listed in order by size.");
  }
}
