import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** 
 * <code>VisFiniteFieldChainExtensions</code> generates a list of the
 * single-element extensions of a FiniteFieldMatroid and checks for
 * unique matroids using the FiniteFieldMatroidChainChecker class.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 6 Jun 2002
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class VisFiniteFieldChainExtensions implements Visualization
{
  //
  // private data members
  //
  private String mName;

  /**
   * default constructor
   */
  public VisFiniteFieldChainExtensions()
  {
    mName = new String("Finite Field Matroid Extension Generator");
  }

  /** 
   * generates a visualization of the passed object and returns it as
   * a JPanel -- in this case, the passed object must be an instance
   * of FiniteFieldMatroid, and the returned JPanel is a read-only text panel 
   * with a list of the single-element extensions of the matroid
   *
   * @param obj matroid to use
   */
  public JPanel getVisualization(Object obj)
    throws IllegalArgumentException
  {
    if (!(obj instanceof FiniteFieldMatroid)) {
      throw new IllegalArgumentException("Requires a GF(q) matroid");
    }

    String lf = "\n";
    FiniteFieldMatroid base = (FiniteFieldMatroid)obj;
    int n = base.getSize();
    int r = base.getRank();
    FiniteField calc = base.getField();
    FiniteFieldVector[] element = new FiniteFieldVector[n+1];
    int i,j;
    PGFactory factory = new PGFactory(r, calc);
    FiniteFieldVector[] projSpace = factory.getProjectiveSpace();
    int pn = factory.getProjectiveSpaceSize();
    int[] index = new int[pn];
    FiniteFieldMatroid extension;
    FiniteFieldMatroid extDual;
    FiniteFieldMatroid compMatroid;
    FiniteFieldMatroid dual;
    Vector compChains, compDegrees, compDChains, compDDegrees;
    Vector currChains, currDegrees, currDChains, currDDegrees;
    Vector uniqExtensions = new Vector();
    Vector extChains = new Vector();
    Vector extDegrees = new Vector();
    Vector extDChains = new Vector();
    Vector extDDegrees = new Vector();
    boolean found;
    boolean anyExtensions=false;
    FiniteFieldMatroidChainChecker checker = 
      new FiniteFieldMatroidChainChecker();
    checker.setMakeExplanation(false);
    mName = "GF(" + calc.getFieldSize() + ") Matroid Extension Generator";
    StringBuffer output = new StringBuffer(getName() + lf);

    // initialize indices to -1 (prevents matroid columns from being listed)
    for (i=0; i<pn; i++) {
      index[i] = -1;
    }

    // build base for extension matrix
    for (i=0; i<n; i++) {
      element[i] = base.getElement(i);
    }

    // loop through elements of the projective space
    for (i=0; i<pn; i++) {
      if (!matroidContains(base, projSpace[i])) {
	anyExtensions = true;

	// build extension matrix
	element[n] = projSpace[i];
	extension = new FiniteFieldMatroid(element);
	currChains = (Vector)null;
	currDegrees = (Vector)null;
	currDChains = (Vector)null;
	currDDegrees = (Vector)null;
	
	// check against previously discovered extensions
	found = false;
	for (j=0; j<uniqExtensions.size() && !found; j++) {
	  compMatroid = (FiniteFieldMatroid)uniqExtensions.get(j);
	  dual = compMatroid.getDual();
	  compChains = (Vector)extChains.get(j);
	  compDegrees = (Vector)extDegrees.get(j);
	  compDChains = (Vector)extDChains.get(j);
	  compDDegrees = (Vector)extDDegrees.get(j);
	  
	  // if we've already computed the chains & degrees & bases, use them
	  if (compChains instanceof Vector) {
	    checker.setFiniteFieldMatroid1(compMatroid,
					   dual,
					   compChains,
					   compDegrees,
					   compDChains,
					   compDDegrees);
	  }
	  else {
	    checker.setFiniteFieldMatroid1(compMatroid);
	  }
	  
	  // also use current extensions chains & degrees & bases if possible
	  if (currChains instanceof Vector) {
	    checker.setFiniteFieldMatroid2(extension,
					   extension.getDual(),
					   currChains,
					   currDegrees,
					   currDChains,
					   currDDegrees);
	  }
	  else {
	    checker.setFiniteFieldMatroid2(extension);
	  }

	  // check for isomorphism
	  if (checker.checkFiniteFieldMatroids()) {
	    found = true;
	    index[i] = j;
	  }

	  // if we haven't stored the vectors, get them now
	  if (!(compChains instanceof Vector)) {
	    extChains.set(j, checker.getChains1());
	    extDegrees.set(j, checker.getDegrees1());
	    extDChains.set(j, checker.getDualChains1());
	    extDDegrees.set(j, checker.getDualDegrees1());
	  }

	  // if we haven't stored the current extension's vectors get them now
	  if (!(currChains instanceof Vector)) {
	    currChains = checker.getChains2();
	    currDegrees = checker.getDegrees2();
	    currDChains = checker.getDualChains2();
	    currDDegrees = checker.getDualDegrees2();
	  }
	}

	if (!found) {
	  // make sure we have vectors for this matroid (checker can get them)
	  if (!(currChains instanceof Vector)) {
	    checker.setFiniteFieldMatroid1(extension);
	    currChains = checker.getChains1();
	    currDegrees = checker.getDegrees1();
	    currDChains = checker.getDualChains1();
	    currDDegrees = checker.getDualDegrees1();
	  }

	  // add new matroid to the list
	  uniqExtensions.add(extension);
	  extChains.add(currChains);
	  extDegrees.add(currDegrees);
	  extDChains.add(currDChains);
	  extDDegrees.add(currDDegrees);
	    index[i] = uniqExtensions.size()-1;
	}
      }
    }

    // make output string
    if (anyExtensions) {
      output.append
	(FiniteFieldMatroidFormatter.displayWithLabels(base));
      output.append(lf);

      for (i=0; i<uniqExtensions.size(); i++) {
	output.append("================================" + lf);
	output.append("Extension ");
	output.append(i+1);
	output.append(":" + lf);
	output.append
	  (FiniteFieldMatroidFormatter.displayWithLabels
	   ((FiniteFieldMatroid)uniqExtensions.get(i)));

	output.append(matroidDesc((FiniteFieldMatroid)uniqExtensions.get(i),
				  (Vector)extChains.get(i),
				  (Vector)extDChains.get(i)));
	output.append("Vectors:" + lf);
	for (j=0; j<pn; j++) {
	  if (index[j] == i) {
	    output.append(projSpace[j].toString());
	    output.append(lf);
	  }
	}
      }
    }
    else {
      output.append("No extensions found" + lf);
    }
    
    ReadOnlyTextPanel answer = new ReadOnlyTextPanel();
    answer.setText(output.toString());
    return answer;
  }
  
  /**
   * returns the visualization's name 
   */
  public String getName()
  {
    return new String(mName);
  }

  /**
   * returns a string describing the visualization
   */
  public String getDescription()
  {
    return new String("Generates single-element extensions of a GF(p) "+
		      "matroid and checks them using the finite field " + 
		      "matroid chain checker");
  }

  //
  // private method determines if a FiniteFieldMatroid instance contains a 
  // specified vector
  //
  private boolean matroidContains(FiniteFieldMatroid M, FiniteFieldVector v)
  {
    int i, n=M.getSize();
    boolean found=false;

    for (i=0; i<n; i++) {
      if (M.getElement(i).multiple(v) != 0) {
	found = true;
	break;
      }
    }
    return found;
  }

  //
  // private method outputs a matroid and the chains produced by the
  // FiniteFieldMatroidChainChecker class
  //
  private String matroidDesc(FiniteFieldMatroid matroid, 
			     Vector chains, 
			     Vector dChains)
  {
    String lf = "\n";
    int r=matroid.getRank(), n=matroid.getSize();
    int i, j, w = 5;
    Vector chain;
    
    StringBuffer answer = new StringBuffer();
    // header
    answer.append("          ");
    for (i=0; i<=n; i++) {
      answer.append(padLeft(String.valueOf(i), w));
    }
    answer.append(lf);
    j = n+2;
    for (i=0; i<j; i++) {
      answer.append("-----");
    }
    answer.append(lf);

    // independent sets
    answer.append("Ind Sets  ");
    chain = (Vector)chains.get(0);
    for (i=0; i<=r; i++) {
      answer.append(padLeft(chain.get(i).toString(), w));
    }
    answer.append(lf);

    // circuits
    answer.append("Circuits  ");
    chain = (Vector)chains.get(1);
    for (i=0; i<=r+1; i++) {
      answer.append(padLeft(chain.get(i).toString(), w));
    }
    answer.append(lf);

    // coindependent sets
    r = n - r;
    answer.append("coInd Sets");
    chain = (Vector)dChains.get(0);
    for (i=0; i<=r; i++) {
      answer.append(padLeft(chain.get(i).toString(), w));
    }
    answer.append(lf);

    // circuits
    answer.append("coCircuits");
    chain = (Vector)dChains.get(1);
    for (i=0; i<=r+1; i++) {
      answer.append(padLeft(chain.get(i).toString(), w));
    }
    answer.append(lf);

    return answer.toString();
  }

  // private method pads a string right-justified with spaces
  private String padLeft(String s, int w) 
  {
    String answer = new String(s);
    while (answer.length() < w) {
      answer = " " + answer;
    }
    return answer;
  }
}
