import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** 
 * <code>VisGF2IsoTutteCnt</code> generates a list of the
 * single-element contractions of a <code>FiniteFieldMatroid</code> and
 * identifies the matroids that are graphic, cographic, both or
 * neither.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 20 May 2003
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class VisGF2IsoTutteCnt implements Visualization
{
  private String LF = "\n";

  /** 
   * generates a visualization of the passed object and returns it as
   * a JPanel -- in this case, the passed object must be an instance
   * of FiniteFieldMatroid, and the returned JPanel is a read-only text panel 
   * with a list of the single-element extensions of the matroid
   */
  public JPanel getVisualization(Object obj)
    throws IllegalArgumentException
  {
    if (!(obj instanceof FiniteFieldMatroid)) {
      throw new IllegalArgumentException("Requires a GF(p) matroid");
    }

    FiniteFieldMatroid base = (FiniteFieldMatroid)obj;

    if (2 != base.getField().getFieldSize()) {
      throw new IllegalArgumentException("Requires a GF(2) matroid");
    }

    int n = base.getSize();
    int r = base.getRank();
    FiniteField calc = base.getField();
    int i,j,k;
    FiniteFieldMatroid contraction, cntDual;
    StringBuffer output = new StringBuffer(getName() + LF);
    SFDual sfDualGen = new SFDual(); // need dual in standard form

    // Tutte's algorithm stuff
    if (!base.inStandardForm()) {
      throw new IllegalArgumentException("Matroid must be in standard form");
    }

    int[][]matrix = new int[r][n-1];
    int[]colNum = new int[n-1];
    int[]rowNum = new int[r];

    int[][]smMatrix = new int[r-1][n-1];
    int[] smRowNum = new int[r-1];
    
    int r2 = n-1-r;
    int[][] dualMatrix = new int[r2][n-1];
    int[] dualRowNum = new int[r2];

    int[][] smDualMatrix = new int[r2+1][n-1];
    int[] smDualRowNum = new int[r2+1];

    FiniteFieldVector v;
    boolean isFirst;
    boolean isGraphic=false, isCoGraphic=false;

    // form as much of matrices as we can with matroid itself
    for (i=0; i<r; i++) {
      rowNum[i] = i+1;
    }

    for (i=0; i<r2; i++) {
      dualRowNum[i] = i+1;
    }

    for (i=0; i<n-1; i++) {
      colNum[i] = i+1;
    }

    for (i=0; i<r-1; i++) {
      smRowNum[i] = i+1;
    }

    for (i=0; i<=r2; i++) {
      smDualRowNum[i] = i+1;
    }


    // create matrix checker
    TutteMatrixChecker tmc = new TutteMatrixChecker();

    // loop through elements of the matroid
    for (i=0; i<n; i++) {
      contraction = base.getContraction(i);
      sfDualGen.setInstance(contraction);
      cntDual = (FiniteFieldMatroid)sfDualGen.getConstruction();

      // contraction is probably lower in rank
      if (contraction.getRank() < r) {
	// fill in matrices for Tutte checker with smaller rank
	for (j=0; j<r-1; j++) {
	  for (k=0; k<n-1; k++) {
	    smMatrix[j][k] = contraction.getElement(k).get(j);
	  }
	}

	for (j=0; j<n-r; j++) {
	  for (k=0; k<n-1; k++) {
	    smDualMatrix[j][k] = cntDual.getElement(k).get(j);
	  }
	}

	// check whether contraction is graphic
	tmc.setMatrix("M1", smMatrix, colNum, smRowNum);
	isGraphic = tmc.isGraphic();
      
	tmc.setMatrix("M1'", smDualMatrix, colNum, smDualRowNum);
	isCoGraphic = tmc.isGraphic();	
      }
      else {
	// fill in matrices for Tutte checker with full size
	for (j=0; j<r; j++) {
	  for (k=0; k<n-1; k++) {
	    matrix[j][k] = contraction.getElement(k).get(j);
	  }
	}

	for (j=0; j<n-1-r; j++) {
	  for (k=0; k<n-1; k++) {
	    dualMatrix[j][k] = cntDual.getElement(k).get(j);
	  }
	}

	// check whether contraction is graphic
	tmc.setMatrix("M1", matrix, colNum, rowNum);
	isGraphic = tmc.isGraphic();
      
	tmc.setMatrix("M1'", dualMatrix, colNum, dualRowNum);
	isCoGraphic = tmc.isGraphic();
      }

      output.append
	(FiniteFieldMatroidFormatter.displayWithLabels(contraction));
      output.append(((isGraphic) ? "Graphic " : "Not graphic "));
      output.append(((isCoGraphic) ? "CoGraphic" + LF : "Not cographic"+LF));
      output.append(LF + LF);
    }
    
    ReadOnlyTextPanel answer = new ReadOnlyTextPanel();
    answer.setText(output.toString());
    return answer;
  }
  
  /**
   * returns the visualization's name 
   */
  public String getName()
  {
    return new String("GF(2) matroid contraction generator with graphic check");
  }

  /**
   * returns a string describing the visualization
   */
  public String getDescription()
  {
    return new String("Generates single-element extensions of a GF(p) matroid and checks them using the matroid isomorphism checker");
  }

  //
  // private method determines if a FiniteFieldMatroid instance contains a 
  // specified vector
  //
  private boolean matroidContains(FiniteFieldMatroid M, FiniteFieldVector v)
  {
    int i, n=M.getSize();
    boolean found=false;

    for (i=0; i<n; i++) {
      if (M.getElement(i).multiple(v) != 0) {
	found = true;
	break;
      }
    }
    return found;
  }

  //
  // private method outputs a matroid and the chains produced by the
  // MatroidIsomorphismChecker class
  //
  private String matroidDesc(FiniteFieldMatroid matroid, Vector chains)
  {
    int r=matroid.getRank(), n=matroid.getSize();
    int i, j, w = 5;
    Vector chain;
    
    StringBuffer answer = new StringBuffer();
    // header
    answer.append("          ");
    for (i=0; i<=n; i++) {
      answer.append(TextAlignment.rightAlign(String.valueOf(i), w));
    }
    answer.append(LF);
    j = n+2;
    for (i=0; i<j; i++) {
      answer.append("-----");
    }
    answer.append(LF);

    // independent sets
    answer.append("Ind Sets  ");
    chain = (Vector)chains.get(0);
    for (i=0; i<=r; i++) {
      answer.append(TextAlignment.rightAlign(chain.get(i).toString(), w));
    }
    answer.append(LF);

    // circuits
    answer.append("Circuits  ");
    chain = (Vector)chains.get(1);
    for (i=0; i<=r+1; i++) {
      answer.append(TextAlignment.rightAlign(chain.get(i).toString(), w));
    }
    answer.append(LF);

    // spanning sets
    answer.append("Span sets ");
    chain = (Vector)chains.get(3);
    for (i=0; i<n; i++) {
      answer.append(TextAlignment.rightAlign(chain.get(i).toString(), w));
    }
    answer.append(LF);

    return answer.toString();
  }
}
