import Oid.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * <code>VisTuttesAlgorithm</code> implements the
 * <code>Visualization</code> interface. Given a binary matroid (a
 * <code>FiniteFieldMatroid</code> instance over GF(2)),
 * <code>VisTuttesAlgorithm</code> extracts the {0,1}-matrix from the
 * matroid, creates an instance of <code>TutteMatrixChecker</code>,
 * and uses it to determine whether the matroid is graphic.  It
 * creates a <code>ReadOnlyTextPanel</code> instance (a subclass of
 * <code>JPanel</code>), places the explanation provided by the
 * <code>TutteMatrixChecker</code> on the panel, and returns it.
 *
 * @author Robert J. Kingan and Sandra R. Kingan
 * @date 20 Nov 2001
 * @version 3.0
 *
 * Copyright 2004 Robert J. Kingan and Sandra R. Kingan
 * 
 * This file is part of Oid.
 *
 * Oid is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Oid is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Oid; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
public class VisTuttesAlgorithm implements Visualization
{

  /** 
   * generates a visualization of the passed object and returns it as
   * a JPanel
   */
  public JPanel getVisualization(Object obj)
    throws IllegalArgumentException
  {
    if (!(obj instanceof FiniteFieldMatroid)) {
      throw new IllegalArgumentException("FiniteFieldMatroid required by "
					 + "Tutte's Algorithm");
    }
    FiniteFieldMatroid matroid = (FiniteFieldMatroid)obj;
    if (matroid.getField().getFieldSize() != 2) {
      throw new IllegalArgumentException("Binary matroid required by " 
					 + "Tutte's Algorithm");
    }

    if (!matroid.inStandardForm()) {
      throw new IllegalArgumentException("Matroid must be in standard form");
    }

    int r = matroid.getRank();
    int n = matroid.getSize();
    int[][]matrix = new int[r][n];
    int[]colNum = new int[n];
    int[]rowNum = new int[r];
    int i,j;
    FiniteFieldVector v;

    for (i=0; i<r; i++) {
      rowNum[i] = i+1;
    }

    for (i=0; i<n; i++) {
      v = matroid.getElement(i);
      colNum[i] = i+1;
      for (j=0; j<r; j++) {
	matrix[j][i] = v.get(j);
      }
    }

    // create matrix checker, check matrix
    TutteMatrixChecker tmc = new TutteMatrixChecker();
    tmc.setMatrix("M1", matrix, colNum, rowNum);
    
    // create ReadOnlyTextPanel, add explanation text to it, return it
    ReadOnlyTextPanel answer = new ReadOnlyTextPanel();
    answer.setText(tmc.getExplanation());
    return answer;
  }

  /**
   * returns the visualization's name 
   */
  public String getName()
  {
    return new String("Tutte's Algorithm");
  }

  /**
   * returns a string describing the visualization
   */
  public String getDescription()
  {
    return new String(" VisTuttesAlgorithm implements the Visualization interface. Given a binary matroid (a FiniteFieldMatroid instance whose modulus is 2), VisTuttesAlgorithm extracts the {0,1}-matrix from the matroid, creates an instance of  TutteMatrixChecker, and uses it to determine whether the matroid is graphic. It creates a ReadOnlyTextPanel instance (a subclass of JPanel),  places the explanation provided by the TutteMatrixChecker on the panel, and returns it.");
  }
}
